<?php
/**
 * RSVP Pro component for Joomla 2.5+
 *
 * @version     $Id: jevboolean.php 1331 2010-10-19 12:35:49Z geraintedwards $
 * @package     JEvents
 * @copyright   Copyright (C) 2008-2021 GWE Systems Ltd
 * @license     GNU/GPLv2, see http://www.gnu.org/licenses/gpl-2.0.html
 * @link        https://www.jevents.net
 *
 */
if (!defined('_JEXEC'))
	die('Direct Access  is not allowed.');

jimport('joomla.plugin.plugin');
jimport('joomla.log.log');

// Load Omnipay autoloader
require JPATH_SITE . '/libraries/jev_omnipay/vendor/autoload.php';

// Use Omnipay
use Joomla\Registry\Registry;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Http\HttpFactory;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Log\Log;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\Factory;
use Joomla\CMS\HTML\HTMLHelper;
use Omnipay\Omnipay;
use Omnipay\Common\CreditCard;

require 'authorizenet/vendor/autoload.php';
//require_once 'constants/SampleCodeConstants.php';
use net\authorize\api\contract\v1 as AnetAPI;
use net\authorize\api\controller as AnetController;

define("AUTHORIZENET_LOG_FILE", "phplog");


class plgRsvpproAuthorizenet extends CMSPlugin
{

	function __construct(& $subject, $config)
	{
		// Enable Logging
		Log::addLogger(
			array(
				'text_file' => 'com_rsvppro.authorizenet.log.php'
			),
			Log::ALL,
			array('com_rsvppro.authorizenet')
		);

		parent::__construct($subject, $config);

	}

	public function generatePaymentPage(&$html, $attendee, $rsvpdata, $event, &$transaction)
	{

		$plugin = PluginHelper::getPlugin("rsvppro", "authorizenet");
		$params = new Registry($plugin->params);
		$Itemid = Factory::getApplication()->input->getInt("Itemid");

		// TODO transaction based invoiceid
		$invoice = Factory::getApplication()->input->getString("invoiceid", "") . '_' . $transaction->transaction_id;

		$Itemid = Factory::getApplication()->input->getInt("Itemid");

		$html = $params->get("aztemplate", "Invoiceid = {INVOICEID}<br/>Total Fees = {TOTALFEES}<br/>Fees Already Paid= {FEESPAID}<br/>Outstanding Balance = {BALANCE}<br/><br/>Please proceed to Authorize.net ...<br/><br/>{FORM}");

		if (isset($rsvpdata->template) && is_numeric($rsvpdata->template))
		{
			$db = Factory::getDBO();
			$db->setQuery("Select params from #__jev_rsvp_templates where id=" . intval($rsvpdata->template));
			$templateParams = $db->loadObject();
			if ($templateParams)
			{
				$templateParams = json_decode($templateParams->params);
				$templateParams = RsvpHelper::translateTemplateParams($rsvpdata->template, $templateParams);
			}
			else
			{
				$templateParams = $params;
			}
		}
		else
		{
			$templateParams = $params;
		}

		if (isset($templateParams->aztemplate))
		{
			$html = $templateParams->aztemplate;
		}
		if ($html == "")
		{
			$html = "Invoiceid = {INVOICEID}<br/>Total Fees = {TOTALFEES}<br/>Fees Already Paid= {FEESPAID}<br/>Outstanding Balance = {BALANCE}<br/><br/>Please proceed to Authorize.net ...<br/><br/>{FORM}";
		}

		if (isset($attendee->outstandingBalances))
		{
			$html = str_replace("{INVOICEID}", $invoice, $html);
			$html = str_replace("{TOTALFEES}", $this->phpMoneyFormat($attendee->outstandingBalances['totalfee'], $templateParams), $html);
			$html = str_replace("{FEESPAID}", $this->phpMoneyFormat($attendee->outstandingBalances['feepaid'], $templateParams), $html);
			$html = str_replace("{BALANCE}", $this->phpMoneyFormat($attendee->outstandingBalances['feebalance'], $templateParams), $html);
			$html = str_replace("{RAWBALANCE}", $attendee->outstandingBalances['feebalance'], $html);
			$html = str_replace("{DEPOSIT}", $this->phpMoneyFormat($attendee->outstandingBalances['deposit'], $templateParams), $html);
		}

		$comparams = ComponentHelper::getParams("com_rsvppro");
		$user      = JEVHelper::getUser($attendee->user_id);
		if ($user->id == 0 && RsvpHelper::getTemplateSpecificParameter($rsvpdata,  "attendemails", 0))
		{
			$name     = $attendee->email_address;
			$username = $attendee->email_address;
		}
		else
		{
			$name     = $user->name;
			$username = $user->username;
		}

		include_once(JPATH_ADMINISTRATOR . "/components/com_rsvppro/libraries/attendeehelper.php");
		$this->helper = new RsvpAttendeeHelper($comparams);

		$html = $this->helper->parseMessage($html, $rsvpdata, $event, $name, $username, $attendee, false, false, $transaction);



		$method = $params->get("AZNMechanism","acceptjs");

		if ($method == "accepthosted")
		{
			$amount     = RsvpHelper::ceil_dec(Factory::getApplication()->input->getFloat("amount", 0), 2, ".");

			$currency   = $templateParams->Currency;

			//echo "amount = $amount<br>";
			//echo "currency = $currency<br>";
			//echo "invoice = $invoice<br>";

			/* Create a merchantAuthenticationType object with authentication details */
			$merchantAuthentication = new AnetAPI\MerchantAuthenticationType();
			$merchantAuthentication->setName($params->get("login_id"));
			$merchantAuthentication->setTransactionKey($params->get("transaction_key"));

			//echo "login_id = " . $params->get("login_id") . "<br>";
			//echo "transaction_key = " . $params->get("transaction_key") . "<br>";

			// see https://developer.authorize.net/hello_world/testing_guide.html
			// See https://developer.authorize.net/api/reference/index.html

			//create a transaction
			$transactionRequestType = new AnetAPI\TransactionRequestType();
			$transactionRequestType->setTransactionType("authCaptureTransaction");
			$transactionRequestType->setAmount($amount);
			$transactionRequestType->setCurrencyCode($currency);


			// Set the customer's identifying information
			$customerData = new AnetAPI\CustomerDataType();
			$customerData->setType("individual");
			$customerData->setId("99999456654");
			//$customerData->setEmail("EllenJohnson@example.com");
			//$transactionRequestType->setCustomer($customerData);


			// Set the customer's Bill To address
			$customerAddress = new AnetAPI\CustomerAddressType();
			$customerAddress->setFirstName("Ellen");
			$customerAddress->setLastName("Johnson");
			$customerAddress->setCompany("Souveniropolis");
			$customerAddress->setAddress("14 Main Street");
			$customerAddress->setCity("Pecan Springs");
			$customerAddress->setState("TX");
			$customerAddress->setZip("44628");
			$customerAddress->setCountry("USA");
			//$transactionRequestType->setBillTo($customerAddress);

			// Create order information
			$order = new AnetAPI\OrderType();
			$order->setInvoiceNumber($invoice);
			$order->setDescription(json_decode(json_encode($event->title())));
			$transactionRequestType->setOrder($order);

			// Build transaction request
			$request = new AnetAPI\GetHostedPaymentPageRequest();
			$request->setMerchantAuthentication($merchantAuthentication);
			$request->setRefId($invoice);
			$request->setTransactionRequest($transactionRequestType);

			// see https://developer.authorize.net/api/reference/features/accept_hosted.html
			$setting = new AnetAPI\SettingType();
			$setting->setSettingName("hostedPaymentButtonOptions");
			$setting->setSettingValue(json_encode(array("text" => "Pay Now Please")));
			$request->addToHostedPaymentSettings($setting);

			$setting = new AnetAPI\SettingType();
			$setting->setSettingName("hostedPaymentStyleOptions");
			$setting->setSettingValue(json_encode(array("bgColor" => "green")));
			$request->addToHostedPaymentSettings($setting);

			$setting = new AnetAPI\SettingType();
			$setting->setSettingName("hostedPaymentBillingAddressOptions");
			$setting->setSettingValue(json_encode(array("show" => false)));
			$request->addToHostedPaymentSettings($setting);

			$setting = new AnetAPI\SettingType();
			$setting->setSettingName("hostedPaymentCustomerOptions");
			$setting->setSettingValue(json_encode(array("showEmail" => true)));
			$request->addToHostedPaymentSettings($setting);

			$setting = new AnetAPI\SettingType();
			$setting->setSettingName("hostedPaymentReturnOptions");
			$root = Uri::root();
			$root = rtrim($root, '/');
			$urlSettings = array(
				//"url" => $root . "index.php/component/rsvppro/accounts.notify/authorizenet",
				"url" => $root . $event->viewDetailLink($event->yup(), $event->mup(), $event->dup(), true, $Itemid),
				"urlText" => "Back to site",
				"cancelUrl" => $root . $event->viewDetailLink($event->yup(), $event->mup(), $event->dup(), true, $Itemid),
				"cancelUrlText" => "Cancel Payment",
				"showReceipt" => false);

			$setting->setSettingValue( json_encode($urlSettings) );
			$request->addToHostedPaymentSettings($setting);

			$setting = new AnetAPI\SettingType();
			$setting->setSettingName("hostedPaymentPaymentOptions");
			$setting->setSettingValue(json_encode(array("showCreditCard" => true, "cardCodeRequired" => true)));
			$request->addToHostedPaymentSettings($setting);

			//execute request
			$controller = new AnetController\GetHostedPaymentPageController($request);
			$response   = $controller->executeWithApiResponse(\net\authorize\api\constants\ANetEnvironment::SANDBOX);

			if (($response != null) && ($response->getMessages()->getResultCode() == "Ok"))
			{
				$nonce =  $response->getToken() ;

				$targetHost = $templateParams->AZNMode;
				ob_start()
				?>
				<form id="send_hptoken" action="https://<?php echo $targetHost; ?>/payment/payment" method="post" target="_self" >
					<input type="hidden" name="token" value="<?php echo $nonce ?>" />
					<button type="submit" class="uk-button uk-button-primary" >Pay Now</button>
				</form>
				<?php
				$form = ob_get_clean();
				$html = str_replace("{FORM}", $form, $html);

				$detaillink = Route::_(Uri::root() . $event->viewDetailLink($event->yup(), $event->mup(), $event->dup(), false), false);

				// ANet Params
				$anet                 = new stdClass();
				$anet->invoice        = $invoice;
				$anet->amount         = $amount;
				$anet->currency       = $currency;
				$anet->detaillink     = $detaillink;
				$anet->transaction_id = $transaction->transaction_id;
				$anet->event          = $event;
				$anet->title          = $event->title;
				$anet->startdate      = JEventsHTML::getDateFormat($event->yup(), $event->mup(), $event->dup(), 0);
				$anet->templateParams = $templateParams;
				$anet->attendee_id    = $attendee->id;

				$session = Factory::getSession();
				$session->set('rsvppro_checkout', $anet);

			}
			else
			{
				$html .= "ERROR :  Failed to get hosted payment page token<br>";
				$html .= "Please report to site administrator<br>";
				return;
				//$errorMessages = $response->getMessages()->getMessage();
				//echo "RESPONSE : " . $errorMessages[0]->getCode() . "  " . $errorMessages[0]->getText() . "\n";
			}

			return $response;
		}

		$api_login_id = $params->get("login_id");
		$client_key   = $params->get("client_key", false);
		$transaction_key = $params->get("transaction_key");
		/*
				$signature_key = $params->get("signature_key", false);
				if (!$signature_key)
				{
					echo "<h2>Signature key is not set in the plugin - please review plugin parameters</h2>";
				}
		*/
		// See https://developer.authorize.net/api/reference/index.html#transaction-reporting-get-merchant-details
		if (!$client_key)
		{
			$goptions = new \joomla\Registry\Registry;
			$goptions->set('userAgent', 'Mozilla/5.0 (Windows NT 10.0; WOW64; rv:56.0) Gecko/20100101 Firefox/56.0');

			$http    = HttpFactory::getHTTP($goptions);
			$headers = array();
			$data    = array(
				"getMerchantDetailsRequest" => array(
					"merchantAuthentication" => array(
						"name"           => $api_login_id,
						"transactionKey" => $transaction_key
					)
				)
			);
			$data = json_encode($data);

			$clientkeydata = $http->post("https://api" . ($templateParams->AZNMode == "test.authorize.net" ? 'test' : '') . ".authorize.net/xml/v1/request.api", $data, $headers);
			if ($clientkeydata && $clientkeydata->code == 200)
			{
				// See https://stackoverflow.com/questions/17219916/json-decode-returns-json-error-syntax-but-online-formatter-says-the-json-is-ok
				$keydata = $clientkeydata->body;
				if (0 === strpos(bin2hex($keydata), 'efbbbf'))
				{
					$keydata = substr($keydata, 3);
				}
				$keydata = json_decode($keydata);
				if ($keydata && isset($keydata->messages) && isset($keydata->messages->resultCode) &&  $keydata->messages->resultCode== "Ok")
				{
					$client_key = $keydata->publicClientKey;
				}
			}

		}

		// sandbox test
		if (isset($templateParams->login_id) && $templateParams->login_id != $api_login_id)
		{
			$api_login_id = $templateParams->login_id;
		}

		// See https://developer.authorize.net/api/reference/features/acceptjs.html#Integrating_the_Hosted_Payment_Information_Form
		// https://developer.authorize.net/hello_world/testing_guide/
		HTMLHelper::_('jquery.framework');
		JEVHelper::script("components/com_jevents/assets/js/jQnc.js");
		$script = <<< SCRIPT
        function authorizeResponseHandler (response) {
        console.log(response);
            if (response.messages.resultCode == "Error") {
                var i = 0;
                var msg = "";
                while (i < response.messages.message.length) {
	                msg += " " +response.messages.message[i].text;
                    i ++;
                }
                alert(msg);
            }
            else if (response.messages.resultCode == "Ok") {
                paymentFormUpdate(response.opaqueData);
            }
        }
             
        function paymentFormUpdate(opaqueData) {
            document.getElementById("dataDescriptor").value = opaqueData.dataDescriptor;
            document.getElementById("dataValue").value = opaqueData.dataValue;
            
            document.getElementById("paymentForm").submit();
        }
SCRIPT;
		Factory::getDocument()->addScriptDeclaration($script);

		ob_start();
		?>
		<form id="paymentForm"
		      method="POST"
		      action="<?php echo JURI::root() . "index.php?option=com_rsvppro&task=accounts.notify&gateway=authorizenet&Itemid=$Itemid"; ?>">
			<input type="hidden" name="dataValue" id="dataValue"/>
			<input type="hidden" name="dataDescriptor" id="dataDescriptor"/>
			<input type="hidden" name="invoice" id="dataDescriptor"/>
			<button type="button"
			        class="AcceptUI uk-button uk-button-primary"
			        data-billingAddressOptions='{"show":true, "required":false}'
			        data-apiLoginID="<?php echo $api_login_id; ?>"
			        data-clientKey="<?php echo $client_key; ?>"
			        data-acceptUIFormBtnTxt="Submit"
			        data-acceptUIFormHeaderTxt="Payment Details"
			        data-paymentOptions='{"showCreditCard": true, "showBankAccount": false}'
			        data-responseHandler="authorizeResponseHandler">
				Pay Now
			</button>
		</form>


		<script type="text/javascript"
		        src="https://js<?php echo $templateParams->AZNMode == "test.authorize.net" ? 'test' : ''; ?>.authorize.net/v3/AcceptUI.js"
		        charset="utf-8"
		        defer
		>
		</script>
		<?php
		$form = ob_get_clean();
		$html = str_replace("{FORM}", $form, $html);

		$amount     = RsvpHelper::ceil_dec(Factory::getApplication()->input->getFloat("amount", 0), 2, ".");
		$detaillink = Route::_(Uri::root() . $event->viewDetailLink($event->yup(), $event->mup(), $event->dup(), false), false);

		$currency   = $templateParams->Currency;

		// ANet Params
		$anet                 = new stdClass();
		$anet->invoice        = $invoice;
		$anet->amount         = $amount;
		$anet->currency       = $currency;
		$anet->detaillink     = $detaillink;
		$anet->transaction_id = $transaction->transaction_id;
		$anet->event          = $event;
		$anet->title          = $event->title;
		$anet->startdate      = JEventsHTML::getDateFormat($event->yup(), $event->mup(), $event->dup(), 0);
		$anet->templateParams = $templateParams;
		$anet->attendee_id    = $attendee->id;

		$session = Factory::getSession();
		$session->set('rsvppro_checkout', $anet);

	}

	public static function NotifyPayment($templateParams)
	{
		return $templateParams->get("aznotifyppay", 1);
	}

	public static function PaymentMessageType()
	{
		return "azpay";
	}

	static public function transactionDetailLink($transaction)
	{

		$plugin = PluginHelper::getPlugin("rsvppro", "authorizenet");
		$params = new Registry($plugin->params);

		$transactionLogdata = json_decode($transaction->logdata);
		if (!is_object($transactionLogdata) || !isset($transactionLogdata->request))
			return "";


		$url = $params->get('AZNMode') == "test.authorize.net" ?
			"sandbox.authorize.net/UI/themes/sandbox/transaction/transactiondetail.aspx?transID=" :
			"account.authorize.net/UI/themes/anet/transaction/transactiondetail.aspx?transID=";

		// Old requests
		if (isset($transactionLogdata->request->x_trans_id))
		{
			return '<a href="https://' . $url . $transactionLogdata->request->x_trans_id . '" target="_blank">' . $transactionLogdata->request->x_trans_id . '</a>';
		}
		else
		{
			return '<a href="https://' . $url . $transaction->transactionId . '" target="_blank">' . $transaction->transactionId . '</a>';
		}

	}

	public function onActiveGatewayClass(&$activeGatewayClass, $action = "notify")
	{
		$gateway = Factory::getApplication()->input->getString("gateway");

		if ($gateway == "authorizenet" || strpos($gateway, "authorizenet_") === 0)
		{
			$activeGatewayClass = __CLASS__;
		}

	}

	public function activeGateways(&$activeGatewayClasses)
	{
		$activeGatewayClasses[] = __CLASS__;

	}

	static public function notifyRsvpGateway(&$args)
	{

		// get Jevents params and send messages to admin if necessary
		$jevparams   = ComponentHelper::getParams("com_jevents");
		$jevadmin = $jevparams->get("jevadmin", -1);
		if ($jevadmin == -1)
		{
			return;
		}

		$plugin = PluginHelper::getPlugin("rsvppro", "authorizenet");
		$params = new Registry($plugin->params);

		$jevadmin = JEVHelper::getUser($jevadmin);
		$receiver_email    = $jevadmin->email;

		$gateway = Factory::getApplication()->input->getString("gateway");

		if ($gateway == "authorizenet" || $gateway == "1")
		{

			HTMLHelper::_('jquery.framework');
			JEVHelper::script("components/com_jevents/assets/js/jQnc.js");

			$dataValue      = Factory::getApplication()->input->getCmd("dataValue");
			$dataDescriptor = Factory::getApplication()->input->getCmd("dataDescriptor");

			$method = $params->get("AZNMechanism","acceptjs");

			$plugin = PluginHelper::getPlugin("rsvppro", "authorizenet");
			$params = new Registry($plugin->params);

			if ($method == "accepthosted")
			{
				// See https://developer.authorize.net/api/reference/features/webhooks.html

				// verification
				// see https://developer.authorize.net/api/reference/features/webhooks.html#Verifying_the_Notification

				$headers = getallheaders();
				$payload = file_get_contents("php://input");

				if (isset($headers['X-Anet-Signature']) && !empty($headers['X-Anet-Signature']))
				{
					$signature_key = $params->get("signature_key");
					$parts = explode("=",$headers['X-Anet-Signature'] );
					if (count($parts) == 2)
					{
						$algo = $parts[0];
						$targetHash = strtoupper($parts[1]);
						if (in_array($algo, hash_hmac_algos()))
						{
							$hash = strtoupper(hash_hmac("sha512", $payload, $signature_key));

							if ($hash === $targetHash)
							{
								// example payload
								/*
								 Payload {"notificationId":"b52589cc-9f71-4139-918a-ec09ff206e70","eventType":"net.authorize.payment.authcapture.created","eventDate":"2021-05-07T11:43:33.360663Z","webhookId":"33eb5a72-aec0-41f4-999b-6e8013173499","payload":{"responseCode":1,"authCode":"UQC823","avsResponse":"Y","authAmount":10.00,"merchantReferenceId":"3_61","invoiceNumber":"3_61","entityName":"transaction","id":"60167244484"}}
								 */
								$logText = "Authorize.net Response - " . gmstrftime("%b %d %Y %H:%M:%S", time()) . "\n";

								$payload = json_decode($payload);
								if ($payload->eventType == "net.authorize.payment.authcapture.created")
								{
									if (isset($payload->payload))
									{
										$paymentData = $payload->payload;

										$msgs = array("1" => "This transaction has been approved. <br/>",
										              "2" => "This transaction has been declined.<br/> ",
										              "3" => "There has been an error processing this transaction. <br/>",
										              "4" => "This transaction is being held for review.<br/>");

										// Load the transaction instance
										$transaction = new rsvpTransaction();
										$invoiceParts = explode("_", $paymentData->invoiceNumber);
										if (count($invoiceParts) == 2)
										{
											$transaction_id = intval($invoiceParts[1]);

											if (!$transaction->load($transaction_id))
											{
												$logText .= "Error loading rsvpTransaction instance " . $transaction_id . "\n";
												$logText .= $transaction->getError();
												$logText .= 'headers: ' . print_r($headers) . "\n";
												$logText .= 'payload: ' . print_r($payload) . "\n";
												plgRsvpproAuthorizenet::Authorize_Log($logText);
												plgRsvpproAuthorizenet::Authorize_Error($logText, $receiver_email);

												return;
											}

											$transaction->amount              = $paymentData->authAmount;
											$transaction->gateway             = 'authorizenet';
											$transaction->logdata             = array();
											$transaction->logdata["headers"]  = json_decode(json_encode($headers));
											$transaction->logdata["payload"]  = $payload;

											// fetch the session and the attendee
											$db  = Factory::getDBO();
											$sql = "SELECT * FROM #__jev_attendees WHERE id=" . $transaction->attendee_id;
											$db->setQuery($sql);
											$attendee = $db->loadObject();
											if (!$attendee)
											{
												$logText .= "Error loading attendee instance " . $transaction->attendee_id . "\n";
												$logText .= "DB Error is " . $db->getErrorMessage() . "\n";
												$logText .= 'Log data: ' . print_r($transaction->logdata, true) . "\n";
												plgRsvpproAuthorizenet::Authorize_Log($logText);
												plgRsvpproAuthorizenet::Authorize_Error($logText, $receiver_email);

												return;
											}

											$sql = "SELECT * FROM #__jev_attendance WHERE id=" . $attendee->at_id;
											$db->setQuery($sql);
											$rsvpdata = $db->loadObject();
											if (!$rsvpdata)
											{
												$logText .= "Error loading rsvpdata instance" . $attendee->at_id . "\n";
												$logText .= "DB Error is " . $db->getErrorMessage() . "\n";
												$logText .= 'Log data: ' . print_r($transaction->logdata, true) . "\n";
												plgRsvpproAuthorizenet::Authorize_Log($logText);
												plgRsvpproAuthorizenet::Authorize_Error($logText, $receiver_email);

												return;
											}

											if ($paymentData->responseCode == 1)
											{

												$transaction->paymentstate = 1;
												// return int 1 = Approved, 2 = Declined, 3 = Error, 4 = Held for Review
												$transaction->logdata['responseCode']         = $paymentData->responseCode;
												$transaction->logdata['transactionId']        = $paymentData->id;
												$transaction->logdata['authcode']             = isset($paymentData->authCode) ? $paymentData->authCode : '';

												if (!$transaction->check())
												{

													$logText .= "Error checking rsvpTransaction instance\n";
													$logText .= $transaction->getError();
													$logText .= 'Log data: ' . print_r($transaction->logdata, true) . "\n";
													plgRsvpproAuthorizenet::Authorize_Log($logText);
													plgRsvpproAuthorizenet::Authorize_Error($logText, $receiver_email);

													return;
												}

												if (!$transaction->store())
												{

													$logText .= "Error saving rsvpTransaction instance\n";
													$logText .= $transaction->getError();
													$logText .= 'Log data: ' . print_r($transaction->logdata, true) . "\n";
													plgRsvpproAuthorizenet::Authorize_Log($logText);
													plgRsvpproAuthorizenet::Authorize_Error($logText, $receiver_email);

													return;
												}

												plgRsvpproAuthorizenet::notifyAuthorizePayment($transaction, $attendee, $rsvpdata);
/*
												$logText .= "All went well with this transaction\n";
												$logText .= 'Log data: ' . print_r($transaction->logdata, true) . "\n";
												plgRsvpproAuthorizenet::Authorize_Log($logText);
												plgRsvpproAuthorizenet::Authorize_Error($logText, $receiver_email);
*/

												$comparams = ComponentHelper::getParams("com_rsvppro");
												include_once(JPATH_ADMINISTRATOR . "/components/com_rsvppro/libraries/attendeehelper.php");
												$attendeehelper = new RsvpAttendeeHelper($comparams);

												$attendeehelper->countAttendees($attendee->at_id);

												return;

											}
											else
											{
												$transaction->paymentstate = -1;

												echo "There was a problem with a payment " . $msgs[$paymentData->responseCode] . "<br/>";
												$logText .= "Error processing payment\n";
												$logText .= $msgs[$paymentData->responseCode] . "\n";
												$logText .= "Reason code is " . $paymentData->responseCode . " See http://www.authorize.net/support/merchant/Transaction_Response/Response_Reason_Codes_and_Response_Reason_Text.htm";

												$logText .= 'request was: ' . print_r($transaction->logdata, true) . "\n";

												return;

											}
										}

									}
								}

							}
							else
							{
								$logText = "Authorize.net Response Problem - " . gmstrftime("%b %d %Y %H:%M:%S", time()) . "\n";

								$logText .= 'Missmatched Hash ' . $hash . " vs " . $targetHash . "\n";
								$logText .= 'Headers ' . print_r($headers, true) . "\n";
								$logText .= 'Payload ' . print_r($payload, true) . "\n";

								plgRsvpproAuthorizenet::Authorize_Log($logText);
								plgRsvpproAuthorizenet::Authorize_Error($logText, $receiver_email);
								return;
							}
						}
					}
				}


				$logText = "Authorize.net Response Problem  fell through - " . gmstrftime("%b %d %Y %H:%M:%S", time()) . "\n";

				$logText .= 'Headers ' . print_r($headers, true) . "\n";
				$logText .= 'Payload ' . print_r($payload, true) . "\n";

				plgRsvpproAuthorizenet::Authorize_Log($logText);
				plgRsvpproAuthorizenet::Authorize_Error($logText, $receiver_email);

				return;
			}
			else
			{
				$session = Factory::getSession();
				$anet    = $session->get('rsvppro_checkout', '{}');

				if (!isset($anet->currency))
				{
					$logText = "Authorize.net Response - " . gmstrftime("%b %d %Y %H:%M:%S", time()) . "\n";

					$logText .= "Houston we have a problem";
					$logText .= 'anet ' . print_r($anet, true) . "\n";
					$logText .= 'REQUEST ' . print_r($_REQUEST, true) . "\n";

					$headers = getallheaders();
					$payload = file_get_contents("php://input");

					$logText .= 'Headers ' . print_r($headers, true) . "\n";
					$logText .= 'Payload ' . print_r($payload, true) . "\n";

					plgRsvpproAuthorizenet::Authorize_Log($logText);
					plgRsvpproAuthorizenet::Authorize_Error($logText, $receiver_email);

					return;

					echo "Houston we have a problem";

					return false;
				}



				$gateway = Omnipay::create('AuthorizeNet_AIM');

				$api_login_id    = $params->get("login_id");
				$transaction_key = $params->get("transaction_key");
				/*
							$signature_key = $params->get("signature_key", false);
							if (!$signature_key)
							{
								?>
								<h2>MISSING SIGNATURE KEY - PLEASE REPORT TO SITE ADMINISTRATOR ASAP</h2>
								<?php return;
							}
				*/
				$templateParams = $anet->templateParams;

				// sandbox test
				if (isset($templateParams->login_id) && $templateParams->login_id != $api_login_id)
				{
					$api_login_id = $templateParams->login_id;
				}

				//$gateway->setSignatureKey($signature_key);
				$gateway->setDeveloperMode($templateParams->AZNMode == "test.authorize.net");
				$gateway->setTestMode($templateParams->AZNMode == "test.authorize.net");
				$gateway->setApiLoginId($api_login_id);
				$gateway->setTransactionKey($transaction_key);

				//$transaction = $anet->transaction;

				Log::add(
					"Authorizenet Gateway Purchase - Amount: " . self::phpMoneyFormat($anet->amount, $templateParams)
					. ", Currency: " . $anet->currency . ", Transaction ID: " . $anet->transaction_id, Log::INFO, 'com_rsvppro.authorizenet'
				);
				$response = $gateway->purchase([
					'amount'               => $anet->amount,
					'currency'             => $anet->currency,
					'description'          => $anet->title . " : " . $anet->startdate,
					'invoiceNumber'        => $anet->invoice,
					'opaqueDataDescriptor' => $dataDescriptor,
					'opaqueDataValue'      => $dataValue,
				])->send();

				if ($response)
				{

					$logText = "Authorize.net Response - " . gmstrftime("%b %d %Y %H:%M:%S", time()) . "\n";

					if ($response->isSuccessful())
					{
						$msgs = array("1" => "This transaction has been approved. <br/>",
						              "2" => "This transaction has been declined.<br/> ",
						              "3" => "There has been an error processing this transaction. <br/>",
						              "4" => "This transaction is being held for review.<br/>");

						// Load the transaction instance
						$transaction = new rsvpTransaction();
						if (!$transaction->load($anet->transaction_id))
						{
							$logText .= "Error loading rsvpTransaction instance " . $anet->transaction_id . "\n";
							$logText .= $transaction->getError();
							$logText .= 'REQUEST: ' . print_r(json_decode(json_encode($response->getRequest())), true) . "\n";
							$logText .= 'Response: ' . print_r(json_decode(json_encode($response->getData())), true) . "\n";
							plgRsvpproAuthorizenet::Authorize_Log($logText);
							plgRsvpproAuthorizenet::Authorize_Error($logText, $receiver_email);

							Factory::getApplication()->enqueueMessage("Invalid transaction record specified");
							Factory::getApplication()->redirect($anet->detaillink);

							return;
						}

						$transaction->amount              = $anet->amount;
						$transaction->currency            = $anet->currency;
						$transaction->gateway             = 'authorizenet';
						$transaction->logdata             = array();
						$transaction->logdata["request"]  = json_decode(json_encode($response->getRequest()));
						$transaction->logdata["response"] = json_decode(json_encode($response->getData()));

						// fetch the session and the attendee
						$db  = Factory::getDBO();
						$sql = "SELECT * FROM #__jev_attendees WHERE id=" . $anet->attendee_id;
						$db->setQuery($sql);
						$attendee = $db->loadObject();
						if (!$attendee)
						{
							$logText .= "Error loading attendee instance " . $anet->attendee_id . "\n";
							$logText .= "DB Error is " . $db->getErrorMessage() . "\n";
							$logText .= 'Log data: ' . print_r($transaction->logdata, true) . "\n";
							plgRsvpproAuthorizenet::Authorize_Log($logText);
							plgRsvpproAuthorizenet::Authorize_Error($logText, $receiver_email);

							Factory::getApplication()->enqueueMessage("Invalid attendee record specified");
							Factory::getApplication()->redirect($anet->detaillink);

							return;
						}

						$sql = "SELECT * FROM #__jev_attendance WHERE id=" . $attendee->at_id;
						$db->setQuery($sql);
						$rsvpdata = $db->loadObject();
						if (!$rsvpdata)
						{
							$logText .= "Error loading rsvpdata instance" . $attendee->at_id . "\n";
							$logText .= "DB Error is " . $db->getErrorMessage() . "\n";
							$logText .= 'Log data: ' . print_r($transaction->logdata, true) . "\n";
							plgRsvpproAuthorizenet::Authorize_Log($logText);
							plgRsvpproAuthorizenet::Authorize_Error($logText, $receiver_email);

							Factory::getApplication()->enqueueMessage("Invalid attendance record specified");
							Factory::getApplication()->redirect($anet->detaillink);

							return;
						}

						if ($response->getResultCode() == 1)
						{
							$transaction->paymentstate = 1;

							$transaction->paymentstate = $response->getResultCode();
							// return int 1 = Approved, 2 = Declined, 3 = Error, 4 = Held for Review
							$transaction->logdata['resultCode']           = $response->getResultCode();
							$transaction->logdata['transactionReference'] = $response->getTransactionReference();
							$transaction->logdata['transactionId']        = $response->getTransactionId();

							if (!$transaction->check())
							{

								$logText .= "Error checking rsvpTransaction instance\n";
								$logText .= $transaction->getError();
								$logText .= 'Log data: ' . print_r($transaction->logdata, true) . "\n";
								plgRsvpproAuthorizenet::Authorize_Log($logText);
								plgRsvpproAuthorizenet::Authorize_Error($logText, $receiver_email);

								Factory::getApplication()->enqueueMessage("There was a problem with your payment");
								Factory::getApplication()->redirect($anet->detaillink);

								return;
							}

							if (!$transaction->store())
							{

								$logText .= "Error saving rsvpTransaction instance\n";
								$logText .= $transaction->getError();
								$logText .= 'Log data: ' . print_r($transaction->logdata, true) . "\n";
								plgRsvpproAuthorizenet::Authorize_Log($logText);
								plgRsvpproAuthorizenet::Authorize_Error($logText, $receiver_email);

								Factory::getApplication()->enqueueMessage("There was a problem with your payment 2");
								Factory::getApplication()->redirect($anet->detaillink);

								return;
							}

							plgRsvpproAuthorizenet::notifyAuthorizePayment($transaction, $attendee, $rsvpdata);

							$comparams = ComponentHelper::getParams("com_rsvppro");
							include_once(JPATH_ADMINISTRATOR . "/components/com_rsvppro/libraries/attendeehelper.php");
							$attendeehelper = new RsvpAttendeeHelper($comparams);

							$attendeehelper->countAttendees($attendee->at_id);

							Factory::getApplication()->enqueueMessage("Your payment succeeded");
							Factory::getApplication()->redirect($anet->detaillink);


						}
						else
						{
							$transaction->paymentstate = -1;

							echo "There was a problem with your payment " . $msgs[$response->getResultCode()] . "<br/>";
							$logText .= "Error processing payment\n";
							$logText .= $msgs[$response->getResultCode()] . "\n";
							$logText .= "Reason code is " . $response->getResultCode() . " See http://www.authorize.net/support/merchant/Transaction_Response/Response_Reason_Codes_and_Response_Reason_Text.htm";

							$logText .= 'request was: ' . print_r($transaction->logdata, true) . "\n";

							plgRsvpproAuthorizenet::Authorize_Log($logText);
							plgRsvpproAuthorizenet::Authorize_Error($logText, $receiver_email, false);

							Factory::getApplication()->enqueueMessage("There was a problem with your payment " . $msgs[$response->getResultCode()]);
							Factory::getApplication()->redirect($anet->detaillink);

							return;

						}


					}
					else
					{
						$logText .= "Transaction was not successful " . $anet->transaction_id . "\n";
						$logText .= $response->getMessage();
						$logText .= 'REQUEST: ' . print_r(json_decode(json_encode($response->getRequest())), true) . "\n";
						$logText .= 'Response: ' . print_r(json_decode(json_encode($response->getData())), true) . "\n";
						plgRsvpproAuthorizenet::Authorize_Log($logText);
						plgRsvpproAuthorizenet::Authorize_Error($logText, $receiver_email);

						Factory::getApplication()->enqueueMessage("Transaction was not successful " . $response->getMessage(), 'error');
						Factory::getApplication()->redirect($anet->detaillink);

						return;

					}
				}

				else
				{
					Factory::getApplication()->enqueueMessage("Unable to process payment at Authorize.net");
					Factory::getApplication()->redirect($anet->detaillink);
				}
			}
		}

	}

	function phpMoneyFormat($amount, $template)
	{
		if (isset($template->Currency))
		{
			$digits    = $template->CurrencyDigits;
			$symbol    = $template->CurrencySymbol;
			$onLeft    = (strcmp($template->CurrencyPlacement, 'left') == 0);
			$separator = $template->CurrencySeparator;
			$decimal   = $template->CurrencyDecimal;
		}
		else
		{
			$eSessParams = ComponentHelper::getParams('com_rsvppro');

			$digits    = $eSessParams->get("CurrencyDigits");
			$symbol    = $eSessParams->get("CurrencySymbol");
			$onLeft    = (strcmp($eSessParams->get("CurrencyPlacement"), 'left') == 0);
			$separator = $eSessParams->get("CurrencySeparator");
			$decimal   = $eSessParams->get("CurrencyDecimal");
		}
		$formattedText = '';

		// negative amount
		if ($amount < 0)
			$formattedText .= '-';

		// currency symbol on the left
		if ($onLeft)
			$formattedText .= $symbol;

		// format with correct number of digits and separator
		if ($digits > 0)
		{
			$amount        = RsvpHelper::ceil_dec(abs($amount), $digits, ".");
			$formattedText .= number_format(abs($amount), $digits, $decimal, $separator);
		}
		else
		{
			$formattedText = abs($amount);
		}

		// currency symbols on the right
		if (!$onLeft)
			$formattedText .= $symbol;


		return $formattedText;

	}

	public static function Authorize_Log($text)
	{

		$params = ComponentHelper::getParams(RSVP_COM_COMPONENT);

		$logfile = JPATH_SITE . str_replace("//", "/", $params->get('AZLogFileLocation', '/administrator/components/com_rsvppro/logs/') . '/authorize_log.txt');
		$logfile = str_replace('/authorize_log.txt/authorize_log.txt', '/authorize_log.txt', $logfile);

		$fp = @fopen($logfile, 'a');
		if ($fp)
		{
			@fwrite($fp, $text . "\n\n");
			@fclose($fp);
		}
	}

	public static function Authorize_Error($text, $email, $die = true)
	{

		$mail = Factory::getMailer();

		try
		{
			return $mail->sendMail($email, 'authorize.net log', $email, 'Authorize.net Error', $text, 0);
		}
		catch (Exception $e)
		{
			return false;
		}

	}

	private function notifyAuthorizePayment($transaction, $attendee, $rsvpdata)
	{
		// immediate notification
		if (RsvpHelper::getTemplateSpecificParameter($rsvpdata,  "aznotifyppay", 1) > 0 && $transaction->gateway == "authorizenet")
		{

			$comparams = ComponentHelper::getParams("com_rsvppro");
			include_once(JPATH_ADMINISTRATOR . "/components/com_rsvppro/libraries/attendeehelper.php");
			$helper = new RsvpAttendeeHelper($comparams);

			$rpid = $attendee->rp_id;

			$dataModel  = new JEventsDataModel();
			$queryModel = new JEventsDBModel($dataModel);

			// Find the first repeat
			$vevent = $dataModel->queryModel->getEventById($rsvpdata->ev_id, false, "icaldb");
			if ($rpid == 0)
			{
				$repeat = $vevent->getFirstRepeat();
			}
			else
			{
				list($year, $month, $day) = JEVHelper::getYMD();
				$repeatdata = $dataModel->getEventData(intval($rpid), "icaldb", $year, $month, $day);
				if ($repeatdata && isset($repeatdata["row"]))
					$repeat = $repeatdata["row"];
			}

			$user = JEVHelper::getUser($attendee->user_id);
			if ($user->id == 0 && RsvpHelper::getTemplateSpecificParameter($rsvpdata,  "attendemails", 0))
			{
				$name     = $attendee->email_address;
				$username = $attendee->email_address;
			}
			else
			{
				$name     = $user->name;
				$username = $user->username;
			}

			$class      = "plgRsvppro" . ucfirst($transaction->gateway);
			$pluginpath = version_compare(JVERSION, "1.6.0", 'ge') ? JPATH_SITE . "/plugins/rsvppro/$transaction->gateway/" : JPATH_SITE . "/plugins/rsvppro/";
			JLoader::register($class, $pluginpath . $transaction->gateway . ".php");

			$helper->notifyUser($rsvpdata, $repeat, $user, $name, $username, $attendee, 'azpay', false, $transaction);

			return $repeat;
		}
	}


}
