<?php
/**
 * JEvents Component for Joomla 1.5.x
 *
 * @version     $Id: icalevent.php 1712 2010-03-04 07:33:11Z geraint $
 * @package     JEvents
 * @copyright   Copyright (C) 2008-2021 GWE Systems Ltd, 2006-2008 JEvents Project Group
 * @license     GNU/GPLv2, see http://www.gnu.org/licenses/gpl-2.0.html
 * @link        https://www.jevents.net
 */

defined( '_JEXEC' ) or die( 'Restricted Access' );

use Joomla\CMS\MVC\Controller\BaseController;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Pagination\Pagination;
use Joomla\CMS\Factory;
use Joomla\CMS\Input\Input;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Plugin\PluginHelper;

jimport('joomla.application.component.controller');


class AdminSessionsController extends BaseController {

	/**
	 * Controler for Sessions
	 * @param array		configuration
	 */
	function __construct($config = array())
	{
		parent::__construct($config);
		$this->registerTask( 'list',  'overview' );
		$this->registerDefaultTask("overview");

		// Must load admin language files
		$lang = Factory::getLanguage();
		$lang->load("com_jevents", JPATH_ADMINISTRATOR);

		$this->dataModel = new JEventsDataModel();
		$this->queryModel = new JEventsDBModel($this->dataModel);

		$plugin = PluginHelper::getPlugin("jevents", "jevrsvppro");
		$this->params = ComponentHelper::getParams("com_rsvppro");

		include_once(JPATH_ADMINISTRATOR . "/components/com_rsvppro/libraries/attendeehelper.php");
		$this->attendeehelper = new RsvpAttendeeHelper($this->params);

	}

	/**
	 * List Sessions
	 *
	 */
	function overview( )
	{
		// First task is to do the auto-cleanup of unpaid attendees
		// This could add a lot of weight over time for sites with no auto - cleaning so allow for it to need two refreshes
		// $this->attendeehelper->cleanupUnpaidAttendees();

		// get the view
		$this->view = $this->getView("sessions","html");

		$mainframe = Factory::getApplication();
		$option=Factory::getApplication()->input->getCmd("option");
		$db = Factory::getDbo();
		$jevparams =  ComponentHelper::getParams(JEV_COM_COMPONENT);
        $rsvpparams = ComponentHelper::getParams(RSVP_COM_COMPONENT);

		$search		= $mainframe->getUserStateFromRequest( "search{".RSVP_COM_COMPONENT."}", 'search', '' );
		$search		= $db->escape( trim( strtolower( $search ) ) );

		$hidepast = intval($mainframe->getUserStateFromRequest( "hidepastrsvp", 'hidepast', 1 ));

		$after		= $mainframe->getUserStateFromRequest( "after{".RSVP_COM_COMPONENT."}", 'after', '' );
		$after		= $db->escape( trim( strtolower( $after ) ) );
        if ($after){
            $inputdateformat = $jevparams->get("com_editdateformat", "d.m.Y");
            $after = DateTime::createFromFormat($inputdateformat, $after);
            $after = $after->format("Y-m-d");
        }

		$searchattendees = $mainframe->getUserStateFromRequest("atsearchattendees{" . RSVP_COM_COMPONENT . "}", 'searchattendees', '');
		$searchattendees = $db->escape(trim(strtolower($searchattendees)));

        $searchdidattend  = $mainframe->getUserStateFromRequest("atsearchdidattend{" . RSVP_COM_COMPONENT . "}", 'searchdidattend', 0);
        $searchdidattend  = intval($searchdidattend);
        $postinput = new Input($_POST);
        if (!$searchattendees || ($postinput->get("task","","cmd")=="sessions.list" && $postinput->get("searchdidattend",0,"int")==0)){
            $searchdidattend  = 0;
            $mainframe->setUserState("atsearchdidattend{" . RSVP_COM_COMPONENT . "}", $searchdidattend);
        }

		$searchinvitees = $mainframe->getUserStateFromRequest("atsearchinvitees{" . RSVP_COM_COMPONENT . "}", 'searchinvitees', '');
		$searchinvitees = $db->escape(trim(strtolower($searchinvitees)));

		$limit		= intval( $mainframe->getUserStateFromRequest( "sessionslistlimit", 'limit', Factory::getApplication()->get("list_limit", 10) ));
		$limitstart = Factory::getApplication()->input->getInt('limitstart', 0 );
		$created_by	=  intval($mainframe->getUserStateFromRequest( "createdbyrsvp", 'created_by', 0 ));
		$nonrepeating = intval($mainframe->getUserStateFromRequest( "repeatingrsvp", 'repeating', -1 ));
		$searchtemplates	=  intval($mainframe->getUserStateFromRequest( "searchtemplatesrsvp", 'searchtemplates', 0 ));

		$catid = intval(Factory::getApplication()->getUserStateFromRequest("catidIcalEvents", 'catid', 0));
		$catidtop = $catid;

		$where = array();
		$join = array();

		if( $search ){
			$where[] = "LOWER(det.summary) LIKE '%$search%'";
		}

		if ($searchattendees)
		{
            if ($searchattendees=="*"){
                $where[] = "atdees.id IS NOT NULL ";
                //$where[] = "atdees.attendstate=1 ";
            }
            else {
                $searchallfields = $rsvpparams->get("searchallfields",0);
                if ($searchallfields ){
                    $where[] = "(atdees.params LIKE '%$searchattendees%' OR atdees.email_address LIKE '%$searchattendees%' OR jua.name LIKE '%$searchattendees%' OR jua.username LIKE '%$searchattendees%' OR jua.email LIKE '%$searchattendees%'  OR atdees.id = '$searchattendees' )";
                }
                else {
                    $where[] = "(atdees.email_address LIKE '%$searchattendees%' OR jua.name LIKE '%$searchattendees%' OR jua.username LIKE '%$searchattendees%' OR jua.email LIKE '%$searchattendees%'  OR atdees.id = '$searchattendees' )";
                }
            }
			$join [] = " #__users AS jua ON jua.id = atdees.user_id"	;

		}

		if ($searchdidattend && $searchattendees)
		{
            $where[] = "atdees.didattend=1";
        }

		if ($searchtemplates)
		{
			$where[] = "atd.template = $searchtemplates";
		}

		if ($searchinvitees)
		{
			$where[] = "(inv.email_address LIKE '%$searchinvitees%' OR jui.username LIKE '%$searchinvitees%' OR jui.email LIKE '%$searchinvitees%' )";
			$join [] = " #__users AS jui ON jui.id = inv.user_id"	;
		}


		if ($created_by!==0) {
			$where[] = "ev.created_by=".$db->Quote($created_by);
		}

		$where[] = "ev.ev_id IS NOT NULL";

		$user = Factory::getUser();
		if ($jevparams->get("multicategory",0)){
			$join[] = "\n #__jevents_catmap as catmap ON catmap.evid = rpt.eventid";
			$join[] = "\n #__categories AS catmapcat ON catmap.catid = catmapcat.id";
			$where[]= " catmapcat.access " . ' IN (' . JEVHelper::getAid($user) . ')' ;
			$needsgroup = true;
		}


		// category filter!
			$authorisedonly = $jevparams->get("authorisedonly", 0);
			$cats = $user->getAuthorisedCategories('com_jevents', 'core.create');
			if (isset($user->id) && !$user->authorise('core.create', 'com_jevents') && !$authorisedonly)
			{
				if (count($cats) > 0 && $catid < 1)
				{
					for ($i = 0; $i < count($cats); $i++)
					{
						if ($jevparams->get("multicategory",0)){
							$whereCats[$i] = "catmap.catid='$cats[$i]'";
						}
						else {
							$whereCats[$i] = "ev.catid='$cats[$i]'";
						}
					}
					$where[] = '(' . implode(" OR ", $whereCats) . ')';
				}
				else if (count($cats) > 0 && $catid > 0 && in_array($catid, $cats))
				{
					if ($jevparams->get("multicategory",0)){
						$where[] = "catmap.catid='$catid'";
					}
					else {
						$where[] = "ev.catid='$catid'";
					}
				}
				else
				{
					if ($jevparams->get("multicategory",0)){
						$where[] = "catmap.catid=''";
					}
					else {
						$where[] = "ev.catid=''";
					}
				}
			}
			else
			{
				if ($catid > 0)
				{
					if ($jevparams->get("multicategory",0)){
						$where[] = "catmap.catid='$catid'";
					}
					else {
						$where[] = "ev.catid='$catid'";
					}
				}
			}

		if ($after) {
			$where[] = "\n (rpt.startrepeat >".$db->quote($after)." OR atd.regclose>".$db->quote($after).")";
		}

// get the total number of records in three blocks
		if ($nonrepeating == -1)
		{
			if ($hidepast){
				if (class_exists("JevDate")) {
					$datenow = JevDate::getDate("-1 day");
					$datenowUnix = $db->quote($datenow->toUnix());
					$datenow = $db->quote($datenow->toMySQL());
				}
				else {
					$datenow = Factory::getDate("-1 day");
					$datenowUnix = $db->quote($datenow->toUnix());
					$datenow = $db->quote($datenow->toSql());
				}
				$where[] = "\n ((rpt.endrepeat>".$datenow." AND atd.allrepeats=0) OR "
					.  "((det.dtstart>".$datenowUnix." OR atd.regclose>".$datenow.") AND atd.allrepeats=1 AND ev.ev_id is not null))";
			}

			$where[] = "atd.allowregistration>0";
			// fix the access
			$where[] = "ev.access " .  ' IN (' . JEVHelper::getAid($user) . ')';
			$where[] = "(atd.sessionaccess=-1 OR atd.sessionaccess " . ' IN (' . JEVHelper::getAid($user) . ')' .")";

			$query = "SELECT count(distinct CASE atd.allrepeats WHEN 1 THEN -1 * ev.ev_id ELSE rpt.rp_id END)"
				. "\n FROM #__jevents_vevent AS ev "
				. "\n LEFT JOIN #__jevents_repetition as rpt ON rpt.eventid=ev.ev_id"
				. "\n LEFT JOIN #__jevents_vevdetail as det ON ev.detail_id=det.evdet_id"
				. "\n LEFT JOIN #__jev_attendance AS atd ON atd.ev_id = ev.ev_id"
				. "\n LEFT JOIN #__jev_invitees AS inv ON atd.id = inv.at_id  AND inv.rp_id=rpt.rp_id"
				. "\n LEFT JOIN #__jev_attendeecount AS atdc ON atd.id = atdc.at_id AND atdc.rp_id=rpt.rp_id"
				. "\n LEFT JOIN #__jev_attendees AS atdees ON atdees.at_id = atd.id AND atdees.rp_id=rpt.rp_id"
				. ( count( $join) ? "\n LEFT JOIN  " . implode( ' LEFT JOIN ', $join) : '' )
				. ( count( $where ) ? "\n WHERE " . implode( ' AND ', $where ) : '' );
			$db->setQuery( $query);
			try {
				$total = $db->loadResult();
			}
			catch (RuntimeException $e)
			{
				$error = $e->getMessage();
				//echo $error;
				$total = 0;
			}

			if( $limit > $total ) {
				$limitstart = 0;
			}

			$filter_order		= $mainframe->getUserStateFromRequest( $option.'sess_filter_order',		'filter_order',		'rpt.startrepeat',	'cmd' );
			if ($filter_order=='det.dtstart' || $filter_order=='attendee' || strpos($filter_order, 'atdees.')) $filter_order='rpt.startrepeat';
			$filter_order_Dir	= $mainframe->getUserStateFromRequest( $option.'sess_filter_order_Dir',	'filter_order_Dir',	' ASC',				'word' );
			$orderby 	= ' ORDER BY '.$filter_order.' '.$filter_order_Dir;

			$query = "SELECT ev.ev_id as evd,
				CASE atd.allrepeats WHEN 1 THEN -1 * ev.ev_id ELSE rpt.rp_id END AS specialid,
				CASE atd.allrepeats WHEN 1 THEN 0 ELSE rpt.rp_id END AS repeat_id,
				CASE atd.allinvites WHEN 1 THEN 0 ELSE rpt.rp_id END AS inviterepeat_id,
			    ev.state as evstate, det.*, atd.* ,atd.id as atd_id, atdc.atdcount, atdc.gucount,
				count(distinct inv.id) as invcount,
				tmpl.id as templateid, tmpl.title as templatetitle, tmpl.global as globaltemplate, tmpl.created_by as templatecreatedby, tmpl.istemplate,
				sum(atdees.atdcount) as waitingcount,
				rpt.startrepeat as startdate"
				. "\n FROM #__jevents_vevent as ev "
				. "\n LEFT JOIN #__jevents_repetition as rpt ON rpt.eventid = ev.ev_id"
				. "\n LEFT JOIN #__jevents_exception as exc ON exc.rp_id = rpt.rp_id"
				. "\n LEFT JOIN #__jevents_vevdetail as det ON ev.detail_id = det.evdet_id"
				. "\n LEFT JOIN #__jev_attendance AS atd ON atd.ev_id = ev.ev_id"
				. "\n LEFT JOIN #__jev_rsvp_templates AS tmpl ON tmpl.id = atd.template"
				// Note we use the CASE defined repeat_id value here
				. "\n LEFT JOIN #__jev_invitees AS inv ON atd.id = inv.at_id  AND ((inv.rp_id = 0 AND atd.allrepeats = 1) OR (inv.rp_id = rpt.rp_id AND atd.allrepeats = 0)) "
				. "\n LEFT JOIN #__jev_attendeecount AS atdc ON atd.id = atdc.at_id AND  ((atdc.rp_id = 0 AND atd.allrepeats = 1) OR (atdc.rp_id = rpt.rp_id AND atd.allrepeats = 0))"
				. "\n LEFT JOIN #__jev_attendees AS atdees ON atdees.at_id = atd.id AND  ((atdees.rp_id = 0 AND atd.allrepeats = 1) OR (atdees.rp_id = rpt.rp_id AND atd.allrepeats = 0))"
				. ( count( $join) ? "\n LEFT JOIN  " . implode( ' LEFT JOIN ', $join) : '' )
				. ( count( $where ) ? "\n WHERE " . implode( ' AND ', $where ) : '' )
				. "\n GROUP BY specialid $orderby"
				//. "\n GROUP BY rpt.rp_id $orderby"
			;

			if ($limit>0){
				$query .= "\n LIMIT $limitstart, $limit";
			}
			$db->setQuery( $query );

			try {
				$rows = $db->loadObjectList();
			}
			catch (RuntimeException $e)
			{
				$error = $e->getMessage();
				echo $error;
				$rows = array();
			}
//			var_dump($rows);

			foreach ($rows as $key=>$val) {
				// Must refetch the waiting attendee count because its messed up when invitees are also present!
				$query = "SELECT sum(atdees.atdcount) as waitingcount "
					. "\n FROM #__jev_attendees AS atdees WHERE atdees.at_id = $val->atd_id AND atdees.rp_id=$val->repeat_id AND atdees.waiting=1";
				$db->setQuery( $query );

				try {
					$waitingcount = $db->loadResult();
				}
				catch (RuntimeException $e)
				{
					$error = $e->getMessage();
					echo $error;
					$waitingcount = 0;
				}

				if (!is_null($waitingcount)) {
					$rows[$key]->waitingcount = $waitingcount;
				}
				else {
					$rows[$key]->waitingcount = 0;
				}

				// set state variable to the event value not the event detail value
				$rows[$key]->state = $rows[$key]->evstate;

				// Find the first repeat
				if ($val->repeat_id > 0)
				{
					$vevent = $this->dataModel->queryModel->listEventsById( $val->repeat_id, false, "icaldb");
				}
				else
				{
					$vevent = $this->dataModel->queryModel->getEventById(  $val->ev_id, false, "icaldb");
				}
				// trap orphans!
				if (is_null($vevent)) {
					unset($rows[$key]);
					continue;
				}

				if ($val->repeat_id > 0)
				{
					$repeat = $vevent;
				}
				else
				{
					$repeat = $vevent->getFirstRepeat();
				}

				$rows[$key]->repeat = $repeat;
				$rows[$key]->starttime = $repeat->getUnixStartTime();
			}
			if ($total>count($rows)  && $limit>$total){
				$total = count($rows);
			}


		}
		// first the whole event sessions
		else if ($nonrepeating){
			if ($hidepast){
				if (class_exists("JevDate")) {
					$datenow = JevDate::getDate("-1 day");
				}
				else {
					$datenow = Factory::getDate("-1 day");
				}
				$where[] = "\n (det.dtstart>'".$datenow->toUnix()."' OR atd.regclose>".$db->quote($datenow->toSql()).")";
			}


            if ($rsvpparams->get("attendance",0) && $rsvpparams->get("invites",0)){
                $where[] = "((atd.allrepeats=1 and atd.allowregistration>0 ) OR (atd.invites=1 AND atd.allinvites=1))";
            }
            else if ($rsvpparams->get("invites",0)){
                $where[] = "(atd.invites=1 AND atd.allinvites=1)";
            }
            else if ($rsvpparams->get("attendance",0)){
                $where[] = "(atd.allrepeats=1 and atd.allowregistration>0 )";
            }
            else {
                $where[] = '0';
            }

			// not trashed
			$where[] = " ev.state >=0 ";

			$filter_order		= $mainframe->getUserStateFromRequest( 'rsvppro_sessions.sess_filter_order',		'filter_order',		'det.dtstart',	'cmd' );
			if ($filter_order=='rpt.startrepeat' || $filter_order=='attendee' || strpos($filter_order, 'atdees.')) $filter_order='det.dtstart';
			$filter_order_Dir	= $mainframe->getUserStateFromRequest( 'rsvppro_sessions.sess_filter_order_Dir',	'filter_order_Dir',	' ASC',				'word' );
			$orderby 	= ' ORDER BY '.$filter_order.' '.$filter_order_Dir;

			$query = "SELECT count(distinct ev.ev_id)"
			. "\n FROM #__jevents_vevent AS ev "
			. "\n LEFT JOIN #__jevents_repetition as rpt ON rpt.eventid=ev.ev_id"
			. "\n LEFT JOIN #__jevents_vevdetail as det ON ev.detail_id=det.evdet_id"
			. "\n LEFT JOIN #__jev_attendance AS atd ON atd.ev_id = ev.ev_id"
			. "\n LEFT JOIN #__jev_attendees AS atdees ON atdees.at_id = atd.id AND atdees.rp_id=0"
			. "\n LEFT JOIN #__jev_invitees AS inv ON atd.id = inv.at_id  AND inv.rp_id=0"
			. ( count( $join) ? "\n LEFT JOIN  " . implode( ' LEFT JOIN ', $join) : '' )
			. ( count( $where ) ? "\n WHERE " . implode( ' AND ', $where ) : '' )
			;
			$db->setQuery( $query);
			//echo $db->_sql;
			try {
				$total = $db->loadResult();
			}
			catch (RuntimeException $e)
			{
				$error = $e->getMessage();
				$total = 0;
				echo $error;
			}

			if( $limit > $total ) {
				$limitstart = 0;
			}

			// WE CAN'T GET INIVTEE COUNT and waiting count at the same time !!!
			$query = "SELECT ev.state as evstate, det.*, atd.* ,atd.id as atd_id, atdc.atdcount, atdc.unapproved, atdc.unconfirmed, atdc.gucount,
				CASE atd.allrepeats WHEN 1 THEN -1 * ev.ev_id ELSE rpt.rp_id END AS specialid,
				CASE atd.allrepeats WHEN 1 THEN 0 ELSE rpt.rp_id END AS repeat_id,
				CASE atd.allinvites WHEN 1 THEN 0 ELSE rpt.rp_id END AS inviterepeat_id,
				count(distinct inv.id) as invcount,
				tmpl.id as templateid, tmpl.title as templatetitle, tmpl.global as globaltemplate, tmpl.created_by as templatecreatedby, tmpl.istemplate,
				sum(atdees.atdcount) as waitingcount,
				det.dtstart as startdate"
			. ", 0 as rp_id "
			. "\n FROM #__jevents_vevent as ev "
			. "\n LEFT JOIN #__jevents_repetition as rpt ON rpt.eventid=ev.ev_id"
			. "\n LEFT JOIN #__jevents_vevdetail as det ON ev.detail_id=det.evdet_id"
			. "\n LEFT JOIN #__jev_attendance AS atd ON atd.ev_id = ev.ev_id"
			. "\n LEFT JOIN #__jev_rsvp_templates AS tmpl ON tmpl.id = atd.template"
			. "\n LEFT JOIN #__jev_attendees AS atdees ON atdees.at_id = atd.id AND atdees.rp_id=0"
			. "\n LEFT JOIN #__jev_invitees AS inv ON atd.id = inv.at_id  AND inv.rp_id=0"
			. "\n LEFT JOIN #__jev_attendeecount AS atdc ON atd.id = atdc.at_id AND atdc.rp_id=0"
			. ( count( $join) ? "\n LEFT JOIN  " . implode( ' LEFT JOIN ', $join) : '' )
			. ( count( $where ) ? "\n WHERE " . implode( ' AND ', $where ) : '' )
			. "\n GROUP BY ev.ev_id $orderby"
			;

			if ($limit>0){
				$query .= "\n LIMIT $limitstart, $limit";
			}
			$db->setQuery( $query );

			try {
				$rows = $db->loadObjectList();
			}
			catch (RuntimeException $e)
			{
				$error = $e->getMessage();
				echo $error;
				$rows = array();
			}

			foreach ($rows as $key=>$val) {

				// first of all clean up any unpaid and unconfirmed attendees
				$this->attendeehelper->cleanupUnpaidAttendees($val->atd_id);

				// Must refetch the waiting attendee count because its messed up when invitees are also present!
				//$query = "SELECT count(atdees.waiting) as waitingcount "
				$query = "SELECT sum(atdees.atdcount) as waitingcount "
				. "\n FROM #__jev_attendees AS atdees WHERE atdees.at_id = $val->atd_id AND atdees.rp_id=0 AND atdees.waiting=1"
				;
				$db->setQuery( $query );

				try {
					$waitingcount = $db->loadResult();
				}
				catch (RuntimeException $e)
				{
					$error = $e->getMessage();
					echo $error;
					$waitingcount = 0;
				}

				if (!is_null($waitingcount)) {
					$rows[$key]->waitingcount = $waitingcount;
				}
				else {
					$rows[$key]->waitingcount = 0;
				}

				// set state variable to the event value not the event detail value
				$rows[$key]->state = $rows[$key]->evstate;

				// Find the first repeat
				$vevent = $this->dataModel->queryModel->getEventById(  $val->ev_id, true, "icaldb");
				// trap orphans!
				if (is_null($vevent)) {
					unset($rows[$key]);
					continue;
				}
				$repeat = $vevent->getFirstRepeat();

				$rows[$key]->repeat = $repeat;
				$rows[$key]->starttime = $repeat->getUnixStartTime();
			}

		}
		else {
			if ($hidepast){
				if (class_exists("JevDate")) {
					$datenow = JevDate::getDate("-1 day");
					$where[] = "\n rpt.endrepeat>'".$datenow->toMySQL()."'";
				}
				else {
					$datenow = Factory::getDate("-1 day");
					$where[] = "\n rpt.endrepeat>'".$datenow->toSql()."'";
				}
			}

			//$where[] = "(atd.allrepeats=0 OR atd.allinvites=0) AND (atd.allowregistration>0 OR atd.invites=1)";
                        if ($rsvpparams->get("attendance",0) && $rsvpparams->get("invites",0)){
        			$where[] = "(atd.allrepeats=0 OR atd.allinvites=0) AND (atd.allowregistration>0 OR atd.invites=1)";
                        }
                        else if ($rsvpparams->get("invites",0)){
                            $where[] = "atd.allinvites=0 AND atd.invites=1";
                        }
                        else if ($rsvpparams->get("attendance",0)){
                            $where[] = "atd.allrepeats=0 AND atd.allowregistration>0";
                        }
                        else {
                            $where[] = '0';
                        }


			// not trashed
			$where[] = " ev.state >=0 ";

			$filter_order		= $mainframe->getUserStateFromRequest( 'rsvppro_sessions_rpt.sess_filter_order',		'filter_order',		'rpt.startrepeat',	'cmd' );
			if ($filter_order=='det.dtstart' || $filter_order=='attendee' || strpos($filter_order, 'atdees.')) $filter_order='rpt.startrepeat';
			$filter_order_Dir	= $mainframe->getUserStateFromRequest( 'rsvppro_sessions_rpt.sess_filter_order_Dir',	'filter_order_Dir',	' ASC',				'word' );
			$orderby 	= ' ORDER BY '.$filter_order.' '.$filter_order_Dir;

			$query = "SELECT count(distinct rpt.rp_id)"
			. "\n FROM #__jevents_vevent AS ev "
			. "\n LEFT JOIN #__jevents_repetition as rpt ON rpt.eventid=ev.ev_id"
			. "\n LEFT JOIN #__jevents_vevdetail as det ON ev.detail_id=det.evdet_id"
			. "\n LEFT JOIN #__jev_attendance AS atd ON atd.ev_id = ev.ev_id"
			. "\n LEFT JOIN #__jev_invitees AS inv ON atd.id = inv.at_id  AND inv.rp_id=rpt.rp_id"
			. "\n LEFT JOIN #__jev_attendees AS atdees ON atdees.at_id = atd.id AND atdees.rp_id=rpt.rp_id"
			. ( count( $join) ? "\n LEFT JOIN  " . implode( ' LEFT JOIN ', $join) : '' )
			. ( count( $where ) ? "\n WHERE " . implode( ' AND ', $where ) : '' )
			;
			$db->setQuery( $query);

			try {
				$total = $db->loadResult();
			}
			catch (RuntimeException $e)
			{
				$error = $e->getMessage();
				echo $error;
				$total = 0;
			}

			if( $limit > $total ) {
				$limitstart = 0;
			}

			$query = "SELECT rpt.rp_id,  ev.state as evstate, det.*, atd.* ,atd.id as atd_id, atdc.atdcount, atdc.unapproved, atdc.unconfirmed, atdc.gucount, "
					. "CASE atd.allrepeats WHEN 1 THEN -1 * ev.ev_id ELSE rpt.rp_id END AS specialid, "
					. "CASE atd.allrepeats WHEN 1 THEN 0 ELSE rpt.rp_id END AS repeat_id, "
					. "CASE atd.allinvites WHEN 1 THEN 0 ELSE rpt.rp_id END AS inviterepeat_id, "
					. "count(distinct inv.id) as invcount, "
					. "tmpl.id as templateid, tmpl.title as templatetitle, tmpl.global as globaltemplate, tmpl.created_by as templatecreatedby, tmpl.istemplate, "
					. "sum(atdees.atdcount) as waitingcount,  rpt.startrepeat as startdate"
			. ", rpt.rp_id "
			. "\n FROM #__jevents_vevent as ev "
			. "\n LEFT JOIN #__jevents_repetition as rpt ON rpt.eventid=ev.ev_id"
			. "\n LEFT JOIN #__jevents_vevdetail as det ON ev.detail_id=det.evdet_id"
			. "\n LEFT JOIN #__jev_attendance AS atd ON atd.ev_id = ev.ev_id"
			. "\n LEFT JOIN #__jev_rsvp_templates AS tmpl ON tmpl.id = atd.template"
			. "\n LEFT JOIN #__jev_invitees AS inv ON atd.id = inv.at_id  AND inv.rp_id=rpt.rp_id"
			. "\n LEFT JOIN #__jev_attendeecount AS atdc ON atd.id = atdc.at_id AND atdc.rp_id=rpt.rp_id"
			. "\n LEFT JOIN #__jev_attendees AS atdees ON atdees.at_id = atd.id AND atdees.rp_id=rpt.rp_id"
			. ( count( $join) ? "\n LEFT JOIN  " . implode( ' LEFT JOIN ', $join) : '' )
			. ( count( $where ) ? "\n WHERE " . implode( ' AND ', $where ) : '' )
			. "\n GROUP BY rpt.rp_id $orderby"
			;

			if ($limit>0){
				$query .= "\n LIMIT $limitstart, $limit";
			}
			$db->setQuery( $query );

			try {
				$rows = $db->loadObjectList();
			}
			catch (RuntimeException $e)
			{
				$error = $e->getMessage();
				echo $error;
				$rows = array();
			}


			foreach ($rows as $key=>$val) {

				// first of all clean up any unpaid and unconfirmed attendees
				$this->attendeehelper->cleanupUnpaidAttendees($val->atd_id);

				// Must refetch the waiting attendee count because its messed up when invitees are also present!
				$query = "SELECT sum(atdees.atdcount) as waitingcount "
				. "\n FROM #__jev_attendees AS atdees WHERE atdees.at_id = $val->atd_id AND atdees.rp_id=$val->rp_id AND atdees.waiting=1"
				;
				$db->setQuery( $query );

				try {
					$waitingcount = $db->loadResult();
				}
				catch (RuntimeException $e)
				{
					$error = $e->getMessage();
					echo $error;
					$waitingcount = 0;
				}

				if (!is_null($waitingcount)) {
					$rows[$key]->waitingcount = $waitingcount;
				}
				else {
					$rows[$key]->waitingcount = 0;
				}

				// set state variable to the event value not the event detail value
				$rows[$key]->state = $rows[$key]->evstate;

				// Find the first repeat
				$repeat = $this->dataModel->queryModel->listEventsById(  $val->rp_id, true, "icaldb");
				// trap orphans!
				if (is_null($repeat)) {
					unset($rows[$key]);
					continue;
				}

				$rows[$key]->repeat = $repeat;
				$rows[$key]->starttime = $repeat->getUnixStartTime();
			}

		}

		// reset for any missing orphan rows
		$rows = array_values($rows);

		// get list of creators
		$sql = "SELECT distinct u.id, u.* FROM #__jevents_vevent as jev LEFT JOIN #__users as u on u.id=jev.created_by";
		$db= Factory::getDBO();
		$db->setQuery( $sql );
		$users = $db->loadObjectList();
		$userOptions = array();
		$userOptions[] = HTMLHelper::_('select.option', 0, Text::_("JEV_EVENT_CREATOR") );
		foreach( $users as $user )
		{
			$userOptions[] = HTMLHelper::_('select.option', $user->id, $user->name. " ($user->username)" );
		}
		$userlist = HTMLHelper::_('select.genericlist', $userOptions, 'created_by', 'class="inputbox" size="1"  onchange="document.adminForm.submit();"', 'value', 'text', $created_by);

		// get list of templates
		$user = Factory::getUser();
		$sql = "SELECT * FROM #__jev_rsvp_templates WHERE global = 1 OR created_by = " . $user->id . " ORDER BY title ASC";
		$db= Factory::getDBO();
		$db->setQuery( $sql );
		$templates = $db->loadObjectList();

		$templateOptions = array();
		$templateOptions[] = HTMLHelper::_('select.option', 0, Text::_("RSVP_SESSION_TEMPLATE") );
		foreach( $templates as $template )
		{
			$templateOptions[] = HTMLHelper::_('select.option', $template->id, $template->title );
		}
		$searchtemplates = HTMLHelper::_('select.genericlist', $templateOptions, 'searchtemplates', 'class="inputbox" size="1"  onchange="document.adminForm.submit();"', 'value', 'text', $searchtemplates);

		$options = array();
		$options[] = HTMLHelper::_('select.option', '-1', Text::_('RSVP_ALL_EVENTS'));
		$options[] = HTMLHelper::_('select.option', '0', Text::_('RSVP_SEPARATE_REPEATS'));
		$options[] = HTMLHelper::_('select.option', '1', Text::_('RSVP_SINGLE_EVENTS'));
		$repeattypelist = HTMLHelper::_('select.genericlist', $options, 'repeating', 'class="inputbox" size="1" onchange="document.adminForm.submit();"', 'value', 'text', $nonrepeating );

		$options = array();
		$options[] = HTMLHelper::_('select.option', '0', Text::_('RSVP_NO'));
		$options[] = HTMLHelper::_('select.option', '1', Text::_('RSVP_YES'));
		$hidepast = HTMLHelper::_('select.genericlist', $options, 'hidepast', 'class="inputbox" size="1" onchange="document.adminForm.submit();"', 'value', 'text', $hidepast );

		// get list of categories
		$attribs = 'class="inputbox" size="1" onchange="document.adminForm.submit();"';
		$showUnpublishedCategories=false;
		$clist = JEventsHTML::buildCategorySelect($catid, $attribs, null, $showUnpublishedCategories, false, $catidtop, "catid",'com_jevents', false,'ordering',false,false,true);

		jimport('joomla.html.pagination');
		$pageNav = new Pagination( $total, $limitstart, $limit  );

		// Set the layout
		$this->view->setLayout('overview');

		$this->view->rows = $rows;
		$this->view->after = $after;
		$this->view->order = $filter_order;
		$this->view->orderdir = $filter_order_Dir;
		$this->view->userlist = $userlist;
		$this->view->catlist = $clist;
		$this->view->hidepast = $hidepast;
		$this->view->nonrepeating = $nonrepeating;
		$this->view->repeattypelist = $repeattypelist;
		$this->view->search = $search;
		$this->view->searchattendees = $searchattendees;
		$this->view->searchtemplates = $searchtemplates;
		$this->view->searchdidattend = $searchdidattend;
		$this->view->searchinvitees = $searchinvitees;
		$this->view->pageNav = $pageNav;

		$this->view->display();
	}

	function close(){
		ob_end_clean();
		?>
		<script type="text/javascript">
			try {
				window.parent.jQuery('.jevmodal').modal('hide');
				//window.parent.jQuery('#translationPopup').modal('hide');
			}
			catch (e){}
			try {
				window.parent.SqueezeBox.close();
			}
			catch (e){}
			try {
				window.parent.closedialog();
			}
			catch (e){}
		</script>
		<?php
		exit();
	}

	function delete(){
		echo "Not implemented yet";return;
	}


	function exportattendees()
		{
			// get the view
			$this->view = $this->getView("attendees", "html");

			$mainframe = Factory::getApplication();
			$db = Factory::getDBO();

			$atd_id = Factory::getApplication()->input->get("atd_id", array(), 'ARRAY');
			if (!isset($atd_id[0]) || strpos($atd_id[0], "|") === false)
			{
				throw new Exception( Text::_("RSVP_MISSING_ATDID"), 403);
			}
			list($atd_id, $rp_id) = explode("|", $atd_id[0]);

			$atd_id = intval($atd_id);
			$rp_id = intval($rp_id);

			$repeating = Factory::getApplication()->input->getInt('repeating', 0);

			$where = array();
			$join = array();

			$where[] = "ev.ev_id IS NOT NULL";
			$where[] = "atd.id = $atd_id";
			$where[] = "atdees.rp_id = $rp_id";

			$cid = Factory::getApplication()->input->get("cid", array(0), 'ARRAY');
			$cid = ArrayHelper::toInteger($cid);
			if (count($cid) > 0 && $cid[0] != 0)
			{
				$where[] = "atdees.id in (" . implode(",", $cid) . ")";
			}

			$filter_order = $mainframe->getUserStateFromRequest('rsvppro_attendees.atd_filter_order', 'filter_order', 'atd.id', 'cmd');
			$filter_order_Dir = $mainframe->getUserStateFromRequest('rsvppro_attendees.atd_filter_order_Dir', 'filter_order_Dir', ' ASC', 'word');
			$orderby = ' ORDER BY ' . $filter_order . ' ' . $filter_order_Dir;

			$filter_waiting = $mainframe->getUserStateFromRequest('rsvppro_attendees.atd_filter_waiting', 'filter_waiting', -1, 'int');
			if ($filter_waiting >= 0)
			{
				$where[] = "atdees.waiting =" . $filter_waiting;
			}

			$filter_confirmed = $mainframe->getUserStateFromRequest('rsvppro_attendees.atd_filter_confirmed', 'filter_confirmed', -1, 'int');
			if ($filter_confirmed >= 0)
			{
				$where[] = "atdees.confirmed =" . $filter_confirmed;
			}

			$search = $mainframe->getUserStateFromRequest("search{" . RSVP_COM_COMPONENT . "}", 'search', '');
			$search = $db->escape(trim(strtolower($search)));

			if ($search)
			{
				$where[] = "(atdees.email_address LIKE '%$search%' OR ju.username LIKE '%$search%' OR ju.email LIKE '%$search%' )";
			}

			if ($repeating)
			{
				$where[] = "atd.allrepeats=1 AND atd.allowregistration>0";
			}
			else
			{
				$where[] = "atd.allrepeats=0  AND atd.allowregistration>0 ";
			}

			$query = "SELECT det.*, atd.* , atd.id as atd_id, atdc.atdcount, atdc.unapproved, atdc.unconfirmed, atdees.*,atdees.id as atdee_id,  "
				. " CASE WHEN atdees.user_id=0 THEN atdees.email_address ELSE CONCAT_WS(' - ',ju.username,ju.email) END as attendee, "
				. " CASE WHEN atdees.user_id=0 THEN '' ELSE ju.name END as attendeename, "
				. " CASE WHEN atdees.user_id=0 THEN '' ELSE ju.username END as attendeeusername, "
				. " CASE WHEN atdees.user_id=0 THEN atdees.email_address ELSE ju.email END as attendeemail "
				. "\n FROM #__jevents_vevent as ev "
				. "\n LEFT JOIN #__jevents_vevdetail as det ON ev.detail_id=det.evdet_id"
				. "\n LEFT JOIN #__jev_attendance AS atd ON atd.ev_id = ev.ev_id"
				. "\n LEFT JOIN #__jev_attendeecount AS atdc ON atd.id = atdc.at_id"
				. "\n LEFT JOIN #__jev_attendees AS atdees ON atdees.at_id = atd.id"
				. "\n LEFT JOIN #__users AS ju ON ju.id = atdees.user_id"
				. ( count($join) ? "\n LEFT JOIN  " . implode(' LEFT JOIN ', $join) : '' )
				. ( count($where) ? "\n WHERE " . implode(' AND ', $where) : '' )
				. "\n GROUP BY atdees.id $orderby"
			;
			/*
			  if ($limit>0){
			  $query .= "\n LIMIT $limitstart, $limit";
			  }
			 */
			$db->setQuery($query);

			try {
				$rows = $db->loadObjectList();
			}
			catch (RuntimeException $e)
			{
				$error = $e->getMessage();
				echo $error;
				$rows = array();
			}

			if (count($rows) > 0)
			{

				$row = $rows[0];

				$db = Factory::getDBO();
				$sql = "SELECT * FROM #__jev_attendance WHERE id=" . $row->at_id;
				$db->setQuery($sql);
				$rsvpdata = $db->loadObject();
			}
			else
			{
				$rsvpdata = false;
				return;
			}

			$vevent = $this->dataModel->queryModel->getEventById($rsvpdata->ev_id, true, "icaldb");
			if ($rp_id == 0)
			{
				$repeat = $vevent->getFirstRepeat();
			}
			else
			{
				list($year, $month, $day) = JEVHelper::getYMD();
				// include unpublished events
				$repeat = $this->dataModel->queryModel->listEventsById(intval($rp_id), true, "icaldb");
				//$repeatdata = $this->dataModel->getEventData(intval($rp_id), "icaldb", $year, $month, $day);
				//if ($repeatdata && isset($repeatdata["row"]))
				//	$repeat = $repeatdata["row"];
			}

			$this->authoriseTransactions($repeat, false);

			// Store details in registry - will need them for waiting lists!
			$registry = JevRegistry::getinstance("jevents");
			$registry->set("rsvpdata", $rsvpdata);
			$registry->set("event", $repeat);


			// attach repeat into row
			foreach ($rows as $key => $val)
			{
				$rows[$key]->eventrepeat = $repeat;
				// Make sure guest count is up to date
				//$this->analyseGuests($rows[$key], $rsvpdata, $repeat);  // not for exports
			}
			// update the count to reflect the guest counts!
			//$this->countAttendees($atd_id);  // not for exports
			// Set the layout
			$this->view->setLayout('export');

			$this->view->rsvpdata = $rsvpdata;
			$this->view->atd_id = $atd_id;
			$this->view->rp_id = $rp_id;
			$this->view->items = $rows;
			$this->view->order = $filter_order;
			$this->view->orderdir = $filter_order_Dir;
			$this->view->repeat = $repeat;

			$this->view->display();
		}
}
