<?php
/**
 * JEvents Component for Joomla 1.5.x
 *
 * @version     $Id$
 * @package     JEvents
 * @copyright   Copyright (C) 2008-2021 GWE Systems Ltd
 * @license     GNU/GPLv2, see http://www.gnu.org/licenses/gpl-2.0.html
 * @link        https://www.jevents.net
 */
// Check to ensure this file is within the rest of the framework
defined('JPATH_BASE') or die();

use Joomla\Utilities\ArrayHelper;
use Joomla\Registry\Registry;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Factory;
use Joomla\CMS\HTML\HTMLHelper;

include_once(JPATH_ADMINISTRATOR ."/components/com_rsvppro/fields/jevrlist.php");
$language = Factory::getLanguage();
$language->load('com_rsvppro', JPATH_ADMINISTRATOR);

class JFormFieldJevrlistminfirst extends JFormFieldJevrlist
{

	/**
	 * Element name
	 *
	 * @access	protectedf
	 * @var		string
	 */
	var $_name = 'jevrlistminfirst';
	const name = 'jevrlistminfirst';
	const advancedoptions = false;
	const advancedoptions2 = false;
	protected static $overridenPrice;

	public static function fieldName()
	{
		return "Select List with minimum first order size";
	}

	public static function loadScript($field=false)
	{
		$comparams = ComponentHelper::getParams("com_rsvppro");

		HTMLHelper::script('administrator/components/' . RSVP_COM_COMPONENT . '/fields/js/jevrist.js');

		if ($field)
		{
			$id = 'field' . $field->field_id;
		}
		else
		{
			$id = '###';
		}
		$hasfeeClass = "rsvp_nofees";
		ob_start();
		?>
		<div class='rsvpfieldinput'>

			<div class="gsl-form-label"><?php echo Text::_("RSVP_FIELD_TYPE"); ?></div>
			<div class="gsl-form-controls gsl-form-small" style="font-weight:bold;">List field with minimum first order<?php RsvpHelper::fieldId($id); ?></div>
			<div class="rsvpclear"></div>

			<?php
			RsvpHelper::hidden($id, $field, self::name);
			RsvpHelper::label($id, $field, self::name);
			RsvpHelper::tooltip($id, $field);

			if ($field)
			{
				try {
					$params = json_decode($field->params);
				}
				catch (Exception $e) {
					$params = array();
				}
			}
			$includeintotalcapacity = isset($params->includeintotalcapacity) ? intval($params->includeintotalcapacity) : 0;
			$capacity = isset($params->capacity) ? intval($params->capacity) : 0;
			$nocapacitymessage = isset($params->nocapacitymessage) ? $params->nocapacitymessage : "";
			$reducevaluefortotalcapacity = isset($params->reducevaluefortotalcapacity) ? intval($params->reducevaluefortotalcapacity) : 0;
			$minfirstorder = isset($params->minfirstorder) ? intval($params->minfirstorder) : 0;
			?>

			<div class="gsl-form-label">Minimum First Order Size</div>
			<div class="gsl-form-controls">
				<input type="text" name="params[<?php echo $id; ?>][minfirstorder]" id="params<?php echo $id; ?>minfirstorder" size="6"   value="<?php echo $minfirstorder; ?>" />
			</div>
			<div class="rsvpclear"></div>

			<div class="gsl-form-label"><?php echo Text::_("RSVP_OPTIONS"); ?></div>
			<div class="gsl-form-controls">
				<?php echo Text::_("RSVP_NUMERIC_OPTION_NOTES"); ?><br/>
				<!-- Put the selected option here //-->
				<input type="hidden" name="dv[<?php echo $id; ?>]" id="dv<?php echo $id; ?>" value="<?php echo $field ? $field->defaultvalue : ""; ?>" />
				<?php
				$options = array();
				if ($field && $field->options != "")
				{
					$optionvalues = json_decode($field->options);
				}
				$maxvalue = -1;

				$countoptions = 0;
				if (isset($optionvalues))
				{
					foreach ($optionvalues->value as $val)
					{
						$maxvalue = $maxvalue > $val ? $maxvalue : $val;
					}

					foreach ($optionvalues->label as $lab)
					{
						if ($lab == "")
						{
							break;
						}
						$option = new stdClass();
						$option->value = $optionvalues->value[$countoptions];
						$option->price = isset($optionvalues->price) ? $optionvalues->price[$countoptions] : "0.0";
						// make sure HTML entities are escaped
						$option->label = htmlspecialchars($lab,ENT_COMPAT | ENT_HTML401, "UTF-8", false);
						$option->capacity = (isset($optionvalues->capacity) && $optionvalues->capacity[$countoptions] != "") ? $optionvalues->capacity[$countoptions] : "";
						$option->waiting = (isset($optionvalues->waiting) && $optionvalues->waiting[$countoptions] != "") ? $optionvalues->waiting[$countoptions] : "";
						$options[] = $option;
						$countoptions++;
					}
				}

				// add 20 blank options at the end
				for ($op = 0; $op < 20; $op++)
				{
					$option = new stdClass();
					$option->value = $maxvalue + 1;
					$option->price = "0.0";
					$maxvalue++;
					$option->label = "";
					$option->capacity = "";
					$option->waiting = "";
					$options[] = $option;
				}
				?>
				<input type="button" value="<?php echo Text::_("RSVP_NEW_OPTION") ?>" onclick="jevrlist.newOption('<?php echo $id; ?>');"/>
				<table id="options<?php echo $id; ?>">
					<tr >
						<th><?php echo Text::_("RSVP_OPTION_TEXT") ?></th>
						<th><?php echo Text::_("RSVP_OPTION_VALUE") ?></th>
						<th class="<?php echo $hasfeeClass; ?>"><?php echo Text::_("RSVP_FEE_VALUE") ?></th>
						<th><?php echo Text::_("RSVP_DEFAULT_VALUE") ?></th>
						<?php if ($comparams->get("allowfieldoptioncapacities", 0))
						{ ?>
							<th><?php echo Text::_("RSVP_OPTION_CAPACITY") ?></th>
						<?php } ?>
						<?php if (self::advancedoptions2)
						{ ?>
							<th><?php echo Text::_("RSVP_OPTION_WAITING") ?></th>
						<?php } ?>
						<th/>
					</tr>
					<?php
					for ($op = 0; $op < count($options); $op++)
					{
						$option = $options[$op];
						$style = "";
						if ($op > 0 && $op >= $countoptions)
						{
							$style = "style='display:none;'";
						}

						$checked = "";
						if (($field && $option->value == $field->defaultvalue) || (!$field && $option->value == ""))
						{
							$checked = "checked='checked'";
						}
						?>
						<tr <?php echo $style; ?> >
							<td>
								<input type="text" class="inputlabel" name="options[<?php echo $id; ?>][label][]" id="options<?php echo $id; ?>_t_<?php echo $op; ?>" value="<?php echo $option->label; ?>" <?php JFormFieldJevrlist::buttonAction($id, $op); ?>/>
							</td>
							<td>
								<input type="text" name="options[<?php echo $id; ?>][value][]" id="options<?php echo $id; ?>_v_<?php echo $op; ?>" value="<?php echo $option->value; ?>" <?php JFormFieldJevrlist::buttonAction($id, $op); ?> class="jevoption_value"/>
							</td>
							<td class="<?php echo $hasfeeClass; ?>">
								<input type="text" name="options[<?php echo $id; ?>][price][]" id="options<?php echo $id; ?>_p_<?php echo $op; ?>" value="<?php echo $option->price; ?>" class="jevfee_value"/>
							</td>
							<td>
								<input type="radio" value="1" onclick="jevrlist.defaultOption(this, '<?php echo $id; ?>', '<?php echo $op; ?>');"  name="default<?php echo $id; ?>" <?php echo $checked; ?> />
							</td>
							<?php if ($comparams->get("allowfieldoptioncapacities", 0))
							{ ?>
								<td>
									<input type="text" name="options[<?php echo $id; ?>][capacity][]" id="options<?php echo $id; ?>_capacity_<?php echo $op; ?>" value="<?php echo $option->capacity; ?>"  class="jevoption_capacity" size ="4"/>
								</td>
							<?php } ?>
							<?php if (self::advancedoptions2)
							{ ?>
								<td>
									<input type="text" name="options[<?php echo $id; ?>][waiting][]" id="options<?php echo $id; ?>_waiting_<?php echo $op; ?>" value="<?php echo $option->waiting; ?>"  class="jevoption_waiting" size ="4"/>
								</td>
							<?php } ?>
							<td>
								<input type="button" value="<?php echo Text::_("RSVP_DELETE_OPTION") ?>" onclick="jevrlist.deleteOption(this);"/>
							</td>
						</tr>
						<?php
					}
					?>
				</table>

			</div>
			<div class="rsvpclear"></div>

			<div class="gsl-form-label"><?php echo Text::_("RSVP_INCLUDE_IN_CAPACITY"); ?></div>
			<div class="gsl-form-controls">
				<label for="includeintotalcapacity1<?php echo $id; ?>"><?php echo Text::_("RSVP_YES"); ?>
				<input type="radio" name="params[<?php echo $id; ?>][includeintotalcapacity]"  id="includeintotalcapacity1<?php echo $id; ?>" value="1" <?php
			if ($includeintotalcapacity == 1)
			{
				echo 'checked="checked"';
			}
					?> />
				</label>
				<label for="includeintotalcapacity0<?php echo $id; ?>"><?php echo Text::_("RSVP_NO"); ?>
				<input type="radio" name="params[<?php echo $id; ?>][includeintotalcapacity]" id="includeintotalcapacity0<?php echo $id; ?>" value="0" <?php
			   if ($includeintotalcapacity == 0)
			   {
				   echo 'checked="checked"';
			   }
					?> />
				</label>
			</div>
			<div class="rsvpclear"></div>

			<div class="gsl-form-label"><?php echo Text::_("RSVP_REDUCE_TOTAL_CAPACITY"); ?></div>
			<div class="gsl-form-controls">
				<input type="text" name="params[<?php echo $id; ?>][reducevaluefortotalcapacity]" id="params<?php echo $id; ?>reducevaluefortotalcapacity" size="15"   value="<?php echo $reducevaluefortotalcapacity; ?>" />
			</div>
			<div class="rsvpclear"></div>

			<div class="gsl-form-label"><?php echo Text::_("RSVP_FIELD_CAPACITY"); ?></div>
			<div class="gsl-form-controls">
				<input type="text" name="params[<?php echo $id; ?>][capacity]" id="params<?php echo $id; ?>capacity" size="15"   value="<?php echo $capacity; ?>" />
			</div>
			<div class="rsvpclear"></div>
			<?php if ($comparams->get("allowfieldoptioncapacities", 0)) {?>
			<div class="gsl-form-label" ><?php echo Text::_("RSVP_NO_CAPACITY_MESSAGE"); ?></div>
			<div class="gsl-form-controls">
				<input type="text" name="params[<?php echo $id; ?>][nocapacitymessage]" id="params<?php echo $id; ?>nocapacitymessage" size="50"   value="<?php echo $nocapacitymessage; ?>" />
			</div>
			<div class="rsvpclear"></div>
			<?php }
			else { ?>
			<div class="gsl-form-label" style="display:none"><?php echo Text::_("RSVP_NO_CAPACITY_MESSAGE"); ?></div>
			<div class="gsl-form-controls" style="display:none">
				<input type="text" name="params[<?php echo $id; ?>][nocapacitymessage]" id="params<?php echo $id; ?>nocapacitymessage" size="50"   value="<?php echo $nocapacitymessage; ?>" />
			</div>
			<div class="rsvpclear"></div>
			<?php } ?>

			<?php
			RsvpHelper::required($id, $field);
			RsvpHelper::requiredMessage($id, $field);
			RsvpHelper::conditional($id, $field);
			RsvpHelper::peruser($id, $field);
			RsvpHelper::formonly($id, $field);
			RsvpHelper::showinform($id, $field);
			RsvpHelper::showindetail($id, $field);
			RsvpHelper::showinlist($id, $field);
			RsvpHelper::showinticket($id, $field);
			RsvpHelper::includeinexport($id,  $field);
			RsvpHelper::allowoverride($id, $field);
			RsvpHelper::accessOptions($id, $field);
			RsvpHelper::applicableCategories("facc[$id]", "facs[$id]", $id, $field ? $field->applicablecategories : "all");
			?>

			<div class="rsvpclear"></div>

		</div>
		<div class='rsvpfieldpreview' id='<?php echo $id; ?>preview'>
			<div class="previewlabel"><?php echo Text::_("RSVP_PREVIEW"); ?></div>
			<div class="gsl-form-label rsvppl" id='pl<?php echo $id; ?>' ><?php echo $field ? $field->label : Text::_("RSVP_FIELD_LABEL"); ?></div>
			<select name="pdv[<?php echo $id; ?>]" id="pdv<?php echo $id; ?>" >
				<?php
				foreach ($options as $option)
				{
					if ($option->label == "")
						continue;
					$selected = "";
					if (($field && $option->value == $field->defaultvalue) || (!$field && $option->value == ""))
					{
						$selected = "selected='selected'";
					}
					?>
				<option value="<?php echo $option->value; ?>" <?php echo $selected; ?> ><?php echo htmlspecialchars_decode($option->label); ?></option>
					<?php
				}
				?>
			</select>

		</div>
		<div class="rsvpclear"></div>
		<?php
		$html = ob_get_clean();

		return RsvpHelper::setField($id, $field, $html, self::name);

	}

	function getInput()
	{
		$name = $this->name;
		$fieldname = $this->fieldname;
		$id = $this->id;
		$value = $this->value;

		$attribs = ( $this->attribute('class') ? 'class="' . $this->attribute('class'). ' xxx"' : 'class="inputbox xxx"' );
		$comparams = ComponentHelper::getParams("com_rsvppro");
		$includeintotalcapacity = intval($this->attribute("includeintotalcapacity"));

		$html = "";
		$hasprice = false;
		$options = array();
		$newoptions = array();
		$prices = array();

		// used for option specific capacities
		$capacities[-1] = 0;
		$waiting[-1] = 0;
		$counts[-1] = 0;
		$allcounts[-1] = 0;
		$optionlabels[-1] = "";
		$storedRows = 0;

		// Now find how much of the capacity is used up
		//
		// capacity count for this specific field
		$fieldcount = 0;
		// for specific options (for future use)
		$allcounts = array();

		if (is_array($value)){
			$registrationSpecificCount = array_sum($value);
		}
		else {
			$registrationSpecificCount =intval($value);
		}
		// if not an attendee the the $registrationSpecificCount  must be zero!
		if (!$this->attendee || (isset($this->event->attendee) && !$this->event->attendee ) || (is_object($this->attendee)) && $this->attendee->id==0) {
			$registrationSpecificCount = 0;
		}

		// Fetch reference to current row and rsvpdata to the registry so that we have access to these in the fields
		$registry = JevRegistry::getinstance("jevents");
		$rsvpdata = $registry->get("rsvpdata");
		$row = $registry->get("event");

		// do we want to include unpaid users in capacity
		$db = Factory::getDBO();
		static $templateParams;
		if (!isset($templateParams)){
			$templateParams = false;
			if ($rsvpdata->template != "")
			{
				$db->setQuery("Select params from #__jev_rsvp_templates where id=" . intval($rsvpdata->template));

				$templateParams = $db->loadObject();
				if ($templateParams)
				{
					$templateParams = json_decode($templateParams->params);
				}
			}
		}
		$includeunpaid = "";
		if ($templateParams && isset($templateParams->unpaidcapacity) && $templateParams->unpaidcapacity)
		{
			$includeunpaid = " OR a.attendstate=4";
		}

		$sql = "SELECT params FROM #__jev_attendees as a WHERE a.at_id=" . $rsvpdata->id . " AND (a.attendstate=1 $includeunpaid ) AND waiting=0";
		if (!$rsvpdata->allrepeats)
		{
			$sql .= " and a.rp_id=" . $row->rp_id();
		}
		$db = Factory::getDBO();
		$db->setQuery($sql);
		$attendeeData = $db->loadObjectList();
		foreach ($attendeeData as $data)
		{
			$dataparams = new Registry($data->params);
			$pvalue = $dataparams->get($fieldname, -1);
			if (!is_array($pvalue))
			{
				$pvalue = array($pvalue);
			}
			$pvalue = ArrayHelper::toInteger($pvalue);

			for ($i = 0; $i < count($pvalue); $i++)
			{
				$pval = $pvalue[$i];
				if (!isset($allcounts[$pval]))
					$allcounts[$pval] = 0;
				$allcounts[$pval] += 1;
				$fieldcount += $pval;
			}
		}

		if ($rsvpdata->capacity){
			if (!isset($rsvpdata->attendeeCount)){
				$db= Factory::getDBO();
				$sql = "SELECT atdcount FROM #__jev_attendeecount as a WHERE a.at_id=".$rsvpdata->id;
				if (!$rsvpdata->allrepeats){
					$sql .= " and a.rp_id=".$row->rp_id();
				}
				$db->setQuery($sql);
				$rsvpdata->attendeeCount = $db->loadResult();
			}
			$attendeeCount = $rsvpdata->attendeeCount;
		}

		$setcapacity = intval($this->attribute("capacity"))==0;

		foreach ($this->element->children() as $option)
		{
			$val	= (string) $option["value"];
			$text = (string)$option;

			// some short strings like seat number e.g. J1=J20 are translated by Joomla to numbers - we don't want this
			if (!is_numeric($text) && is_numeric( Text::_($text))){
				$text =$text;
			}
			else {
				 $text = Text::_($text);
			}
			$htmloption = HTMLHelper::_('select.option', $val,$text);
			$price =  (string) $option['price'];
			if (!is_null($price))
			{
				$htmloption->price = $price;
				$prices[$val] = $price;
				$hasprice = true;
			}
			else
			{
				$prices[$val] = 0;
			}
			$options[] = $htmloption;

			// If the field capacity attribute is blank/set to zero but we have line capacities then do a workaround here
			if ($setcapacity) {
				$this->addAttribute('capacity',$this->attribute("capacity") + intval( $option['capacity'])*intval($val));
			}

			// field specific capacity too high
			if ($this->attribute("capacity")>0 && $fieldcount+intval($htmloption->value)>$this->attribute("capacity")){
				continue;
			}
			// overall capacity check
			if ($rsvpdata->capacity && $this->attribute("includeintotalcapacity") && $attendeeCount+intval($htmloption->value) -$registrationSpecificCount > $rsvpdata->capacity){
				continue;
			}

			// Must buy at least 4 tickets - BUT do not strip these out on accounts payment page
			if ((intval($val) + $attendeeCount < $this->attribute("minfirstorder")) && !Factory::getApplication()->isClient('administrator')	){
				continue;
			}
			$newoptions[] = $htmloption;
		}
		if ($rsvpdata->capacity && (count($newoptions)==0 || (count($newoptions)==1 && $newoptions[0]->value==0))){
			$htmloption = HTMLHelper::_('select.option', 0, ($this->attribute("nocapacitymessage")!="" ? $this->attribute("nocapacitymessage") : Text::_("JEV_NO_MORE_CAPACITY")));
			$newoptions = array();
			$newoptions[] = $htmloption;
		}

		if ($comparams->get("allowfieldoptioncapacities", 0) && $this->attribute("capacity")>0)
		{
			$rsvpdata = $this->rsvpdata;
			$rsvpparams = new Registry($this->rsvpdata->params);

			$currentfieldname = $rsvpparams->get("fieldname_" . $fieldname, $this->attribute('label'));

			foreach ($this->element->children() as $option)
			{
				$p	= intval((string) $option["value"]);
				if ((string) $option["capacity"]=="" && !is_numeric((string) $option["value"])){
					$p = (string) $option["value"];
				}
				$capacities[$p] = intval( $option['capacity']);
				$waiting[$p] = intval($option['waiting']);
				$hascapacity = true;
				$allcounts[$p] = isset($allcounts[$p])?$allcounts[$p]:0;
				$counts[$p] = $allcounts[$p];
				$optionlabels[$p] = (string)$option;
			}

			// Add reference for use else where by other plugins etc
			$this->capacities = $capacities;
			$this->waiting = $waiting;
			$this->allcounts = $allcounts;
			$this->counts = $counts;
			$this->optionlabels = $optionlabels;
		}
		else {
			foreach ($this->element->children() as $option)
			{
				$p = $includeintotalcapacity ? intval($option["value"]):(string) $option["value"];
				$capacities[$p] = 0;
				$waiting[$p] = 0;
				$allcounts[$p] = isset($allcounts[$p])?$allcounts[$p]:0;
				$counts[$p] = $allcounts[$p];
				$optionlabels[$p] = (string)$option;
			}

			// Add reference for use else where by other plugins etc
			$this->capacities = $capacities;
			$this->waiting = $waiting;
			$this->allcounts = $allcounts;
			$this->counts = $counts;
			$this->optionlabels = $optionlabels;
		}

		if ($hasprice)
		{

			$this->hasPrices = count($prices) > 0;
			$this->pricesArray = $prices;
			$this->prices = json_encode($prices);

			$attribs .= " onchange='JevrFees.calculate(document.updateattendance);'";
		}

		$hasSpaceLeft = false;
		$options = array();
		foreach ($optionlabels as $p => $text)
		{
			if ($text!="")
			{
				// some short strings like seat number e.g. J1=J20 are translated by Joomla to numbers - we don't want this
				if (!is_numeric($text) && is_numeric( Text::_($text))){
					$text =$text;
				}
				else {
					 $text = Text::_($text);
				}

				// if (excluding this attendee) we are over capacity then do not offer it!
				if ($capacities[$p] > 0 && ($allcounts[$p] - $counts[$p] ) >= ($capacities[$p] + $waiting[$p]) && (!is_array($value) || !in_array($p, $value)))
				{
					continue;
				}

				// if overall capacity over then skip it too
				if ($rsvpdata->capacity && $this->attribute("includeintotalcapacity") && $attendeeCount+$p -$registrationSpecificCount  > $rsvpdata->capacity){
					continue;
				}

				$hasSpaceLeft = true;

				$spacesleft = 0;
				if ($p >= 0 && $capacities[$p] > 0 && $allcounts[$p] < $capacities[$p])
				{
					$spacesleft = $capacities[$p] - $allcounts[$p];
					$text .= " (" . $spacesleft . ")";
				}
				else if ($p >= 0 && $capacities[$p] > 0 && $allcounts[$p] >= ($capacities[$p] + $waiting[$p]))
				{
					$text .= " (0)";
				}
				else if ($p >= 0 && $capacities[$p] > 0 && $allcounts[$p] >= $capacities[$p] && $waiting[$p] > 0)
				{
					$spacesleft = $capacities[$p] - $allcounts[$p] - (isset($allcounts[$p + 10000]) ? $allcounts[$p + 10000] : 0);
					if ($value[0] != $p)
					{
						$text .= " (Waiting spaces available : " . (-$spacesleft) . " already waiting)";
					}
				}

				// switching option values when into waiting zone to negative values
				if ($p >= 0 && $capacities[$p] > 0 && $allcounts[$p] >= $capacities[$p] && $value[0] != $p && $waiting[$p] > 0)
				{
					$htmloption = HTMLHelper::_('select.option', 10000 + $p, $text);
				}
				else if ($p >= 0 && $capacities[$p] > 0 && $allcounts[$p] >= $capacities[$p] && $value[0] != $p)
				{
					// option is disabled
					$htmloption = HTMLHelper::_('select.option', $p, $text, "value","text", true);
				}
				// this is editing a waiting entry AFTER the list has been released so reset the value!
				else if (is_array($value) && $value[0] == $p + 10000)
				{
					$value[0] = $p;
					$htmloption = HTMLHelper::_('select.option', $p, $text);
				}
				else
				{
					$htmloption = HTMLHelper::_('select.option', $p, $text);
				}

				$options[] = $htmloption;
			}
		}

		if ($this->attribute("peruser") == 1 || $this->attribute("peruser") == 2)
		{
			$this->fixValue($value, $this);

			$elementname =  $name . '[]';
			$html = "";
			$i = 0;
			foreach ($value as $val)
			{
				if ($i == 0)
				{
					if ($this->attribute("peruser") == 2)
					{
						$thisclass = str_replace(" xxx", " disabledfirstparam rsvpparam rsvpparam0 rsvp_$fieldname rsvp_xmlfile_$fieldname", $attribs);
					}
					else
					{
						$thisclass = str_replace(" xxx", " rsvpparam rsvpparam0 rsvp_$fieldname rsvp_xmlfile_$fieldname", $attribs);
					}
				}
				else
				{
					$thisclass = str_replace(" xxx", " rsvpparam rsvpparam$i  rsvp_$fieldname rsvp_xmlfile_$fieldname", $attribs);
				}
				$thisclass .= " id = 'rsvp_" . $fieldname . "_span_$i' ";

				// block any choices that increase current selection over the capacity
				$specificoptions = array();
				foreach ($options as $htmloption){
					if ($this->attribute("capacity")>0 && $fieldcount+intval($htmloption->value)-$val>$this->attribute("capacity") ){
						continue;
					}
					$specificoptions[] = $htmloption;
				}
				if ($rsvpdata->capacity && (count($specificoptions)==0 || (count($specificoptions)==1 && $specificoptions[0]->value==0))){
					$htmloption = HTMLHelper::_('select.option', 0, ($this->attribute("nocapacitymessage")!="" ? $this->attribute("nocapacitymessage") : Text::_("JEV_NO_MORE_CAPACITY")));
					$specificoptions = array();
					$specificoptions[] = $htmloption;
				}

				$html .= HTMLHelper::_('jevrList.genericlist', $specificoptions, $elementname, $thisclass, 'value', 'text', $val, $id. "_" . $i);
				$i++;
			}
			$val = "";
			$val = "#%^£xx£^%#";
			$thisclass = str_replace(" xxx", " paramtmpl rsvp_$fieldname rsvp_xmlfile_$fieldname", $attribs);
			$thisclass .= " id = 'rsvp_" . $fieldname . "_span_xxxyyyzzz' ";
			$html .= HTMLHelper::_('jevrList.genericlist', $newoptions, "paramtmpl_" . $elementname, $thisclass, 'value', 'text', $val, $id . "_xxx");
		}
		else
		{
			// data intgerity check (in case value was an array before a template change removing guests on this field)
			if (is_array($value)){
				$value = current($value);
			}

			$thisclass = str_replace(" xxx", " rsvpparam rsvpparam0 rsvp_$fieldname rsvp_xmlfile_$fieldname", $attribs);
			// per user is 0 so don't put a guest number on the element!
			$thisclass .= " id = 'rsvp_" . $fieldname . "_span_' ";

			// block any choices that increase current selection over the capacity
			$specificoptions = array();
			foreach ($options as $htmloption){
				if ($this->attribute("capacity")>0 && $fieldcount+intval($htmloption->value)-$value>$this->attribute("capacity") ){
					continue;
				}
				$specificoptions[] = $htmloption;
			}
			if ($rsvpdata->capacity && (count($specificoptions)==0 || (count($specificoptions)==1 && $specificoptions[0]->value==0))){
				$htmloption = HTMLHelper::_('select.option', 0, ($this->attribute("nocapacitymessage")!="" ? $this->attribute("nocapacitymessage") : Text::_("JEV_NO_MORE_CAPACITY")));
				$specificoptions = array();
				$specificoptions[] = $htmloption;
			}

			$html = HTMLHelper::_('jevrList.genericlist', $specificoptions, '' . $name , $thisclass, 'value', 'text', $value , $id);
		}
		return $html;

	}

	// Special overridden method to force first attendee to buy 'n' tickets!
	/*
	function toXML($field)
	{
		$registry = JevRegistry::getinstance("jevents");
		$rsvpdata = $registry->get("rsvpdata", false);
		$event = $registry->get("event", false);
		$attendee = $registry->get("attendee", false);

		$attendee = isset($event->attendee) ? $event->attendee : $attendee;
		if (isset($rsvpdata->attendeeCount)){
			$attendeeCount = $rsvpdata->attendeeCount;
		}
		else if ($rsvpdata->capacity && $event){
			$db= Factory::getDBO();
			$sql = "SELECT atdcount FROM #__jev_attendeecount as a WHERE a.at_id=".$rsvpdata->id;
			if (!$rsvpdata->allrepeats){
				$sql .= " and a.rp_id=".$event->rp_id();
			}
			$db->setQuery($sql);
			$rsvpdata->attendeeCount = $db->loadResult();
			$attendeeCount = $rsvpdata->attendeeCount;
		}
		else {
			$attendeeCount = 0;
		}

		$result = array();
		$result[] = "<field ";
		foreach (get_object_vars($field) as $k => $v)
		{
			if ($k=="options" || $k=="html"  || $k=="defaultvalue" || $k=="name") continue;
			if ($k=="field_id") {
				$k="name";
				$v = "field".$v;
			}
			 if ($k == "params")
			{
				   if (is_string($field->params))
				   {
					   $field->params = @json_decode($field->params);
				   }
				   if (is_object($field->params))
				   {
					   foreach (get_object_vars($field->params) as $label=>$value)
					   {
						   $result[] = $label . '="' . addslashes($value) . '" ';
					   }
				   }
				   continue;
			   }

			$result[] = $k . '="' . addslashes(htmlspecialchars($v)) . '" ';
		}
		$result[] = " >";
		if (is_string($field->options))
		{
			$field->options = @json_decode($field->options);
		}
		if (is_object($field->options))
		{
			for ($i = 0; $i < count($field->options->label); $i++)
			{
				if ($field->options->label[$i] == "") {
					break;
				}

				// Must buy at least 4 tickets - BUT do not strip these out on accounts payment page
				//if ((intval($field->options->value[$i]) + $attendeeCount <$field->params->minfirstorder)
				//		&& ! (Factory::getApplication()->input->getCmd("task", "") == "accounts.paymentpage"
				//	|| Factory::getApplication()->input->getCmd("task", "") == "attendees.overview"
				//	|| Factory::getApplication()->input->getCmd("task", "") == "attendees.export"
				//	|| Factory::getApplication()->isClient('administrator'))
				//	){
				//	continue;
				//}

				$result[] = "<option ";
				$result[] = ' value="' . addslashes($field->options->value[$i]) . '"';
				if (isset($field->options->price))
				{
					$result[] = ' price="' . addslashes($field->options->price[$i]) . '"';
				}
				if (isset($field->options->surcharge))
				{
					$result[] = ' surcharge="' . addslashes($field->options->surcharge[$i]) . '"';
				}
				if (isset($field->options->capacity))
				{
					$result[] = ' capacity="' . addslashes($field->options->capacity[$i]) . '"';
				}
				if (isset($field->options->waiting))
				{
					$result[] = ' waiting="' . addslashes($field->options->waiting[$i]) . '"';
				}
				$result[] = ">".addslashes(htmlspecialchars($field->options->label[$i]))."</option>";
				//$result[] = " ><![CDATA[".addslashes($field->options->label[$i])."]]></option>";
			}
		}
		$result[] = " </field>";
		$xml = implode(" ", $result);
		return $xml;

	}
*/

}

class_alias("JFormFieldJevrlistminfirst", "FormFieldJevrlistminfirst");