<?php
/**
 * JEvents Component for Joomla 1.5.x
 *
 * @version     $Id$
 * @package     JEvents
 * @copyright   Copyright (C) 2008-2021 GWE Systems Ltd
 * @license     GNU/GPLv2, see http://www.gnu.org/licenses/gpl-2.0.html
 * @link        https://www.jevents.net
 */
// Check to ensure this file is within the rest of the framework
defined('JPATH_BASE') or die();

use Joomla\CMS\Language\Text;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Session\Session;
use Joomla\CMS\Factory;
use Joomla\CMS\HTML\HTMLHelper;

include_once(JPATH_ADMINISTRATOR ."/components/com_rsvppro/rsvppro.defines.php");
include_once(JPATH_ADMINISTRATOR ."/components/com_rsvppro/fields/JevrField.php");

class JFormFieldJevrecaptcha_v2 extends JevrField
{

	static function isEnabled()
	{
		return ComponentHelper::getParams('com_rsvppro')->get("recaptchav2", 1);
	}

	/**
	 * Element name
	 *
	 * @access	protected
	 * @var		string
	 */
	var $_name = 'jevreaptcha_v2';

	const name = 'jevrecaptcha_v2';

	public static function loadScript($field = false)
	{
		HTMLHelper::script('administrator/components/' . RSVP_COM_COMPONENT . '/fields/js/jevrcaptcha.js');

		if ($field)
		{
			$id = 'field' . $field->field_id;
		}
		else
		{
			$id = '###';
		}
		ob_start();
		?>
		<div class='rsvpfieldinput'>

			<div class="gsl-form-label"><?php echo Text::_("RSVP_FIELD_TYPE"); ?></div>
			<div class="gsl-form-controls gsl-form-small" style="font-weight:bold;"><?php echo Text::_("RSVP_TEMPLATE_TYPE_JEVRECAPTCHA_V2"); ?><?php RsvpHelper::fieldId($id); ?></div>
			<div class="rsvpclear"></div>

			<?php
			RsvpHelper::hidden($id, $field, self::name);
			RsvpHelper::label($id, $field);
			RsvpHelper::tooltip($id, $field);

			RsvpHelper::required($id, $field, 1);
			RsvpHelper::requiredMessage($id, $field);

                        RsvpHelper::className($id, $field);

			RsvpHelper::formonly($id, $field, 1);
			RsvpHelper::showinform($id, $field, 1);

			if ($field)
			{
				try {
					$params = json_decode($field->params);
				}
				catch (Exception $e) {
					$params = array();
				}
			}
			$deposittype = isset($params->deposittype) ? intval($params->deposittype) : 0;
			?>

			<div class="rsvpclear"></div>

			<?php
			RsvpHelper::accessOptions($id, $field);
			RsvpHelper::applicableCategories("facc[$id]", "facs[$id]", $id, $field ? $field->applicablecategories : "all");
			?>

			<div class="rsvpclear"></div>

		</div>
		<div class='rsvpfieldpreview'  id='<?php echo $id; ?>preview'>
			<div class="previewlabel"><?php echo Text::_("RSVP_PREVIEW"); ?></div>
			<div class="gsl-form-label rsvppl" id='pl<?php echo $id; ?>' ><?php echo $field ? $field->label : Text::_("RSVP_FIELD_LABEL"); ?></div>
		</div>
		<div class="rsvpclear"></div>
		<?php
		$html = ob_get_clean();

		return RsvpHelper::setField($id, $field, $html, self::name);

	}

	public static function paidOption()
	{
		return 0;

	}

	function getInput()
	{
		$node =  $this->element;
		$name = $this->name;
		$id = $this->id;
		$value = $this->value;
		$fieldname = $this->fieldname;

		if (Factory::getApplication()->isClient('administrator') || Factory::getApplication()->input->getCmd("task")=="attendees.edit"){
			$this->addAttribute("required",0);
			$this->addAttribute("label","");
			return "";
		}
		if (isset($this->attendee) && $this->attendee && $this->attendee->user_id==0 &&  $this->attendee->confirmed){
			return "";
		}

		$lang = Factory::getLanguage();
		list ($tag1, $tag2) = explode("-", $lang->getTag());
		// See https://developers.google.com/recaptcha/docs/language for list of supported languages
		// $langs = array("en", "nl", "fr", "de", "pt", "ru", "es", "tr");
                // its very long to just take a risk and use the Joomla language code!
                $locallangs = array("zh-CN","zh-TW","zh-HK","en-GB", "fr-CA", "de-AT", "de-CH", "pt-BR", "pt-PT");
		if (in_array($lang->getTag(), $locallangs))
		{
			$recaptchalang = $lang->getTag();
		}
                else {
                    $recaptchalang = $tag1;
                }

		$root = JURI::root()."index.php";
		$token = Session::getFormToken();;

		$checkscript = <<<SCRIPT
var recaptchaurlroot = '$root';
var recaptchatoken = '$token';
var RecaptchaOptions = {
theme : 'clean',
lang : '$recaptchalang'
};
SCRIPT;
		$document = Factory::getDocument();
		$document->addScriptDeclaration($checkscript);

		$params = ComponentHelper::getParams("com_rsvppro");

		// Version 2 API
		HTMLHelper::script("https://www.google.com/recaptcha/api.js");
		$input = '<div class="g-recaptcha" data-sitekey="'.$params->get("recaptchapublic", "").'"></div>';

		// to pass secret validation code to captcha check in the backend
		$input .= "<input type='hidden' name='secretcaptcha'  id='secretcaptcha'  value='' />";

		return $input;

	}

	function fetchRequiredScript()
	{
		if (Factory::getApplication()->isClient('administrator')){
			return "";
		}

		if (isset($this->attendee) && $this->attendee && $this->attendee->user_id==0 &&  $this->attendee->confirmed){
			return "";
		}

		$elementid = $this->id;
		$elementname = $this->name;

		$pluginpath = "plugins/jevents/jevrsvppro/rsvppro/recaptcha/";
		JevHelper::script("recaptcha16_v2.js", $pluginpath, true);

		$mainframe = Factory::getApplication();
		$mainframe->setUserState("jevrecaptcha", "error");

		$script = "jevrsvpRequiredFields.fields.push({'requiredCheckScript':'checkRecaptchav2', 'reqmsg':'" . trim(Text::_($this->attribute('requiredmessage'), true)) . "'}); ";
		return $script;
	}

}
class_alias("JFormFieldJevrecaptcha_v2", "FormFieldJevrecaptcha_v2");
