<?php
/**
 * @version    CVS: 3.7.8
 * @package    com_yoursites
 * @author     Geraint Edwards <yoursites@gwesystems.com>
 * @copyright  2016-2021 GWE Systems Ltd
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

defined('JPATH_BASE') or die;

use Joomla\CMS\Access\Access;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Version;
use Joomla\CMS\Router\Route;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Factory;
use Joomla\CMS\Layout\LayoutHelper;
use Joomla\CMS\Uri\Uri;

class GslHelper
{
	public static function loadAssets()
	{
		$document = Factory::getDocument();

		// set container scope for code
		$document->addScriptDeclaration("gslUIkit.container = '.gsl-scope';");

		HTMLHelper::stylesheet('media/com_jevents/css/uikit.gsl.css', array('version' => GslHelper::RSVPPro_Version(false), 'relative' => false));
		HTMLHelper::stylesheet('administrator/components/com_rsvppro/assets/css/rsvppro.css', array('version' => GslHelper::RSVPPro_Version(false), 'relative' => false));
		$jversion = new Version;
		if ($jversion->isCompatible('4.0'))
		{
			HTMLHelper::stylesheet('administrator/components/com_rsvppro/assets/css/j4.css', array('version' => GslHelper::RSVPPro_Version(false), 'relative' => false));
		}
		else
		{
			HTMLHelper::stylesheet('administrator/components/com_rsvppro/assets/css/j3.css', array('version' => GslHelper::RSVPPro_Version(false), 'relative' => false));
		}
		HTMLHelper::script('media/com_jevents/js/uikit.js', array('version' => GslHelper::RSVPPro_Version(false), 'relative' => false));
		HTMLHelper::script('media/com_jevents/js/uikit-icons.js', array('version' => GslHelper::RSVPPro_Version(false), 'relative' => false));
		HTMLHelper::script('components/com_jevents/assets/js/gslframework.js', array('version' => JEventsHelper::JEvents_Version(false), 'relative' => false), array('defer' => true));
		//HTMLHelper::script('administrator/components/com_rsvppro/assets/js/gslframework.js', array('version' => GslHelper::RSVPPro_Version(false), 'relative' => false));
		HTMLHelper::script('administrator/components/com_rsvppro/assets/js/rsvppro.js', array('version' => GslHelper::RSVPPro_Version(false), 'relative' => false));
	}

	public static function renderModal()
	{
		return;
	}

	static public function renderVersion()
	{
		echo LayoutHelper::render('rsvppro.version');
	}

	static public function translate($string, $jssafe = true)
	{
		$string = "COM_RSVPPRO_" . $string;

		return Text::_($string, $jssafe);
	}

	static public
	function isAdminUser($user = null)
	{

		if (is_null($user))
		{
			$user = Factory::getUser();
		}
		//$access = Access::check($user->id, "core.admin","com_jevents");
		// Add a second check incase the getuser failed.
		if (!$user)
		{
			return false;
		}
		$access = $user->authorise('core.admin', 'com_jevents');

		return $access;
	}

	static public function supportLink()
	{
		return "https://www.jevents.net/discussions/gold-member-forum";
	}

	static public function documentationLink()
	{
		return "https://www.jevents.net/documentation";
	}

	static public function configLink()
	{
		$uri    = (string) Uri::getInstance();
		$return = urlencode(base64_encode($uri));

		return Uri::base() . 'index.php?option=com_config&view=component&component=com_rsvppro&return=' . $return;
	}

	static public function cpanelIconLink()
	{
		$jeventsParams = ComponentHelper::getParams('com_jevents');
		$leftmenutrigger = $jeventsParams->get("leftmenutrigger", 0);

		$onclick = $leftmenutrigger == 2 ? "" : 'onclick="if((window.getComputedStyle(this.querySelector(\'.nav-label\')).getPropertyValue(\'display\')==\'none\' && window.innerWidth <= 960) || window.getComputedStyle(this.querySelector(\'.nav-label\')).getPropertyValue(\'display\')!==\'none\') {document.location=this.href;}return false;"';
		?>
		<div>
			<a href="<?php echo JRoute::_("index.php?option=com_rsvppro&view=cpanel"); ?>" class="" <?php echo $onclick;?> >
				<span data-gsl-icon="icon: home" class="gsl-margin-small-right gsl-margin-small-left"></span>
				<span class="nav-label"><?php echo JText::_('COM_RSVPPRO_ADMIN_CPANEL'); ?></span>
			</a>
		</div>
		<?php
	}

	static public function getLeftIconLinks()
	{
		$app    = Factory::getApplication();
		$input  = $app->input;
		$task   = $input->getCmd('task', 'tags.overview');
		$option = $input->getCmd('option', 'com_rsvppro');
		list($view, $layout) = explode(".", $task);

		$jeventsParams = ComponentHelper::getParams('com_jevents');
		$leftmenutrigger = (int) $jeventsParams->get("leftmenutrigger", 0);

		$compparams = ComponentHelper::getParams("com_rsvppro");
		$usecats    = $compparams->get("usecats", 0);

		$iconLinks = array();

		$iconLink                 = new stdClass();
		$iconLink->class          = "";
		$iconLink->active         = $view == "sessions";
		$iconLink->link           = Route::_("index.php?option=com_rsvppro&task=sessions.overview");
		$iconLink->icon           = "calendar";
		$iconLink->label          = Text::_('RSVP_SESSIONS');
		$iconLink->tooltip        = $leftmenutrigger !== 2 ? "" : Text::_('RSVP_SESSIONS');
		$iconLink->tooltip_detail = "";
		$iconLinks[]              = $iconLink;

		$iconLink                 = new stdClass();
		$iconLink->class          = "";
		$iconLink->active         = $view == "pmethods";
		$iconLink->link           = Route::_("index.php?option=com_rsvppro&task=pmethods.overview");
		$iconLink->icon           = "credit-card";
		$iconLink->label          = Text::_('RSVP_PRO_PAYMENT_METHODS');
		$iconLink->tooltip        = $leftmenutrigger !== 2 ? "" : Text::_('RSVP_PRO_PAYMENT_METHODS');
		$iconLink->tooltip_detail = "";
		$iconLinks[]              = $iconLink;

        if (JevTemplateHelper::canCreateOwn())
        {
	        $iconLink                 = new stdClass();
	        $iconLink->class          = "";
	        $iconLink->active         = $view == "templates";
	        $iconLink->link           = Route::_("index.php?option=com_rsvppro&task=templates.overview");
	        $iconLink->icon           = "thumbnails";
	        $iconLink->label          = Text::_('RSVP_TEMPLATES');
	        $iconLink->tooltip        = $leftmenutrigger !== 2 ? "" : Text::_('RSVP_TEMPLATES');
	        $iconLink->tooltip_detail = "";
	        $iconLinks[]              = $iconLink;
        }

		// Links to addons
		// Managed Locations
		$db = Factory::getDbo();
		$db->setQuery("SELECT enabled FROM #__extensions WHERE element = 'com_jevlocations' AND type='component' ");
		$is_enabled = $db->loadResult();
		if ($is_enabled)
		{
			Factory::getLanguage()->load("com_jevlocations", JPATH_ADMINISTRATOR);

			$iconLink                 = new stdClass();
			$iconLink->class          = "";
			$iconLink->active         = $view == "jevlocations";
			$iconLink->link           = Route::_("index.php?option=com_jevlocations");
			$iconLink->icon           = "location";
			$iconLink->label          = Text::_('COM_JEVLOCATIONS');
			$iconLink->tooltip        = $leftmenutrigger !== 2 ? "" : Text::_('COM_JEVLOCATIONS');
			$iconLink->tooltip_detail = "";
			if (file_exists(JPATH_ADMINISTRATOR . "/components/com_jevlocations/helpers/gslmenuhelper.php"))
			{
				include_once JPATH_ADMINISTRATOR . "/components/com_jevlocations/helpers/gslmenuhelper.php";
				$iconLink->sublinks = GslLocationsMenuHelper::getLeftIconSubLinks($leftmenutrigger);
			}
			$iconLinks[]              = $iconLink;

		}

		// Managed People
		$db = Factory::getDbo();
		$db->setQuery("SELECT enabled FROM #__extensions WHERE element = 'com_jevpeople' AND type='component' ");
		$is_enabled = $db->loadResult();
		if ($is_enabled)
		{
			Factory::getLanguage()->load("com_jevpeople", JPATH_ADMINISTRATOR);

			$iconLink                 = new stdClass();
			$iconLink->class          = "";
			$iconLink->active         = $view == "jevpeople";
			$iconLink->link           = Route::_("index.php?option=com_jevpeople");
			$iconLink->icon           = "user";
			$iconLink->label          = Text::_('COM_JEVPEOPLE');
			$iconLink->tooltip        = $leftmenutrigger !== 2 ? "" : Text::_('COM_JEVPEOPLE');
			$iconLink->tooltip_detail = "";
			if (file_exists(JPATH_ADMINISTRATOR . "/components/com_jevpeople/helpers/gslmenuhelper.php"))
			{
				include_once JPATH_ADMINISTRATOR . "/components/com_jevpeople/helpers/gslmenuhelper.php";
				$iconLink->sublinks = GslPeopleMenuHelper::getLeftIconSubLinks($leftmenutrigger);
			}
			$iconLinks[]              = $iconLink;

		}

		// JEvents Tags
		$db = Factory::getDbo();
		$db->setQuery("SELECT enabled FROM #__extensions WHERE element = 'com_jeventstags' AND type='component' ");
		$is_enabled = $db->loadResult();
		if ($is_enabled)
		{
			Factory::getLanguage()->load("com_jeventstags", JPATH_ADMINISTRATOR);

			$iconLink                 = new stdClass();
			$iconLink->class          = "";
			$iconLink->active         = $view == "com_rsvppro";
			$iconLink->link           = Route::_("index.php?option=com_jeventstags");
			$iconLink->icon           = "hashtag";
			$iconLink->label          = Text::_('COM_JEVENTSTAGS');
			$iconLink->tooltip        = $leftmenutrigger !== 2 ? "" : Text::_('COM_JEVENTSTAGS');
			$iconLink->tooltip_detail = "";
			if (file_exists(JPATH_ADMINISTRATOR . "/components/com_jeventstags/helpers/gslmenuhelper.php"))
			{
				include_once JPATH_ADMINISTRATOR . "/components/com_jeventstags/helpers/gslmenuhelper.php";
				$iconLink->sublinks = GslTagsMenuHelper::getLeftIconSubLinks($leftmenutrigger);
			}
			$iconLinks[]              = $iconLink;

		}
		/*
        // RSVP Pro
		$db = Factory::getDbo();
		$db->setQuery("SELECT enabled FROM #__extensions WHERE element = 'com_rsvppro' AND type='component' ");
		$is_enabled = $db->loadResult();
		if ($is_enabled)
		{
			Factory::getLanguage()->load("com_rsvppro", JPATH_ADMINISTRATOR);

			$iconLink                 = new stdClass();
			$iconLink->class          = "";
			$iconLink->active         = $view == "rsvppro";
			$iconLink->link           = Route::_("index.php?option=com_rsvppro");
			$iconLink->icon           = "cart";
			$iconLink->label          = Text::_('COM_RSVPPRO');
			$iconLink->tooltip        = $leftmenutrigger !== 2 ? "" : Text::_('COM_RSVPPRO');
			$iconLink->tooltip_detail = "";
			$iconLinks[]              = $iconLink;
		}
		*/

        // Custom Fields
        // Custom Fields
		$db = Factory::getDbo();
		$db->setQuery("SELECT * FROM #__extensions WHERE element = 'jevcustomfields' AND type='plugin' AND folder='jevents' ");
		$extension = $db->loadObject();
// Stop if user is not authorised to manage JEvents
		if ($extension && $extension->enabled && JEVHelper::isAdminUser())
		{

			Factory::getLanguage()->load("plg_jevents_jevcustomfields", JPATH_ADMINISTRATOR);

			$iconLink                 = new stdClass();
			$iconLink->class          = "";
			$iconLink->active         = $view == "rsvppro";
			$iconLink->link           = Route::_("index.php?option=com_jevents&task=plugin.jev_customfields.overview");
			$iconLink->icon           = "code";
			$iconLink->label          = Text::_('JEV_CUSTOM_FIELDS');
			$iconLink->tooltip        = $leftmenutrigger !== 2 ? "" : Text::_('JEV_CUSTOM_FIELDS');
			$iconLink->tooltip_detail = "";
			$iconLinks[]              = $iconLink;
		}

		$iconLink                 = new stdClass();
		$iconLink->class          = "returntojoomla";
		$iconLink->active         = false;
		$iconLink->link           = Route::_("index.php");
		$iconLink->icon           = "joomla";
		$iconLink->label          = Text::_('COM_JEVENTS_RETURN_TO_JOOMLA');
		$iconLink->tooltip        = Text::_("COM_JEVENTS_RETURN_TO_JOOMLA_TOOLTIP");
		$iconLink->tooltip_detail = "";
		$iconLink->events         = [];
		$iconLinks[]              = $iconLink;

		return $iconLinks;
	}

	static public function returnToMainComponent()
	{
		$jeventsParams = ComponentHelper::getParams('com_jevents');
		$leftmenutrigger = $jeventsParams->get("leftmenutrigger", 0);

		$onclick = $leftmenutrigger == 2 ? "" : 'onclick="if((window.getComputedStyle(this.querySelector(\'.nav-label\')).getPropertyValue(\'display\')==\'none\' && window.innerWidth <= 960) || window.getComputedStyle(this.querySelector(\'.nav-label\')).getPropertyValue(\'display\')!==\'none\') {document.location=this.href;}return false;"';
		// Return to JEvents
		?>
		<li>
			<a href="<?php echo JRoute::_("index.php?option=com_jevents"); ?>" <?php echo $onclick;?> >
		<span class="" style="margin-right:6px">
			<img src="<?php echo JUri::base(); ?>components/com_jevents/assets/images/logo.png"
			     style="height:20px"
			     alt="JEvents Logo">
		</span>
				<span class="nav-label"><?php echo Text::_('COM_RSVPPRO_RETURN_TO_JEVENTS'); ?></span>
			</a>
		</li>
		<?php
		if (file_exists(JPATH_ADMINISTRATOR . "/components/com_jevents/helpers/gslmenuhelper.php"))
		{
			include_once JPATH_ADMINISTRATOR . "/components/com_jevents/helpers/gslmenuhelper.php";
			$sublinks = GslJEventsMenuHelper::getLeftIconSubLinks($leftmenutrigger);
			if ($sublinks && count($sublinks))
			{
				?>
				<div class="gsl-dropdown  gsl-background-secondary" gsl-dropdown='{"mode": "click, hover", "delay-hide":100, "offset":0 ,"pos":"right-top"}'>
					<ul class="gsl-padding-remove">
						<?php
						foreach ( $sublinks as $sublink)
						{
							?>
							<li class="gsl-padding-remove">
								<button onclick="<?php echo $sublink->onclick; ?>"
								        class="<?php echo $sublink->class; ?>">
			                        <span gsl-icon="icon: <?php echo $sublink->icon; ?>"
			                              class="<?php echo $sublink->iconclass; ?>">
			                        </span>
									<?php echo $sublink->label; ?>
								</button>
							</li>
							<?php
						}
						?>
					</ul>
				</div>
				<?php
			}
		}

	}

	static public function RSVPPro_Version($outputinput = true)
	{
	    static $packageversionset = false;
		static $packageversion = '3.7.8';

		if (!$packageversionset)
		{
			$packageversionset = true;

			// When installed directly from BitBucket the manifest cache is not kept up to date also YOURSITES_VERSION needs to be replaced
			if ($packageversion == ('RSVPRO_' . 'VERSION') && file_exists(dirname(dirname(dirname(dirname(__DIR__)))) . "/package/pkg_rsvppro.xml"))
			{
				$pkgcontents = file_get_contents(dirname(dirname(dirname(dirname(__DIR__)))) . "/package/pkg_rsvppro.xml");
				$matches     = array();
				preg_match('#<version>(.*)<\/version>#', $pkgcontents, $matches);
				if (count($matches) == 2)
				{
					$packageversion = $matches[1];
				}
			}
		}
		if ($outputinput)
		{
			?>
			<input type="hidden" id="rsvppro_version" value="<?php echo $packageversion; ?>"/>
			<?php
		}
		return $packageversion;
	}

}
