<?php
/**
 * Copyright (C)2010-2021 GWE Systems Ltd
 *
 * All rights reserved.
 *
 */
defined('_JEXEC') or die('No Direct Access');

use Joomla\CMS\Language\Text;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Factory;
use Joomla\CMS\HTML\HTMLHelper;

jimport("joomla.html.pagination");

class RsvpHelper
{

	static function canCreateSessions()
	{
		$params = ComponentHelper::getParams('com_jevents');
		$authorisedonly = $params->get("authorisedonly", 0);
		if (!$authorisedonly)
		{
			$plugin = PluginHelper::getPlugin("jevents", "jevrsvppro");
			$rsvpparams = ComponentHelper::getParams('com_rsvppro');
			$juser =  Factory::getUser();
			if ($juser->authorise('core.createreg', 'com_rsvppro'))
			{
				return true;
			}
		}
		else
		{
			$jevuser =  JEVHelper::getAuthorisedUser();
			if ($jevuser && $jevuser->cancreateown)
			{
				// if jevents is not in authorised only mode then switch off this user's permissions
				return true;
			}
		}
		return false;
	}

	static function canManageSessionAttendees($specificSession = false, $event = false)
	{
		return false;
		$user = Factory::getUser();
		$userGroups = $user->getAuthorisedGroups();
		$params = ComponentHelper::getParams("com_rsvppro");
		$matches = array_intersect($userGroups, $params->get("attendeemanagement", array()));
		return count($matches) > 0 ;
	}

	static function accessOptions($id, $field)
	{
		$fieldname = "fa[$id]";
		$value = $field ? $field->access : 0;
		$params = ComponentHelper::getParams("com_rsvppro");
		$style = $params->get("allowaccessrestrictions",1)?'' : 'style="display:none"';

		$fparams = new stdClass();
		if ($field)
		{
			try {
				$fparams = json_decode($field->params);
			}
			catch (Exception $e) {

			}
		}
		if (isset($fparams->overrideaccess))
		{
			$overrideaccess = $fparams->overrideaccess;
		}
		else
		{
			$overrideaccess = 1;
		}

		?>
		<div class="rsvpnotranslate">
			<div class="gsl-form-label" <?php echo $style;?> ><?php echo Text::_("RSVP_ACCESS"); ?></div>
			<div class="gsl-form-controls  radio btn-group"  <?php echo $style;?> >
				<?php
				include_once(JPATH_SITE . "/components/com_jevents/libraries/jeventshtml.php");
				if (is_callable(array("JEventsHTML", "buildAccessSelect")))
				{
					if (version_compare(JVERSION, "1.6.0", 'ge'))
					{
						echo JEventsHTML::buildAccessSelect(intval($value), 'class="inputbox" size="1" ', "", $fieldname);
					}
					else
					{
						echo JEventsHTML::buildAccessSelect(intval($value), 'class="inputbox" size="1"  ', "", $fieldname);
					}
				}
				else
				{
					static $groups;
					if (!isset($groups))
					{
						// get list of groups
						$db = Factory::getDBO();
						$query = "SELECT id AS value, name AS text"
								. "\n FROM #__groups"
								. "\n ORDER BY ordering DESC";
						$db->setQuery($query);
						$groups = $db->loadObjectList();
					}

					if(empty($value)) {
					    // Set default config value
                        $config         = Factory::getConfig();
                        $value          = (int) $config->get('access', 12);
                    }

					// build the html select list
					echo HTMLHelper::_('select.genericlist', $groups, $fieldname, 'class="inputbox" size="1"', 'value', 'text', $value);
				}

				// field access flag - everyone apart from members of this group can access if this has value 1
				$fieldname = "faf[$id]";
				$value = $field ? $field->accessflag : 1;
				?>
            </div>
			<div class="rsvpclear"></div>

			<div class="gsl-form-label" <?php echo $style;?> ></div>
            <div class="gsl-form-controls  radio btn-group"   <?php echo $style;?> >
                <label for="accessflag1<?php echo $id; ?>" class="btn radio" ><?php echo Text::_("RSVP_ALLOWED_ACCESS"); ?>
                    <input type="radio" name="accessflag[<?php echo $id; ?>]"    id="accessflag1<?php echo $id; ?>" value="1" <?php
					if ($field && $field->accessflag == 1)
					{
						echo 'checked="checked"';
					}
					if (!$field)
					{
						echo 'checked="checked"';
					}
					?> />
                </label>
                <label for="accessflag0<?php echo $id; ?>" class="btn radio" ><?php echo Text::_("RSVP_ACCESS_BLOCKED"); ?>
                    <input type="radio" name="accessflag[<?php echo $id; ?>]"   id="accessflag0<?php echo $id; ?>" value="0" <?php
					if ($field && $field->accessflag == 0)
					{
						echo 'checked="checked"';
					}
					?> />
                </label>
            </div>
			<div class="rsvpclear"></div>
            <div class="gsl-form-label" <?php echo $style;?> ><?php echo Text::_("RSVP_ATTENDEE_EDITOR_OVERRIDE_ACCESS"); ?></div>
            <div class="gsl-form-controls  radio btn-group"   <?php echo $style;?> >
                <label for="overrideaccess1<?php echo $id; ?>" class="btn radio"><?php echo Text::_("RSVP_YES"); ?>
                    <input type="radio" name="params[<?php echo $id; ?>][overrideaccess]"  id="overrideaccess1<?php echo $id; ?>" value="1" <?php
					if ($overrideaccess == 1)
					{
						echo 'checked="checked"';
					}
					else  if (!$field)
					{
						echo 'checked="checked"';
					}

					?> />
                </label>
                <label for="overrideaccess0<?php echo $id; ?>"  class="btn radio"><?php echo Text::_("RSVP_NO"); ?>
                    <input type="radio" name="params[<?php echo $id; ?>][overrideaccess]" id="overrideaccess0<?php echo $id; ?>" value="0" <?php
					if ($field && $overrideaccess == 0)
					{
						echo 'checked="checked"';
					}
					?> />
                </label>
            </div>
            <div class="rsvpclear"></div>
        </div>
		<?php

	}
	static function required($id, $field, $default = 0)
	{
		?>
		<div class="rsvpnotranslate rsvprrfield">
		<div class="gsl-form-label"><?php echo Text::_("RSVP_IS_REQUIRED"); ?></div>
		<div class="gsl-form-controls radio btn-group">
			<label for="rr1<?php echo $id; ?>" class="btn radio"><?php echo Text::_("RSVP_YES"); ?>
			<input type="radio" name="rr[<?php echo $id; ?>]"  id="rr1<?php echo $id; ?>" value="1" <?php
		if (($field && $field->required == 1) || (!$field && $default) )
		{
			echo 'checked="checked"';
		}
		?> />
			</label>
			<label for="rr0<?php echo $id; ?>"  class="btn radio"><?php echo Text::_("RSVP_NO"); ?>
			<input type="radio" name="rr[<?php echo $id; ?>]" id="rr0<?php echo $id; ?>" value="0" <?php
		   if ($field && $field->required == 0)
		   {
			   echo 'checked="checked"';
		   }
		   if (!$field && !$default)
		   {
			   echo 'checked="checked"';
		   }
		?> />
			</label>
		</div>
		<div class="rsvpclear"></div>
		</div>
		<?php

	}

	static function requiredAnyAll($id, $field, $default = 0)
	{
		?>
		<div class="rsvpnotranslate">
		<div class="gsl-form-label"><?php echo Text::_("RSVP_IS_REQUIRED"); ?></div>
		<div class="gsl-form-controls radio btn-group">
			<label for="rr1<?php echo $id; ?>" class="btn radio"><?php echo Text::_("RSVP_ALL_REQUIRED"); ?>
			<input type="radio" name="rr[<?php echo $id; ?>]"  id="rr1<?php echo $id; ?>" value="1" <?php
		if (($field && $field->required == 1) || (!$field && $default) )
		{
			echo 'checked="checked"';
		}
		?> />
			</label>
			<label for="rr2<?php echo $id; ?>" class="btn radio"><?php echo Text::_("RSVP_ANY_REQUIRED"); ?>
			<input type="radio" name="rr[<?php echo $id; ?>]"  id="rr2<?php echo $id; ?>" value="2" <?php
		if (($field && $field->required == 2) || (!$field && $default) )
		{
			echo 'checked="checked"';
		}
		?> />
			</label>
			<label for="rr0<?php echo $id; ?>"  class="btn radio"><?php echo Text::_("RSVP_NO"); ?>
			<input type="radio" name="rr[<?php echo $id; ?>]" id="rr0<?php echo $id; ?>" value="0" <?php
		   if ($field && $field->required == 0)
		   {
			   echo 'checked="checked"';
		   }
		   if (!$field && !$default)
		   {
			   echo 'checked="checked"';
		   }
		?> />
			</label>
		</div>
		<div class="rsvpclear"></div>
		</div>
		<?php

	}

	static function requiredMessage($id, $field)
	{
		?>
		<div class="rsvpnotranslate rsvprmfield">
		<div class="gsl-form-label"><label for="rm<?php echo $id; ?>"><?php echo Text::_("RSVP_REQUIRED_MESSAGE"); ?></label></div>
		<div class="gsl-form-controls">
			<input type="text" name="rm[<?php echo $id; ?>]" id="rm<?php echo $id; ?>" value="<?php echo $field ? $field->requiredmessage : ''; ?>" size="40" maxlength="255" />
		</div>
		<div class="rsvpclear"></div>
		</div>
		<?php

	}

	static function placeholder($id, $field)
	{
		?>
		<div class="rsvpnotranslate">
		<div class="gsl-form-label"><label for="ph<?php echo $id; ?>"><?php echo Text::_("RSVP_PLACEHOLDER"); ?></label></div>
		<div class="gsl-form-controls">
			<input type="text" name="ph[<?php echo $id; ?>]" id="ph<?php echo $id; ?>" value="<?php echo $field ? $field->placeholder : ''; ?>" size="40" maxlength="255" />
		</div>
		<div class="rsvpclear"></div>
		</div>
		<?php
	}

	static function className($id, $field)
	{
		$params = ComponentHelper::getParams("com_rsvppro");
		if ($params->get("showclassname",0)) {
		?>
		<div class="rsvpnotranslate">
		<div class="gsl-form-label"><?php echo Text::_("RSVP_CLASSNAME"); ?></div>
		<div class="gsl-form-controls">
			<input type="text" name="cn[<?php echo $id; ?>]" id="cn<?php echo $id; ?>" value="<?php echo $field ? $field->classname : ''; ?>" size="40" maxlength="255" />
		</div>
		<div class="rsvpclear"></div>
		</div>
		<?php
                }
                else {
                    ?>
                <input type="hidden" name="cn[<?php echo $id; ?>]" id="cn<?php echo $id; ?>" value="<?php echo $field ? $field->classname : ''; ?>" size="40" maxlength="255" />
                    <?php
                }

	}

        static function allowoverride($id, $field)
	{
		return "";
		?>
		<div class="rsvpnotranslate">
		<div class="gsl-form-label"><?php echo Text::_("RSVP_ALLOW_OVERRIDE"); ?></div>
		<div class="gsl-form-controls  radio btn-group">
			<label for="ao1<?php echo $id; ?>" class="btn radio" ><?php echo Text::_("RSVP_YES"); ?>
			<input type="radio"   name="ao[<?php echo $id; ?>]"  id="ao1<?php echo $id; ?>" value="1" <?php
		if ($field && $field->allowoverride == 1)
		{
			echo 'checked="checked"';
		}
		?> />
			</label>
			<label for="ao0<?php echo $id; ?>" class="btn radio" ><?php echo Text::_("RSVP_NO"); ?>
			<input type="radio"  name="ao[<?php echo $id; ?>]" id="ao0<?php echo $id; ?>" value="0" <?php
		   if ($field && $field->allowoverride == 0)
		   {
			   echo 'checked="checked"';
		   }
		   if (!$field)
		   {
			   echo 'checked="checked"';
		   }
		?> />
			</label>
		</div>
		<div class="rsvpclear"></div>
		</div>
		<?php

	}

	static function tooltip($id, $field)
	{
		?>
		<div class="gsl-form-label"><label for="ft<?php echo $id; ?>"><?php echo Text::_("RSVP_FIELD_TOOLTIP"); ?></label></div>
		<div class="gsl-form-controls">
			<input type="text" name="ft[<?php echo $id; ?>]" id="ft<?php echo $id; ?>" value="<?php echo $field ? htmlspecialchars($field->tooltip) : ''; ?>" size="40" maxlength="255" />
		</div>
		<div class="rsvpclear"></div>
		<?php

	}

	static function size($id, $field, $fieldtype)
	{
		?>
		<div class="rsvpnotranslate">
		<div class="gsl-form-label"><label for="size<?php echo $id; ?>"><?php echo Text::_("RSVP_FIELD_SIZE"); ?></label></div>
		<div class="gsl-form-controls">
			<input type="text" name="size[<?php echo $id; ?>]" id="size<?php echo $id; ?>" value="<?php echo $field && $field->size > 0 ? $field->size : 10; ?>" size="5" maxlength="5"
				   onchange="<?php echo $fieldtype; ?>.changeSize('<?php echo $id; ?>')"    onkeyup="<?php echo $fieldtype; ?>.changeSize('<?php echo $id; ?>')" />
		</div>
		<div class="rsvpclear"></div>
		</div>
		<?php

	}

	static function maxlength($id, $field, $fieldtype)
	{
		?>
		<div class="rsvpnotranslate">
		<div class="gsl-form-label"><label for="maxlength<?php echo $id; ?>"><?php echo Text::_("RSVP_FIELD_MAXLENGTH"); ?></label></div>
		<div class="gsl-form-controls">
			<input type="text" name="maxlength[<?php echo $id; ?>]" id="maxlength<?php echo $id; ?>" value="<?php echo $field && $field->maxlength > 0 ? $field->maxlength : 20; ?>" size="5" maxlength="5"
				   onchange="<?php echo $fieldtype; ?>.changeMaxlength('<?php echo $id; ?>')" />
		</div>
		<div class="rsvpclear"></div>
		</div>
		<?php

	}

	static function cols($id, $field, $fieldtype)
	{
		?>
		<div class="rsvpnotranslate">
		<div class="gsl-form-label"><label for="cols<?php echo $id; ?>"><?php echo Text::_("RSVP_FIELD_COLS"); ?></label></div>
		<div class="gsl-form-controls">
			<input type="text" name="cols[<?php echo $id; ?>]" id="cols<?php echo $id; ?>" value="<?php echo $field && $field->cols > 0 ? $field->cols : 20; ?>" size="5" maxlength="5"
				   onchange="<?php echo $fieldtype; ?>.changeCols('<?php echo $id; ?>')" />
		</div>
		<div class="rsvpclear"></div>
		</div>
		<?php

	}

	static function rows($id, $field, $fieldtype)
	{
		?>
		<div class="rsvpnotranslate">
		<div class="gsl-form-label"><label for="rows<?php echo $id; ?>"><?php echo Text::_("RSVP_FIELD_ROWS"); ?></label></div>
		<div class="gsl-form-controls">
			<input type="text" name="rows[<?php echo $id; ?>]" id="rows<?php echo $id; ?>" value="<?php echo $field && $field->rows > 0 ? $field->rows : 5; ?>" size="5" maxlength="5"
				   onchange="<?php echo $fieldtype; ?>.changeRows('<?php echo $id; ?>')" />
		</div>
		<div class="rsvpclear"></div>
		</div>
		<?php

	}

	static function label($id, $field, $fieldtype="")
	{
		?>
		<div class="gsl-form-label"><label for="fl<?php echo $id; ?>"><?php echo Text::_("RSVP_FIELD_LABEL"); ?></label></div>
		<div class="gsl-form-controls">
			<input type="text" name="fl[<?php echo $id; ?>]" id="fl<?php echo $id; ?>" value="<?php echo $field ? $field->label : Text::_("RSVP_FIELD_LABEL"); ?>" size="40" maxlength="255" class="rsvpfl"
				   onkeyup="rsvppro.updateLabel('<?php echo $id; ?>');"  rel="<?php echo $fieldtype; ?>"/>
		</div>
		<div class="rsvpclear"></div>
        <?php

		$rsvpparams = ComponentHelper::getParams('com_rsvppro');
		if ($rsvpparams->get("customisename", 0))
		{
			?>
            <div class="gsl-form-label"><label for="fn<?php echo $id; ?>"><?php echo Text::_("RSVP_FIELD_NAME"); ?></label>
            </div>
            <div class="gsl-form-controls">
                <input type="text" name="fn[<?php echo $id; ?>]" id="fn<?php echo $id; ?>"
                       value="<?php echo $field ? $field->name : ''; ?>" size="40"
                       maxlength="255" class="rsvpfn"
                       placeholder="<?php echo Text::_("RSVP_FIELD_NAME_PLACEHOLDER");?>"
                       rel="<?php echo $fieldtype; ?>"/>
            </div>
            <div class="rsvpclear"></div>
			<?php
		}
	}

	static function fieldId($id)
	{
		echo " <em>{" . $id . "}</em>";
		return "";
	}

	static function hidden($id, $field, $name)
	{
		?>
		<input type="hidden" name="type[<?php echo $id; ?>]" id="type<?php echo $id; ?>" value="<?php echo $name; ?>" />
		<input type="hidden" name="fid[<?php echo $id; ?>]" id="fid<?php echo $id; ?>" value="<?php echo $field ? $id : 0; ?>" />
		<input type="hidden" name="ordering[<?php echo $id; ?>]" id="ordering<?php echo $id; ?>" value="<?php echo $field ? $field->ordering : 0; ?>" />
                <textarea style="display:none;" name="defaultvalue[<?php echo $id; ?>]" id="defaultvalue<?php echo $id; ?>" ><?php echo ($field && is_string($field->defaultvalue)) ? $field->defaultvalue : 0 ?></textarea>
		<?php

	}

                static function textField($id, $name, $field, $label, $tooltip,  $fieldattribute, $fielddefault = "", $attributes = "") {
                    $value = $fielddefault;
                    if (isset($field) && !isset($field->$fieldattribute)){
                        if (isset($field->params) && $field->params!=""){
                            $params = json_decode($field->params);
                            if (isset($params->$fieldattribute)){
                                $value = $params->$fieldattribute;
                            }
                        }
                    }
                    ?>
        <div class="gsl-form-label">
            <label for="<?php echo $id; ?>">
        <?php echo $label; ?>
        <?php RsvpHelper::fieldTooltip($tooltip); ?>
            </label>
        </div>
        <div class="gsl-form-controls">
            <input type="text" name="<?php echo $name; ?>" id="<?php echo $id; ?>" <?php echo $attributes; ?>
                   value="<?php echo htmlspecialchars($value); ?>"
                   size="40" maxlength="255" />
        </div>
        <div class="rsvpclear"></div>
                <?php
            }

                static function textArea($id, $name, $field, $label, $tooltip,  $fieldattribute, $fielddefault = "", $attributes = "", $rows=5, $cols=60, $placeholder= "") {
                    $value = $fielddefault;
                    if (isset($field) && !isset($field->$fieldattribute)){
                        if (isset($field->params) && $field->params!=""){
                            $params = json_decode($field->params);
                            if (isset($params->$fieldattribute)){
                                $value = $params->$fieldattribute;
                            }
                        }
                    }
                    ?>
        <div class="gsl-form-label">
            <label for="<?php echo $id; ?>">
        <?php echo $label; ?>
        <?php RsvpHelper::fieldTooltip($tooltip); ?>
            </label>
        </div>
        <div class="gsl-form-controls">
            <textarea
            name="<?php echo $name; ?>" id="<?php echo $id; ?>" <?php echo $attributes; ?>
             rows="<?php echo $rows;?>" cols="<?php echo $cols;?>" style="width:auto"
            placeholder="<?php echo htmlspecialchars($placeholder); ?>"
             ><?php echo htmlspecialchars($value); ?></textarea>
        </div>
        <div class="rsvpclear"></div>
                <?php
            }

        static function fieldTooltip($fieldtooltip = false) {
            if ($fieldtooltip) {
                if (strpos($fieldtooltip, "JEVR_") === 0) {
                    $fieldtooltip = Text::_($fieldtooltip);
                }

                echo " " . HTMLHelper::_('tooltip', $fieldtooltip, null, null, '<span class="icon-help fieldtooltip"></span>', null, 0);
            }
        }

	private static $fieldscript;

	static function setField($id, $field, $html, $name)
	{
		if (!$field)
		{
			static $script;
			if (!isset(self::$fieldscript))
			{
				self::$fieldscript = "";
			}
			self::$fieldscript .= "rsvpFieldTypes['" . strtolower($name) . "']=" . json_encode($html) . ";\n";
			return "";
		}
		else
		{
			return $html;
		}

	}

	static function getFieldScript()
	{
		$document = Factory::getDocument();
		$document->addScriptDeclaration(self::$fieldscript);

	}

	static function formonly($id, $field, $default = 0)
	{
		$params = ComponentHelper::getParams("com_rsvppro");
		$style = $params->get("standardvisibility",1)?'' : 'style="display:none"';
		?>
		<div class="rsvpnotranslate">
		<div class="gsl-form-label" <?php echo $style;?> ><?php echo Text::_("RSVP_IN_FORM_ONLY"); ?></div>
		<div class="gsl-form-controls  radio btn-group" <?php echo $style;?> >
			<label for="formonly1<?php echo $id; ?>" class="btn radio"><?php echo Text::_("RSVP_YES"); ?>
			<input type="radio"  name="formonly[<?php echo $id; ?>]"  id="formonly1<?php echo $id; ?>" value="1" <?php
		if ($field && $field->formonly == 1)
		{
			echo 'checked="checked"';
		}
		if (!$field && $default)
		{
			echo 'checked="checked"';
		}
		?> />
			</label>
			<label for="formonly0<?php echo $id; ?>" class="btn radio"><?php echo Text::_("RSVP_NO"); ?>
			<input type="radio"  name="formonly[<?php echo $id; ?>]" id="formonly0<?php echo $id; ?>" value="0" <?php
		   if ($field && $field->formonly == 0)
		   {
			   echo 'checked="checked"';
		   }
		  if (!$field && !$default)
		   {
			   echo 'checked="checked"';
		   }
		?> />
			</label>
		</div>
		<div class="rsvpclear"></div>
		</div>
		<?php

	}

	static function peruser($id, $field)
	{
		$fieldname = "peruser[$id]";
		$value = $field ? $field->peruser : 0;
		?>
		<div class="rsvpnotranslate">
		<div class="gsl-form-label"><?php echo Text::_("RSVP_PER_USER"); ?></div>
		<div class="gsl-form-controls">
			<?php
			$options = array();
			$options[] = HTMLHelper::_('select.option', 0, Text::_("RSVP_PRIMARY_REGISTRATION"));
			$options[] = HTMLHelper::_('select.option', 1, Text::_("RSVP_PRIMARY_REGISTRATION_AND_GUESTS"));
			$options[] = HTMLHelper::_('select.option', 2, Text::_("RSVP_PRIMARY_GUESTS_ONLY"));

                        Text::script('JEV_CANNOT_HAVE_FIELD_OPTION_CAPACITIES_WITH_GUESTS');
			// build the html select list
			echo HTMLHelper::_('select.genericlist', $options, $fieldname, 'class="inputbox peruser" onchange="rsvpforms.fieldOptionCapacityWarning(\''.$id.'\');return false;" ', 'value', 'text', $value);
			?>
		</div>
		<div class="rsvpclear"></div>
		</div>
		<?php

	}

	static function showinform($id, $field)
	{
		$params = ComponentHelper::getParams("com_rsvppro");
		$style = $params->get("standardvisibility",1)?'' : 'style="display:none"';
		?>
		<div class="rsvpnotranslate">
		<div class="gsl-form-label" <?php echo $style;?> ><?php echo Text::_("RSVP_SHOW_IN_FORM"); ?></div>
		<div class="gsl-form-controls  radio btn-group" <?php echo $style;?> >
			<label for="showinform1<?php echo $id; ?>" class="btn radio"><?php echo Text::_("RSVP_YES"); ?>
			<input type="radio"  name="showinform[<?php echo $id; ?>]"  id="showinform1<?php echo $id; ?>" value="1" <?php
		if ($field && $field->showinform == 1)
		{
			echo 'checked="checked"';
		}
		if (!$field)
		{
			echo 'checked="checked"';
		}
		?> />
			</label>
			<label for="showinform0<?php echo $id; ?>" class="btn radio"><?php echo Text::_("RSVP_NO"); ?>
			<input type="radio"  name="showinform[<?php echo $id; ?>]" id="showinform0<?php echo $id; ?>" value="0" <?php
		   if ($field && $field->showinform == 0)
		   {
			   echo 'checked="checked"';
		   }
		?> />
			</label>
		</div>
		<div class="rsvpclear"></div>
		</div>
		<?php

	}

	static function showinlist($id, $field, $default = 1)
	{
		$params = ComponentHelper::getParams("com_rsvppro");
		$style = $params->get("standardvisibility",1)?'' : 'style="display:none"';
		?>
		<div class="rsvpnotranslate">
		<div class="gsl-form-label" <?php echo $style;?> ><?php echo Text::_("RSVP_SHOW_IN_LIST"); ?></div>
		<div class="gsl-form-controls  radio btn-group" <?php echo $style;?> >
			<label for="showinlist1<?php echo $id; ?>" class="btn radio"><?php echo Text::_("RSVP_YES"); ?>
			<input type="radio"  name="showinlist[<?php echo $id; ?>]"  id="showinlist1<?php echo $id; ?>" value="1" <?php
		if ($field && $field->showinlist == 1)
		{
			echo 'checked="checked"';
		}
		if (!$field && $default)
		{
			echo 'checked="checked"';
		}
		?> />
			</label>
			<label for="showinlist0<?php echo $id; ?>" class="btn radio"><?php echo Text::_("RSVP_NO"); ?>
			<input type="radio"  name="showinlist[<?php echo $id; ?>]" id="showinlist0<?php echo $id; ?>" value="0" <?php
		   if ($field && $field->showinlist == 0)
		   {
			   echo 'checked="checked"';
		   }
		   if (!$field && !$default)
		   {
			   echo 'checked="checked"';
		   }
		?> />
			</label>
		</div>
		<div class="rsvpclear"></div>
		</div>
		<?php

	}

	static function showinticket($id, $field, $default = 1)
	{
		$style = '';
		?>
		<div class="rsvpnotranslate">
			<div class="gsl-form-label" <?php echo $style;?> ><?php echo Text::_("RSVP_SHOW_IN_TICKETS"); ?></div>
			<div class="gsl-form-controls  radio btn-group" <?php echo $style;?> >
				<label for="showinticket1<?php echo $id; ?>" class="btn radio"><?php echo Text::_("RSVP_YES"); ?>
					<input type="radio"  name="showinticket[<?php echo $id; ?>]"  id="showinticket1<?php echo $id; ?>" value="1" <?php
					if ($field && $field->showinticket == 1)
					{
						echo 'checked="checked"';
					}
					if (!$field && $default)
					{
						echo 'checked="checked"';
					}
					?> />
				</label>
				<label for="showinticket0<?php echo $id; ?>" class="btn radio"><?php echo Text::_("RSVP_NO"); ?>
					<input type="radio"  name="showinticket[<?php echo $id; ?>]" id="showinticket0<?php echo $id; ?>" value="0" <?php
					if ($field && $field->showinticket == 0)
					{
						echo 'checked="checked"';
					}
					if (!$field && !$default)
					{
						echo 'checked="checked"';
					}
					?> />
				</label>
			</div>
			<div class="rsvpclear"></div>
		</div>
		<?php

	}

	static function showindetail($id, $field, $default = 1)
	{
		$params = ComponentHelper::getParams("com_rsvppro");
		$style = $params->get("standardvisibility",1)?'' : 'style="display:none"';
		?>
		<div class="rsvpnotranslate">
		<div class="gsl-form-label" <?php echo $style;?> ><?php echo Text::_("RSVP_SHOW_IN_DETAIL"); ?></div>
		<div class="gsl-form-controls  radio btn-group" <?php echo $style;?> >
			<label for="showindetail1<?php echo $id; ?>" class="btn radio"><?php echo Text::_("RSVP_YES"); ?>
			<input type="radio"  name="showindetail[<?php echo $id; ?>]"  id="showindetail1<?php echo $id; ?>" value="1" <?php
		if ($field && $field->showindetail == 1)
		{
			echo 'checked="checked"';
		}
		if (!$field && $default)
		{
			echo 'checked="checked"';
		}
		?> />
			</label>
			<label for="showindetail0<?php echo $id; ?>" class="btn radio"><?php echo Text::_("RSVP_NO"); ?>
			<input type="radio"  name="showindetail[<?php echo $id; ?>]" id="showindetail0<?php echo $id; ?>" value="0" <?php
		   if ($field && $field->showindetail == 0)
		   {
			   echo 'checked="checked"';
		   }
		  if (!$field && !$default)
		   {
			   echo 'checked="checked"';
		   }
		?> />
			</label>
		</div>
		<div class="rsvpclear"></div>
		</div>
		<?php

	}

	static function includeinexport($id, $field)
	{
		$params = ComponentHelper::getParams("com_rsvppro");
		$style = $params->get("standardvisibility",1)?'' : 'style="display:none"';

		$fparams = new stdClass();
		if ($field)
		{
			try {
				$fparams = json_decode($field->params);
			}
			catch (Exception $e) {

			}
		}
		if (isset($fparams->inexport))
		{
			$inexport = $fparams->inexport;
		}
		else
		{
			$inexport = 1;
		}

		?>
		<div class="rsvpnotranslate">
		<div class="gsl-form-label" <?php echo $style;?> ><?php echo Text::_("RSVP_INCLUDE_IN_EXPORT"); ?></div>
		<div class="gsl-form-controls  radio btn-group" <?php echo $style;?> >
			<label for="inexport1<?php echo $id; ?>" class="btn radio"><?php echo Text::_("RSVP_YES"); ?>
			<input type="radio"  name="params[<?php echo $id; ?>][inexport]"  id="inexport1<?php echo $id; ?>" value="1" <?php
		if ($inexport == 1)
		{
			echo 'checked="checked"';
		}
		if (!$field)
		{
			echo 'checked="checked"';
		}
		?> />
			</label>
			<label for="inexport0<?php echo $id; ?>" class="btn radio"><?php echo Text::_("RSVP_NO"); ?>
			<input type="radio"  name="params[<?php echo $id; ?>][inexport]" id="inexport0<?php echo $id; ?>" value="0" <?php
		   if ($field && $inexport == 0)
		   {
			   echo 'checked="checked"';
		   }
		?> />
			</label>
		</div>
		<div class="rsvpclear"></div>
		</div>
		<?php

	}

	static function applicableCategories($fieldids, $fieldidc, $id, $categories)
	{
		return "";
		?>
		<div class="rsvpnotranslate">
		<div class="gsl-form-label"><?php echo Text::_("RSVP_APPLICABLE_CATEGORIES"); ?></div>
		<div class="gsl-form-controls">
			<?php
			static $donescript;
			if (!isset($donescript))
			{
				$doc = Factory::getDocument();
				$script = <<<SCRIPT
		   		function allselections(id) {
		   			var e = document.getElementById(id);
		   			e.disabled = true;
		   			var i = 0;
		   			var n = e.options.length;
		   			for (i = 0; i < n; i++) {
		   				e.options[i].disabled = true;
		   				e.options[i].selected = true;
		   			}
		   		}
		   		function enableselections(id) {
		   			var e = document.getElementById(id);
		   			e.disabled = false;
		   			var i = 0;
		   			var n = e.options.length;
		   			for (i = 0; i < n; i++) {
		   				e.options[i].disabled = false;
		   			}
		   		}
SCRIPT;
				$doc->addScriptDeclaration($script);
				$donescript = 1;
			}

			$fieldidcStripped = str_replace(array("[", "]"), "", $fieldidc);

			JLoader::register('JEventsCategory', JEV_ADMINPATH . "/libraries/categoryClass.php");

			$cattree = JEventsCategory::categoriesTree();
			$categorylist = HTMLHelper::_('select.genericlist', $cattree, $fieldidc . '[]', 'multiple="multiple" size="15"', 'value', 'text', explode("|", $categories));
			?>
			<fieldset class="adminform">
				<legend><?php echo Text::_('RSVP_APPLICABLE_CATEGORIES'); ?></legend>
				<table class="admintable" cellspacing="1">
					<tr>
						<td class="key">
							<?php echo Text::_('RSVP_CATEGORIES'); ?>:
						</td>
						<td class="gsl-form-controls  radio btn-group">
							<?php
							if ($categories == 'all' || $categories == '')
							{
								?>
								<label for="categories-all<?php echo $fieldidc; ?>" class="btn radio">
									<input id="categories-all<?php echo $fieldidc; ?>" type="radio"  name="<?php echo $fieldids; ?>" value="all" onclick="allselections('<?php echo $fieldidcStripped; ?>');" checked="checked" />
									<?php echo Text::_('RSVP_ALL'); ?>
								</label>
								<label for="categories-select<?php echo $fieldidc; ?>" class="btn radio">
									<input id="categories-select<?php echo $fieldidc; ?>" type="radio"  name="<?php echo $fieldids; ?>" value="select" onclick="enableselections('<?php echo $fieldidcStripped; ?>');" />
									<?php echo Text::_('RSVP_SELECT_FROM_LIST'); ?>
								</label>
								<?php
							}
							else
							{
								?>
								<label for="categories-all<?php echo $fieldidc; ?>" class="btn radio">
									<input id="categories-all<?php echo $fieldidc; ?>" type="radio"  name="<?php echo $fieldids; ?>" value="all" onclick="allselections('<?php echo $fieldidcStripped; ?>');" />
									<?php echo Text::_('RSVP_ALL'); ?>
								</label>
								<label for="categories-select<?php echo $fieldidc; ?>" class="btn radio">
									<input id="categories-select<?php echo $fieldidc; ?>" type="radio"  name="<?php echo $fieldids; ?>" value="select" onclick="enableselections('<?php echo $fieldidcStripped; ?>');" checked="checked" />
									<?php echo Text::_('RSVP_SELECT_FROM_LIST'); ?>
								</label>
							<?php } ?>
						</td>
					</tr>
					<tr>
						<td class="paramlist_key" width="40%"><span class="editlinktip">
								<label for="categories<?php echo $fieldidc; ?>" class="categories-lbl"><?php echo Text::_('RSVP_CATEGORY_SELECTION'); ?></label>
						</td>
						<td>
							<?php echo $categorylist; ?>
						</td>
					</tr>
				</table>
				<?php
				if ($categories == 'all' || $categories == '')
				{
					?>
					<script type="text/javascript">allselections('<?php echo $fieldidcStripped; ?>');</script>
				<?php } ?>
			</fieldset>
		</div>
		</div>
		<?php

	}

	static function phpMoneyFormat($amount)
	{
		// Fetch reference to current row and rsvpdata to the registry so that we have access to these in the fields
		$registry = JevRegistry::getinstance("jevents");
		$rsvpdata = $registry->get("rsvpdata");

		$digits = self::getTemplateSpecificParameter($rsvpdata, "CurrencyDigits",0);
		$symbol = self::getTemplateSpecificParameter($rsvpdata, "CurrencySymbol", '$');
		$onLeft = (strcmp(self::getTemplateSpecificParameter($rsvpdata, "CurrencyPlacement", 'left'), 'left') == 0);
		$separator = self::getTemplateSpecificParameter($rsvpdata, "CurrencySeparator", ",");
		$decimal = self::getTemplateSpecificParameter($rsvpdata, "CurrencyDecimal", ".");

		// People keep adding symbols! lets remove them.
		$amount = str_replace($symbol, '', $amount);

		$formattedText = '';

		// negative amount

		if ($onLeft)
			$formattedText .= $symbol;

		if ($amount < 0)
			$formattedText .= '-';

		// currency symbol on the left

		// format with correct number of digits and separator
		if ($digits > 0)
		{
			// Floating point errors are a MAJOR PAIN e.g. amount 156.80 -> 156.81 with this code
			//$factor = pow(10, $digits);
			//$amount = ceil(abs($amount)*$factor)/$factor;
			$amount = RsvpHelper::ceil_dec($amount, $digits, ".");
			if ($amount==0){
				$formattedText = '';
				if ($onLeft)
					$formattedText .= $symbol;
			}
			/*
			 * test cases
			 */
			/*
			$test = 123.456789;
			$test2 = RsvpHelper::ceil_dec($test, $digits, ".");

			$test = 123.00004;
			$test2 = RsvpHelper::ceil_dec($test, $digits, ".");

			$test = 123.9999999999;
			$test2 = RsvpHelper::ceil_dec($test, $digits, ".");

			$test = 123.000000000001;
			$test2 = RsvpHelper::ceil_dec($test, $digits, ".");

			$test = -123.456789;
			$test2 = RsvpHelper::ceil_dec($test, $digits, ".");

			$test = -123.00004;
			$test2 = RsvpHelper::ceil_dec($test, $digits, ".");

			$test = -123.9999999999;
			$test2 = RsvpHelper::ceil_dec($test, $digits, ".");

			$test = -123.00000000001;
			$test2 = RsvpHelper::ceil_dec($test, $digits, ".");
			*/
			$formattedText .= number_format( abs($amount), $digits, $decimal, $separator );
			//$formattedText .= abs(number_format($amount, $digits, $decimal, $separator));
		}
		else
		{
			$amount = ceil(abs($amount));
			$formattedText .= $amount;
		}

		// currency symbols on the right
		if (!$onLeft)
			$formattedText .= $symbol;


		return $formattedText;

	}

	// see http://php.net/manual/en/function.ceil.php#108370 - IMPROVED THOUGH!
	static public function ceil_dec($number, $precision, $separator = ".")
	{
		// If the whole number is less than .5 under the precision then return zero
		if (abs(floatval($number))*pow(10, $precision)<0.5){
			return 0;
		}

		$numberpart = explode($separator, $number);
		if (count($numberpart)==1){
		    // Found a situation where float 9 was converted to 8 by this code
			//return intval($number);
            // so use this instead
			$numberpart[1] = "0000";
		}
		// $numberpart[1] could have a leading zero so we make it a LOT bigger and then remove this at the end
		$leadingZero = false;
		if (substr($numberpart[1], 0, 1) === "0") {
			$numberpart[1] = "5" . substr($numberpart[1],1);
			$leadingZero = true;
		}
		$numberpart[1] = substr_replace($numberpart[1], $separator, $precision, 0);
		if ($numberpart[0] >= 0)
		{
			$numberpart[1] = ceil($numberpart[1]);
			if (strlen($numberpart[1]) > $precision){
				$numberpart[0] +=1;
				$numberpart[1] = substr($numberpart[1],1);
			}
		}
		else
		{
			$numberpart[1] = floor($numberpart[1]);
			if (strlen($numberpart[1]) > $precision){
				$numberpart[0] -=1;
				$numberpart[1] = substr($numberpart[1],1);
			}

		}

		if ($leadingZero){
			$lead = intval(substr($numberpart[1], 0, 1)) - 5;
			$numberpart[1] = ((string) $lead) . substr($numberpart[1],1);
		}
		$ceil_number = array($numberpart[0], $numberpart[1]);
		return implode($separator, $ceil_number);

	}

	static function phpNewMoneyFormat($rsvpdata, $amount) {
        $digits 	= RsvpHelper::getTemplateSpecificParameter($rsvpdata, 'CurrencyDigits');
        $symbol 	= RsvpHelper::getTemplateSpecificParameter($rsvpdata, 'CurrencySymbol');
        $onLeft 	= (strcmp(RsvpHelper::getTemplateSpecificParameter($rsvpdata, 'CurrencyPlacement', 'left'),'left') == 0);
        $separator 	= RsvpHelper::getTemplateSpecificParameter($rsvpdata, 'CurrencySeparator');
        $decimal	= RsvpHelper::getTemplateSpecificParameter($rsvpdata, 'CurrencyDecimal');

		$formattedText  = '';

		// negative amount
		if ( $amount < 0 ) $formattedText .= '-';

		// currency symbol on the left
		if ( $onLeft )     $formattedText .= $symbol;

		// format with correct number of digits and separator
		if ( $digits > 0 ) {
			$amount = RsvpHelper::ceil_dec($amount, $digits, ".");
			$formattedText .= number_format( abs($amount), $digits, $decimal, $separator );
		} else {
			$formattedText = abs( $amount );
		}

		// currency symbols on the right
		if ( !$onLeft )    $formattedText .= $symbol;


		return $formattedText;
	}

	static function jsMoneyFormat()
	{

		static $loaded;
		if (isset($loaded))
			return;

		$loaded = true;
		$params = ComponentHelper::getParams("com_rsvppro");

		$digits = $params->get("CurrencyDigits", 2);
		$symbol = $params->get("CurrencySymbol", "$");
		$onLeft = (strcmp($params->get("CurrencyPlacement"), 'left') == 0);
		$separator = $params->get("CurrencySeparator", ",");
		$decimal = $params->get("CurrencyDecimal",".");

		$jsCode = "
			function moneyFormat(amount) {
				// ensure numerical input
				amount = parseFloat(amount);

				// Get the ceiling amount to match the payment plugin process
				var factor = Math.pow(10, " . $digits . ");
				amount = Math.ceil(amount * factor)/factor;

				// format to the correct number of digits
				// @todo prototype needs to implement toFixed() for browsers that don't support this.
				amount = amount.toFixed(" . $digits . ");
				
				// split into whole/partial for thousands separator
				var dollars = amount.split('.')[0];
				var cents	= amount.split('.')[1];
	
				// apply separator between every three digits
				var rgx = /(\d+)(\d{3})/;
				while (rgx.test(dollars)) {
					dollars = dollars.replace(rgx, '\$1' + '$separator' + '\$2');
				}
				
				";
		if ($onLeft)
		{
			if ($digits > 0)
			{
				$jsCode .= "return '$symbol' + dollars + '$decimal' + cents;\n";
			}
			else
			{
				$jsCode .= "return '$symbol' + dollars;\n";
			}
		}
		else
		{
			if ($digits > 0)
			{
				$jsCode .= "return dollars + '$decimal' + cents + '$symbol';\n";
			}
			else
			{
				$jsCode .= "return dollars + '$symbol';\n";
			}
		}
		$jsCode .= "
			}";

		$document = Factory::getDocument();
		$document->addScriptDeclaration($jsCode);

	}

	public static function 	translate($string){
		// Is there a translation - simple translation
		$translation = Text::_($string);
		if($translation != $string) {
			return $translation;
		}
		// more name spaced!
		$totranslate = str_replace(" ","_", trim($string) );
		$totranslate = "RSVP_FIELD_" . (string) preg_replace('/[^A-Z0-9_]/i', '', $totranslate);
		$translation = Text::_($totranslate);
		if($translation != $totranslate) {
			return $translation;
		}
		return $string;
	}

	static function conditional($id, $field)
	{
		$params = ComponentHelper::getParams("com_rsvppro");
		if (!$params->get("allowconditional",1)){
			return ;
		}

		if ($field)
		{
			try {
				$params = json_decode($field->params);
			}
			catch (Exception $e) {
				$params = array();
			}
		}
		if (isset($params->cf))
		{
			$cf = $params->cf;
		}
		else
		{
			$cf = "";
		}
		if (isset($params->cfvfv))
		{
			$cfvfv = $params->cfvfv;
		}
		else
		{
			$cfvfv = 1;
		}
		$params = ComponentHelper::getParams("com_rsvppro");
		$style = 'style="display:none"';

		?>
		<div class="rsvpnotranslate rsvpconditional" <?php echo $style;?>>
		<div class="gsl-form-label"   ><?php echo Text::_("RSVP_CONDITIONAL"); ?></div>
		<div class="gsl-form-controls"   >
			<div class="gsl-form-label" ><label for="cf<?php echo $id; ?>"><?php echo Text::_("RSVP_CONDITION_FIELD"); ?></label></div>
			<div class="gsl-form-controls">
				<select name="params[<?php echo $id; ?>][cf]" id="cf<?php echo $id; ?>" class="cf" onchange="conditionalEditorPlugin.updateSelection('cf<?php echo $id; ?>');"	>
				<option value=""><?php echo Text::_("RSVP_NOT_CONDITIONAL"); ?></option>
				</select>
			</div>
			<div style="clear:left;" ></div>
			<div class="rsvpconditionalselector"  <?php echo $style;?>>
				<div class="gsl-form-label" ><label for="cfvfv<?php echo $id; ?>"><?php echo Text::_("RSVP_CONDITION_VISIBLE_FIELDVALUE"); ?></label></div>
				<div class="gsl-form-controls">
                                        <?php // we must store this value in a hidden field since javascript will use it to populate the correct selected option ?>
                                        <input type="hidden" id="hiddencfvfv<?php echo $id; ?>" value="<?php echo $cfvfv;?>" />
					<select name="params[<?php echo $id; ?>][cfvfv]" id="cfvfv<?php echo $id; ?>" class="cfvfv" >
						<option value="1" <?php echo $cfvfv?"selected='selected'":"";?>><?php echo Text::_("RSVP_YES"); ?></option>
						<option value="0" <?php echo !$cfvfv?"selected='selected'":"";?>><?php echo Text::_("RSVP_NO"); ?></option>
					</select>
				</div>
			</div>

		</div>
		<?php
		static $scripts = array();
		$script = 'jQuery(document).ready(function(){if (conditionalEditorPlugin) conditionalEditorPlugin.update("' . $id . '", "' . $cf . '");});';
		if (!in_array($script, $scripts)){
			$doc = Factory::getDocument();
			$doc->addScriptDeclaration($script);
			$scripts[]  = $script;
		}
		?>
		<div class="rsvpclear"></div>
		</div>
		<?php

	}

	// This method overrides global component parameters with values from template params
	static public function getTemplateParams($rsvpdata, $overrideGlobal = true){
		static $paramsarray;
		if (!isset($paramsarray)){
			$paramsarray = array();
		}
		if (!isset($paramsarray[$rsvpdata->id])){
			$comparams = ComponentHelper::getParams("com_rsvppro");

			if (isset($rsvpdata->template) && is_numeric($rsvpdata->template))
			{
				$db = Factory::getDBO();
				$db->setQuery("Select params from #__jev_rsvp_templates where id=" . intval($rsvpdata->template));
				$templateParams = $db->loadObject();
				if ($templateParams)
				{
					$templateParams = new \Joomla\Registry\Registry($templateParams->params);
				}
				else
				{
					$templateParams = new \Joomla\Registry\Registry();
				}

				/* Override template params with payment plugin specifc params where required */
    			$rsvpPlugins = PluginHelper::getPlugin("rsvppro"); // gets all enabled rsvp plugins
    			foreach ($rsvpPlugins as $plugin) {
    				if (empty($plugin->params)) {
						continue;	// only concerned with plugins which have parms
    				}
    				$pluginClass = 'plgRsvppro'.ucfirst($plugin->name);
    				if (class_exists($pluginClass) === false || method_exists($pluginClass, "usePluginSettingsName") === false) {
						continue;
    				}
    				$usePluginSettingsName = $pluginClass::usePluginSettingsName();
    				if ($templateParams->get($usePluginSettingsName, '0') == '0')
				    {
					    continue;    // don't want to use plugin params
				    }

				    /* Get the gateway params and merge them */
				    $templateParams->merge(new \Joomla\Registry\Registry($plugin->params));
    			}

				if ($overrideGlobal)
				{
					$comparams->merge($templateParams);
				}
			}
			$paramsarray[$rsvpdata->id] = RsvpHelper::translateTemplateParams($rsvpdata->template, $comparams);;
		}
		return $paramsarray[$rsvpdata->id];
	}

	static public function getRawTemplateParams($rsvpdata){
		$rsvpdataid = isset($rsvpdata->id) ? $rsvpdata->id : 0;
		static $templateParamsArray;
		$comparams = ComponentHelper::getParams("com_rsvppro");
		if (!isset($templateParamsArray)){
			$templateParamsArray = array();
		}
		if (!isset($templateParamsArray[$rsvpdataid])){

			if (isset($rsvpdata->template) && is_numeric($rsvpdata->template))
			{
				$db = Factory::getDBO();
				$db->setQuery("Select params from #__jev_rsvp_templates where id=" . intval($rsvpdata->template));
				$templateParams = $db->loadObject();
				if ($templateParams)
				{
					$templateParamsArray[$rsvpdataid] = new \Joomla\Registry\Registry($templateParams->params);
				}
			}

			if (!isset($templateParamsArray[$rsvpdataid]))
			{
				$templateParamsArray[$rsvpdataid] = $comparams;
			}

			/* Override template params with payment plugin specifc params where required */
			$rsvpPlugins = PluginHelper::getPlugin("rsvppro"); // gets all enabled rsvp plugins
			foreach ($rsvpPlugins as $plugin) {
				if (empty($plugin->params)) {
					continue;	// only concerned with plugins which have parms
				}
				$pluginClass = 'plgRsvppro'.ucfirst($plugin->name);
				if (class_exists($pluginClass) === false || method_exists($pluginClass, "usePluginSettingsName") === false) {
					continue;
				}
				$usePluginSettingsName = $pluginClass::usePluginSettingsName();
				if ($templateParamsArray[$rsvpdataid]->get($usePluginSettingsName, '0') == '0')
				{
					continue;    // don't want to use plugin params
				}

				/* Get the gateway params and merge them */
				$templateParamsArray[$rsvpdataid]->merge(new \Joomla\Registry\Registry($plugin->params));
			}
			if (isset($rsvpdata->template))
			{
				$templateParamsArray[$rsvpdataid] = RsvpHelper::translateTemplateParams($rsvpdata->template, $templateParamsArray[$rsvpdataid]);
			}

		}
		return $templateParamsArray[$rsvpdataid];
	}

	public static function getTemplateSpecificParameter($rsvpdata, $fieldname, $defaultValue = null) {
		$comparams = ComponentHelper::getParams("com_rsvppro");
		$templateParams = RsvpHelper::getRawTemplateParams($rsvpdata);

		$paramValue = $templateParams->get($fieldname, null);
		if (is_null($paramValue))
        {
	        $paramValue = $comparams->get($fieldname, $defaultValue);
        }
		return $paramValue;
	}

	public static function setTemplateSpecificParameter($rsvpdata, $fieldname, $value) {
		$comparams = JComponentHelper::getParams("com_rsvppro");
		$templateParams = RsvpHelper::getRawTemplateParams($rsvpdata);

		$templateParams->set($fieldname, $value);
	}

	/**
	 * Converts a double colon seperated string or 2 separate strings to a string ready for bootstrap tooltips
	 *
	 * @param   string  $title      The title of the tooltip (or combined '::' separated string).
	 * @param   string  $content    The content to tooltip.
	 * @param   int     $translate  If true will pass texts through Text.
	 * @param   int     $escape     If true will pass texts through htmlspecialchars.
	 *
	 * @return  string  The tooltip string
	 *
	 * @since   3.1.17
	 */
	public static function tooltipText($title = '', $content = '', $translate = 1, $escape = 1)
	{
		// Return empty in no title or content is given.
		if ($title == '' && $content == '')
		{
			return '';
		}

		// Split title into title and content if the title contains '::' (old Mootools format).
		if ($content == '' && !(strpos($title, '::') === false))
		{
			list($title, $content) = explode('::', $title, 2);
		}

		// Pass texts through the Text.
		if ($translate)
		{
			$title = Text::_($title);
			$content = Text::_($content);
		}

		// Escape the texts.
		if ($escape)
		{
			$title = str_replace('"', '&quot;', $title);
			$content = str_replace('"', '&quot;', $content);
		}

		// Return only the content if no title is given.
		if ($title == '')
		{
			return $content;
		}

		if (version_compare(JVERSION, "3.2.0", 'ge')){

			// Return only the title if title and text are the same.
			if ($title == $content)
			{
				return '<strong>' . $title . '</strong>';
			}

			// Return the formated sting combining the title and  content.
			if ($content != '')
			{
				return '<strong>' . $title . '</strong><br />' . $content;
			}
		}
		else {
			return $title."::".$content;
		}


		// Return only the title.
		return $title;
	}


        public static function translateTemplateParams($templateid, $params)
        {
            // Do we have a translation for each of the messages
	        $db = Factory::getDbo();
	        $jinput = Factory::getApplication()->input;
	        $lang = Factory::getLanguage();
            $lang = $jinput->getString("lang", $lang->getTag());
            $db->setQuery("SELECT params FROM #__jev_rsvp_template_translation where template_id = ".intval($templateid) . " AND language = ". $db->quote($lang));
            $tempdata = $db->loadObject();
            if ($tempdata){
                    $tempdata = json_decode($tempdata->params);
                    foreach (get_object_vars($params) as $key => $val) {
                            if (isset($tempdata->$key)){
                                    $params->$key = $tempdata->$key;
                            }
                    }
            }
            return $params;
        }

	static public function showOnRelField($field)
	{
		$rel = "";
		if ($field && isset($field['showon']) && !empty($field['showon']))
		{
			HTMLHelper::_('jquery.framework');
			HTMLHelper::_('script', 'jui/cms.js', array('version' => 'auto', 'relative' => true));

			$rel           = ' data-showon=\'' .
				json_encode(JFormHelper::parseShowOnConditions($field['showon'], 'params')) . '\'';
		}
		return $rel;
	}

}
