<?php

/**
 * copyright (C) 2008-2021 GWE Systems Ltd - All rights reserved
 */
// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die();

use Joomla\CMS\Table\Table;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;

/**
 * Template Table class
 *
 */
class TableTemplate  extends Table
{

	/**
	 * Primary Key
	 *
	 * @var int
	 */
	var $id = null;

	/**
	 * @var string
	 */
	var $title = null;

	/**
	 * @var string
	 */
	var $description = null;

	/**
	 * @var int
	 * published on by default
	 */
	var $published = 1;

	/**
	 * @var datetime
	 */
	var $created = null;

	/**
	 * @var int
	 */
	var $created_by = null;

	/**
	 * @var int
	 */
	var $modified_by = null;

	/**
	 * @var booelan
	 */
	var $withfees = 0;

	/**
	 * @var booelan
	 */
	var $withticket = 0;

	/**
	 * @var booelan
	 */
	var $global = 0;

	/**
	 * @var booelan
	 */
	var $istemplate = 1;

	/**
	 * @var string
	 */
	var $params = "";

	/**
	 * @var string
	 */
	var $ticket = "";
	var $ticketcss = "";
	var $notes = "";

	var $fields = array();

	/**
	 * Constructor
	 *
	 * @param object Database connector object
	 * @since 1.0
	 */
	function __construct()
	{
		$db = Factory::getDBO();
		parent::__construct('#__jev_rsvp_templates', 'id', $db);
	}

	/**
	 * Overloaded bind function
	 *
	 * @acces public
	 * @param array $hash named array
	 * @return null|string	null is operation was satisfactory, otherwise returns an error
	 * @see Table:bind
	 * @since 1.5
	 */
	public function bind($array, $ignore = array(), $fieldid = "")
	{
		$success = parent::bind($array, $ignore);
		// now merge in the jform parts
		if ($success && array_key_exists("jform", $array))
		{
			//parent::bind($array["jform"], $ignore);
		}
		if (array_key_exists('type', $array))
		{
			foreach ($array['type'] as $fieldid => $fieldtype)
			{
				$id = str_replace("field", "", $fieldid);
				$tableclass = "Table" . ucfirst($fieldtype);
				if (!class_exists($tableclass) && file_exists(dirname(__FILE__) . "/field." . $fieldtype . ".php"))
				{
					include_once(dirname(__FILE__) . "/field." . $fieldtype . ".php");
					$field = new $tableclass();
				}
				else
				{
					include_once(dirname(__FILE__) . "/field.php");
					$field = new TableField();
				}
				$field->bind($array, '', $fieldid);

				$this->fields[] = $field;
			}
		}
		if (array_key_exists('params', $array) && is_array($array["params"]))
		{
			$this->params = json_encode($array["params"]);
		}
		return $success;
	}

	/**
	 * Overloaded bind function that creates a copy to be saved
	 *
	 * @acces public
	 * @param array $hash named array
	 * @return null|string	null is operation was satisfactory, otherwise returns an error
	 * @see Table:bind
	 * @since 1.5
	 */
	function bindcopy($array, $ignore = '')
	{
		$success = parent::bind($array, $ignore);

		$this->id = 0;
		$this->created_by = null;
		$this->created = null;
		if (array_key_exists('type', $array))
		{
			foreach ($array['type'] as $fieldid => $fieldtype)
			{
				$id = str_replace("field", "", $fieldid);
				$tableclass = "Table" . ucfirst($fieldtype);
				if (!class_exists($tableclass) && file_exists(dirname(__FILE__) . "/field." . $fieldtype . ".php"))
				{
					include_once(dirname(__FILE__) . "/field." . $fieldtype . ".php");
					$field = new $tableclass();
				}
				else
				{
					include_once(dirname(__FILE__) . "/field.php");
					$field = new TableField();
				}
				$field->bind($array, '', $fieldid);
				$field->_old_field_id = $field->field_id;
				$field->field_id = 0;
				$field->template_id = 0;

				$this->fields[] = $field;
			}
		}
		if (array_key_exists('params', $array) && is_array($array["params"]))
		{
			$this->params = json_encode($array["params"]);
		}
		return $success;
	}

	/**
	 * Overloaded bind function that creates a copy to be saved
	 *
	 * @acces public
	 * @param array $hash named array
	 * @return null|string	null is operation was satisfactory, otherwise returns an error
	 * @see Table:bind
	 * @since 1.5
	 */
	function bindimport($template, $ignore = '')
	{
		$success = parent::bind($template, $ignore);

		$this->id         = 0;
		$this->created_by = null;
		$this->created    = null;
		$this->fields     = array();

		/*
		 * ToDo
		 * Fields
		 * 1. extract field ids from $fields array
		 * 2. Sort from longest to shortest
		 * 3. Replace fieldids in params with placeholders longest to shorted so that field11 is done before field1  etc.
		 * 4. After storing the template get the new template id and pt in the fields and then save the fields based on their type
		 * 5. Making sure we wet the fieldis to zero before saving
		 * 6. Keep a mapping of the field ids to the old
		 * 7. Replace the placeholders in the template parameters and re-save the template
		 */

		$fields = $template['fields'];
		unset($template['fields']);

		$params = json_decode($template['params']);
/*
		$fieldsByType = array();
		foreach ($fields as $field)
		{
			$fieldsByType[$field['field_id']] = $field['type'];
			$template[$field['field_id']] = $field;
		}
		$template['type'] = $fieldsByType;
*/

		foreach ($fields as $fieldData)
		{
			$id = str_replace("field", "", $fieldData['field_id']);
			include_once(dirname(__FILE__) . "/field.php");

			$field = new TableField();
			$field->bindImport($fieldData);
			$field->_old_field_id = $fieldData['field_id'];
			$field->field_id = 0;
			$field->template_id = 0;

			$this->fields[] = $field;
		}
		if (array_key_exists('params', $template) && is_array($template["params"]))
		{
			$this->params = json_encode($template["params"]);
		}
		return $success;
	}

	/*
	 * overloaded load method that converts the params field
	 */
	function load($keys = NULL, $reset = truel)
	{
		parent::load($keys);

		if ($this->params != "")
		{
			//		$this->params = json_decode($this->params);
		}
	}

	/**
	 * Overloaded check method to ensure data integrity
	 *
	 * @access public
	 * @return boolean True on success
	 * @since 1.0
	 */
	function check()
	{

		/** check for valid name */
		if (trim($this->title) == '')
		{
			$this->setError(Text::_('Your Template must contain a title.'));
			return false;
		}

		/** check for existing name */
		$query = 'SELECT id FROM #__jev_rsvp_templates WHERE title = ' . $this->_db->Quote($this->title);
		$user = Factory::getUser();
		$query .= " AND (created_by = $user->id OR global = 1)";

		$this->_db->setQuery($query);

		$hasPaymentMethod = false;
		$hasBalances = false;
		// check that the payment method and balance elements are included
		foreach (array_keys($this->fields) as $index)
		{
			if ($this->fields[$index]->type == "jevrpaymentoptionlist")
			{
				$hasPaymentMethod = true;
			}
			else if ($this->fields[$index]->type == "jevrbalance")
			{
				$balanceParams = json_decode($this->fields[$index]->params);
				if (isset($balanceParams->balancetype) && $balanceParams->balancetype == "outstanding")
				{
					$hasBalances = true;
				}
			}
		}


		$xid = intval($this->_db->loadResult());
		if ($xid && $xid != intval($this->id))
		{
			// Change the title to somthing unique
			$query = "SELECT max(id) FROM  #__jev_rsvp_templates";
			$this->_db->setQuery($query);
			$nextId = $this->_db->loadResult();

			// reset the field ids too!
			foreach (array_keys($this->fields) as $index)
			{
				// keep the old field id for adjusting custom fields afterwards
				$this->fields[$index]->_old_field_id = $this->fields[$index]->field_id;
				$this->fields[$index]->field_id = 0;
				$this->fields[$index]->template_id = intval($this->id);
			}

			if (Factory::getApplication()->input->getInt("customise", 0))
			{
				$this->title .= " " . ($nextId + 1);
				echo "<script type='text/javascript'>alert('" . Text::_("JEV_TEMPLATE_TITLE_NOT_UNIQUE", true) . "');</script>";
			}
			else
			{
				Factory::getApplication()->enqueueMessage(Text::_("JEV_TEMPLATE_TITLE_NOT_UNIQUE"));
				$this->setError(Text::sprintf('RSVPPRO_WARNNAMETRYAGAIN', $this->title));
				return false;
			}
		}
		// if a new template based on an old one must remember to set the field ids and template ids to 0 in the fields!
		else if (intval($this->id) == 0)
		{
			// reset the field ids too!
			foreach (array_keys($this->fields) as $index)
			{
				// keep the old field id for adjusting custom fields afterwards
				if ($this->fields[$index]->field_id > 0)
				{
					$this->fields[$index]->_old_field_id = $this->fields[$index]->field_id;
				}
				$this->fields[$index]->field_id = 0;
				$this->fields[$index]->template_id = 0;
			}
		}

		if ($this->withfees && !$hasPaymentMethod)
		{
			Factory::getApplication()->enqueueMessage(Text::_('JEV_TEMPLATE_HAS_FEES_BUT_NO_PAYMENT_METHOD'), "error");
		}
		if ($this->withfees && !$hasBalances)
		{
			Factory::getApplication()->enqueueMessage(Text::_('JEV_TEMPLATE_HAS_FEES_BUT_NO_OUTSTANDING_BALANCE_FIELD'), "error");
		}
		return true;
	}

	function store($updateNulls = false)
	{

		// Create the archive version first
		if ($this->id > 0)
		{
			$templatearchive = Table::getInstance("templatearchive");
			if (!$templatearchive->loadByTemplateId($this->id))
			{
				return false;
			}
			$templatearchive->store();
		}

		$user = Factory::getUser();
		if (is_null($this->created_by) || intval($this->created_by) == 0)
		{
			$this->created_by = $user->id;
		}
		else
		{
			$this->modified_by = $user->id;
		}
		if (is_null($this->created))
		{
			if (class_exists("JevDate"))
			{
				$datenow = JevDate::getDate();
			}
			else
			{
				$datenow = Factory::getDate();
			}
			$this->created = $datenow->toSql();
		}
		$success = parent::store($updateNulls);

		if (!$success)
		{
			return false;
		}

			$ids = array(0);
		$oldids = array();
		foreach ($this->fields as &$field)
		{
			$field->template_id = $this->id;
			$success |= $field->store($updateNulls);
			$ids[] = $field->field_id;
			if (isset($field->_old_field_id) && $field->_old_field_id > 0)
			{
				$oldids[$field->_old_field_id] = $field->field_id;
			}
		}
		unset($field);

		// reverse sort to make sure we get the longest field ids first!
		krsort($oldids);

		// now reset any condition fields etc. in the parameters that rely on old field ids
		// TODO make sure these match FULL strings and not substrings
		foreach ($this->fields as $field)
		{
			$changed = false;
			foreach ($oldids as $oldid => $newid)
			{
				if (strpos($field->params, '"field' . $oldid . '"'))
				{
					$field->params = str_replace('"field' . $oldid . '"', '"field' . $newid . '"', $field->params);
				}
			}
			$success |= $field->store($updateNulls);
		}

		// Now update the messages etc.
		if (count($oldids) > 0)
		{
			foreach ($oldids as $oldid => $newid)
			{
				if (strpos($this->params, '"field' . $oldid . '"'))
				{
					$this->params = str_replace('"field' . $oldid . '"', '"field' . $newid . '"', $this->params);
				}
				if (strpos($this->params, '#field' . $oldid . '#'))
				{
					$this->params = str_replace('#field' . $oldid . '#', '#field' . $newid . '#', $this->params);
				}
				if (strpos($this->ticket, '"field' . $oldid . '"'))
				{
					$this->ticket = str_replace('"field' . $oldid . '"', '"field' . $newid . '"', $this->ticket);
				}
				if (strpos($this->ticket, '#field' . $oldid . '#'))
				{
					$this->ticket = str_replace('#field' . $oldid . '#', '#field' . $newid . '#', $this->ticket);
				}
			}
			$success |= parent::store($updateNulls);
		}

		// Do we need to copy any translations?
		$copytranslations = Factory::getApplication()->input->getInt("copytranslations", 0);
		if ($copytranslations && Factory::getApplication()->input->getInt("oldid", 0) > 0)
		{
			$oldtemplateid = Factory::getApplication()->input->getInt("oldid", 0);
			$newtemplateid = $this->id;
			foreach ($this->fields as $field)
			{
				if (isset($field->_old_field_id) && $field->_old_field_id > 0)
				{
					$this->_db->setQuery("SELECT * from #__jev_rsvp_fields_translation where field_id=" . $field->_old_field_id . " AND template_id=" . $oldtemplateid);
					$oldtranslations = $this->_db->loadObjectList();
					foreach ($oldtranslations as $oldtranslation)
					{
						$tableclass = "Table" . ucfirst($field->type);
						if (!class_exists($tableclass) && file_exists(JPATH_COMPONENT_ADMINISTRATOR . "/tables/field." . $field->type . ".php"))
						{
							include_once(JPATH_COMPONENT_ADMINISTRATOR . "/tables/field." . $field->type . ".php");
							$fieldtranslation = new $tableclass();
						}
						if (class_exists($tableclass))
						{
							$fieldtranslation = new $tableclass();
						}
						else
						{
							include_once(JPATH_COMPONENT_ADMINISTRATOR . "/tables/field.php");
							$fieldtranslation = new TableField();
						}

						// Change from a normal field into a field translation (I'm being lazy!)
						//$fieldtranslation->convertToTranslation();
						//$fieldtranslation->load($oldtranslation->translation_id);
						$this->_db->setQuery("SELECT * FROM #__jev_rsvp_fields_translation where translation_id=" . $oldtranslation->translation_id);
						$existingTranslation = $this->_db->loadObject();
						if ($existingTranslation)
						{
							$vars = get_object_vars($existingTranslation);
							foreach ($vars as $k => $v)
							{
								$fieldtranslation->$k = $v;
							}
							$fieldtranslation->translation_id = 0;
							$fieldtranslation->field_id = $field->field_id;
							$fieldtranslation->template_id = $newtemplateid;
							$fieldtranslation->storeTranslation();
						}
						$oldtranslation->newtranslation = $fieldtranslation;
					}
				}
			}
			// Now copy the template translation itself after updating the field references
			$this->_db->setQuery("SELECT * FROM #__jev_rsvp_template_translation where template_id = " . $oldtemplateid);
			$oldTemplateTranslations = $this->_db->loadObjectList();

			foreach ($oldTemplateTranslations as $oldTemplateTranslation)
			{
				$ticket = $oldTemplateTranslation->ticket;
				$params = $oldTemplateTranslation->params;

				// Now update the messages etc.
				if (count($oldids) > 0)
				{
					foreach ($oldids as $oldid => $newid)
					{
						if (strpos($params, '"field' . $oldid . '"'))
						{
							$params = str_replace('"field' . $oldid . '"', '"field' . $newid . '"', $params);
						}
						if (strpos($params, '#field' . $oldid . '#'))
						{
							$params = str_replace('#field' . $oldid . '#', '#field' . $newid . '#', $params);
						}
						if (strpos($ticket, '"field' . $oldid . '"'))
						{
							$ticket = str_replace('"field' . $oldid . '"', '"field' . $newid . '"', $ticket);
						}
						if (strpos($ticket, '#field' . $oldid . '#'))
						{
							$ticket = str_replace('#field' . $oldid . '#', '#field' . $newid . '#', $ticket);
						}
					}
				}

				$ticket = $this->_db->quote($ticket);
				$params = $this->_db->quote($params);

				$this->_db->setQuery("INSERT INTO #__jev_rsvp_template_translation (template_id, title, description, ticket, notes,params, language)  "
					. "SELECT $newtemplateid, title, description, $ticket, notes, $params, language FROM #__jev_rsvp_template_translation "
					. "where template_id = " . $oldtemplateid . ' and translation_id = ' . $oldTemplateTranslation->translation_id);
				$this->_db->execute();
			}
		}

		if ($success)
		{
			// remove fields that are no longer  used
			$query = 'DELETE FROM #__jev_rsvp_fields  WHERE template_id = ' . (int) $this->id . " AND field_id NOT IN (" . implode(",", $ids) . ")";
			$this->_db->setQuery($query);
			$this->_db->execute();
		}

		return $success;
	}

}
class_alias("TableTemplate", "JTableTemplate");
