<?php
/**
 * JEvents Component for Joomla 1.5.x
 *
 * @version     $Id: view.html.php 1399 2009-03-30 08:31:52Z geraint $
 * @package     JEvents
 * @copyright   Copyright (C)  2008-2021 GWE Systems Ltd
 * @license     GNU/GPLv2, see http://www.gnu.org/licenses/gpl-2.0.html
 * @link        https://www.jevents.net
 */

// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die();

use Joomla\Registry\Registry;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Pagination\Pagination;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Factory;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Filesystem\File;
use Joomla\String\StringHelper;
use Joomla\CMS\Editor\Editor;

/**
 * HTML View class for the component
 *
 * @static
 */
class AdminTemplatesViewTemplates extends RSVPAbstractView
{

	function __construct($config = null)
	{
		parent::__construct($config);

		JLoader::register('Joomla\CMS\Toolbar\Button\JeveditButton', JPATH_COMPONENT_ADMINISTRATOR . "/libraries/jevtoolbarbuttons.php");
		JLoader::register('Joomla\CMS\Toolbar\Button\JevstandardButton', JPATH_COMPONENT_ADMINISTRATOR . "/libraries/jevtoolbarbuttons.php");
		JLoader::register('Joomla\CMS\Toolbar\Button\JevconfirmButton', JPATH_COMPONENT_ADMINISTRATOR . "/libraries/jevtoolbarbuttons.php");
		JLoader::register('Joomla\CMS\Toolbar\Button\JevscriptButton', JPATH_COMPONENT_ADMINISTRATOR . "/libraries/jevtoolbarbuttons.php");
		JLoader::register('Joomla\CMS\Toolbar\Button\JevrawButton', JPATH_COMPONENT_ADMINISTRATOR . "/libraries/jevtoolbarbuttons.php");

	}

	/**
	 * Templates display function
	 *
	 * @param template $tpl
	 */
	function overview($tpl = null)
	{
		HTMLHelper::stylesheet(  'components/'.RSVP_COM_COMPONENT.'/assets/css/rsvpadmin.css' );

		$document = Factory::getDocument();
		$document->setTitle(Text::_('RSVP_RSVP') . ' :: ' .Text::_('RSVP_SESSION_TEMPLATES'));

		// Set toolbar items for the page
		JToolBarHelper::title( Text::_( 'RSVP_SESSION_TEMPLATES' ), 'jevents' );

		//JToolBarHelper::publishList('templates.publish');
		//JToolBarHelper::unpublishList('templates.unpublish');
		JToolBarHelper::addNew('templates.edit');
		JToolBarHelper::editList('templates.edit');
		JToolBarHelper::custom('templates.editcopy','copy.png','copy.png','RSVP_COPYEDIT');
		JToolBarHelper::deleteList('RSVP_DELETE_ALL','templates.delete');
		JToolBarHelper::spacer();
		if (Factory::getApplication()->isClient('administrator')){
			JToolBarHelper::custom( 'cpanel.cpanel', 'cancel.png', 'cancel.png', 'Control_Panel', false );
			if (JevTemplateHelper::canCreateOwn())
			{
				JToolBarHelper::custom('templates.export', 'push.png', 'push.png', 'RSVP_SESSION_TEMPLATES_EXPORT', true);
				JToolBarHelper::custom('templates.import', 'pull.png', 'pull.png', 'RSVP_SESSION_TEMPLATES_IMPORT', false);
			}
		}

		RsvpproHelper::addSubmenu();

		$mainframe = Factory::getApplication();$option=Factory::getApplication()->input->getCmd("option");
		$filter_order		= $mainframe->getUserStateFromRequest( $option.'rsvp_filter_order',		'filter_order',		'tmpl.title',	'cmd' );
		$filter_order_Dir	= $mainframe->getUserStateFromRequest( $option.'rsvp_filter_order_Dir',	'filter_order_Dir',	'',				'word' );
		$search				= $mainframe->getUserStateFromRequest( $option.'rsvp_search',			'search',			'',				'string' );
		$search				= StringHelper::strtolower( $search );
		$customised			= $mainframe->getUserStateFromRequest( $option.'rsvp_customised', 'customised', 0, 'int' );
		$limit		= intval( $mainframe->getUserStateFromRequest( "templatelistlimit", 'limit', Factory::getApplication()->get("list_limit", 10)));
		$limitstart = Factory::getApplication()->input->getInt('limitstart', 0 );

		$lists = array();
		// table ordering
		$lists['order_Dir'] = $filter_order_Dir;
		$lists['order'] = $filter_order;

		// search & customised filter
		$lists['search']= $search;
		$lists['customised']= $customised;

		// state filter
		$lists['state']	= "";//HTMLHelper::_('grid.state',  $filter_state );



		// Get data from the model
		$model	= $this->getModel();
		$items		=  $this->get( 'Data');
		$total		=  $this->get( 'Total');

		jimport('joomla.html.pagination');
		$pageNav = new Pagination( $total, $limitstart, $limit  );

		$jjuser = Factory::getUser();
		$this->user = 		$jjuser;
		$this->items = 		$items;
		$this->lists = 		$lists;
		$this->pageNav =	$pageNav;

		// Only offer translations in latest version of Joomla
		if (JevJoomlaVersion::isCompatible("3.4")){
			$this->languages = $this->get('Languages');
		}
		else {
			$this->languages = null;
		}

		parent::displaytemplate($tpl);


	}

	function edit($tpl = null){

		HTMLHelper::stylesheet(  'components/'.RSVP_COM_COMPONENT.'/assets/css/rsvpadmin.css' );

		HTMLHelper::_('jquery.framework');
		HTMLHelper::script('media/com_jevents/js/Sortable.js', array('version' => JeventsHelper::JEvents_Version(false), 'relative' => false));
		HTMLHelper::script( 'components/'.RSVP_COM_COMPONENT.'/assets/js/forms.js' );

		$document = Factory::getDocument();
		$document->setTitle(Text::_('RSVP_EDIT_TEMPLATE'));

		// Set toolbar items for the page

		if (Factory::getApplication()->input->getString("task")=="templates.editcopy"){
			JToolBarHelper::title(Text::_( 'RSVP_EDIT_COPY_TEMPLATE' ), 'jevents' );
			JToolBarHelper::save('templates.savecopy');
		}
		else {
			JToolBarHelper::title(Text::_( 'RSVP_EDIT_TEMPLATE' ), 'jevents' );
			JToolBarHelper::apply('templates.apply',"RSVP_APPLY");
			JToolBarHelper::save('templates.save');
		}
		if (Factory::getApplication()->isClient('administrator') || Factory::getApplication()->input->getInt("customise")==0){
			JToolBarHelper::cancel('templates.cancel');
		}

		//$this->showToolBar();

		RsvpproHelper::addSubmenu();



		// Get data from the model
		$model	= $this->getModel();
		$item	=  $this->get( 'Data');
		$this->params	= $model->getParams($item);
		$templateparams = $item->params;
		if (is_string($templateparams) && strlen(trim($templateparams))>2){
			$templateparams = new Registry($templateparams);
		}
		else if (is_string($templateparams) && trim($templateparams)==""){
			$templateparams = ComponentHelper::getParams(RSVP_COM_COMPONENT);
		}
		$this->templateparams =$templateparams ;

		$hasAttendeesToLock = $model->hasAttendeesToLock($item);

		$this->setLayout("edit");
		$this->item = 		$item;

		$this->hasAttendeesToLock = $hasAttendeesToLock;

		// Extra call for the RSVP Pro Pop-Up edit window to allow CSS tweaks.
		if (File::exists(JPATH_SITE . "/components/com_jevents/assets/css/jevcustom.css"))
		{
			// It is definitely now created, lets load it!
			JEVHelper::stylesheet('jevcustom.css', 'components/' . JEV_COM_COMPONENT . '/assets/css/');
		}

		if (!$item->id || empty($item->ticket))
		{
			$item->ticket = Text::_("RSVP_DEFAULT_TICKET");
		}

        $copyTranslations = 0;
        if (isset($item->id) && $item->id>0 && (Factory::getApplication()->input->getInt("customise",0) || Factory::getApplication()->input->getString("task")=="templates.editcopy" )) {
            $db = Factory::getDbo();
            $db->setQuery("SELECT count(translation_id) FROM #__jev_rsvp_fields_translation where template_id=".$item->id);
            $copyTranslations = intval($db->loadResult())>0;
        }
        $this->copyTranslations = $copyTranslations;
		parent::displaytemplate($tpl);

	}

	function _globalHTML(&$row, $i){
		$pluginpath = 'plugins/jevents/jevrsvppro/rsvppro/';

		$img 	= $row->global ? 'Tick.png':  'Cross.png';
		$alt 	= $row->global ? Text::_( 'Global' ) : Text::_( 'User' );

		$mainframe = Factory::getApplication();
		$img = '<img src="' . JURI::root() . $pluginpath . '/assets/' . $img . '"  style="height:16px;" alt="' . $alt . '" />';

		$action = ucfirst($row->global ? Text::_( 'RSVP_MAKE_TEMPLATE_PRIVATE') : Text::_( 'RSVP_MAKE_TEMPLATE_COMMON' ));
		$actiondetail =  Text::_( 'RSVP_PRIVATE_TEMPLATE_EXPLANATION' , true);
		$task = $row->global ? "templates.privatise":"templates.globalise";

		$tooltip = ' data-yspoptitle="' . $action . '"';
		$tooltip .= ' data-yspopcontent="' . $actiondetail . '"';

		if (JevTemplateHelper::canCreateGlobal())
		{
			$href = '
			<a href="javascript:void(0);" class="hasYsPopover" onclick=" Joomla.listItemTask(\'cb'. $i .'\',\''. $task .'\');return false;" ' . $tooltip. '>
			'.$img.'</a>'	;
		}
		else {
			$href = $img;
		}

		return $href;
	}

	function _lockedHTML(&$row, $i){
		$pluginpath = 'plugins/jevents/jevrsvppro/rsvppro/';

		$img 	= $row->locked ? 'Tick.png':  'Cross.png';
		$alt 	= $row->locked ? Text::_( 'RSVP_LOCKED' ) : Text::_( 'RSVP_UNLOCKED' );

		$mainframe = Factory::getApplication();
		$img = '<img src="' . JURI::root() . $pluginpath . '/assets/' . $img . '"  style="height:16px;" alt="' . $alt . '" />';

		$action = ucfirst($row->locked ? Text::_( 'RSVP_UNLOCK_TEMPLATE') : Text::_( 'RSVP_LOCK_TEMPLATE' ));
		$actiondetail =  Text::_( 'RSVP_LOCK_TEMPLATE_EXPLANATION' , true);
		$task = $row->locked ? "templates.unlock":"templates.lock";

		$tooltip = ' data-yspoptitle="' . $action . '"';
		$tooltip .= ' data-yspopcontent="' . $actiondetail . '"';

		if (JevTemplateHelper::canCreateGlobal())
		{
			$href = '
			<a href="javascript:void(0);" class="hasYsPopover" onclick=" Joomla.listItemTask(\'cb'. $i .'\',\''. $task .'\');return false;" ' . $tooltip. '>
			'.$img.'</a>'	;
		}
		else {
			$href = $img;
		}

		return $href;
	}

	function isTemplateHTML(&$row, $i){
		$pluginpath = 'plugins/jevents/jevrsvppro/rsvppro/';

		$img 	= $row->istemplate? 'Tick.png':  'Cross.png';
		$alt 	= $row->istemplate ? Text::_( 'Global' ) : Text::_( 'User' );

		$mainframe = Factory::getApplication();
		$img = '<img src="' . JURI::root() . $pluginpath . '/assets/' . $img . '"  style="height:16px;" alt="' . $alt . '" />';

		$action = ucfirst($row->istemplate ? Text::_( 'RSVP_UNMAKE_TEMPLATE' , true) : Text::_( 'RSVP_MAKE_TEMPLATE' , true));
		$actiondetail =  Text::_( 'RSVP_IS_TEMPLATE_EXPLANATION' , true);
		$task = $row->istemplate ? "templates.unmaketemplate":"templates.maketemplate";

		$tooltip = ' data-yspoptitle="' . $action . '"';
		$tooltip .= ' data-yspopcontent="' . $actiondetail . '"';

		if (JevTemplateHelper::canCreateGlobal())
		{
			$href = '
			<a href="javascript:void(0);" class="hasYsPopover" onclick=" Joomla.listItemTask(\'cb'. $i .'\',\''. $task .'\');return false;" ' . $tooltip. '>
			'.$img.'</a>'	;
		}
		else {
			$href = $img;
		}

		return $href;
	}

	function translate($tpl = null)
	{

		if (!version_compare(JVERSION, "3.4", 'ge')) 	{
			echo "You need Joomla 3.4+ to do that<br/>";
			return;
		}

		HTMLHelper::_('bootstrap.framework');
		HTMLHelper::_('bootstrap.loadCss');

		HTMLHelper::stylesheet(  'components/'.RSVP_COM_COMPONENT.'/assets/css/rsvpadmin.css' );

		HTMLHelper::script( 'components/'.RSVP_COM_COMPONENT.'/assets/js/forms.js' );

		$params = ComponentHelper::getParams(JEV_COM_COMPONENT);

		$conf = Factory::getConfig();
		$defaulteditor = $conf->get('editor');
		$this->editor =  Editor::getInstance($defaulteditor);

		$this->editor =  Factory::getEditor();
		if ($defaulteditor == "codemirror")
		{
			$this->editor = Editor::getInstance('none');
			Factory::getApplication()->enqueueMessage(Text::_("JEV_CODEMIRROR_NOT_COMPATIBLE_EDITOR", "WARNING"));
			if (!$this->editor) {
				Factory::getApplication()->enqueueMessage(Text::_("JEV_EDITOR_NONE_NOT_PUBLISHED", "WARNING"));
			}
		}

		// Get the form && data
		$model	= $this->getModel();
		//$this->form = $this->get('TranslateForm');
		$this->item = $this->get("Translation");

		$this->params	= $model->getParams($this->item);
		$templateparams = $this->item->params;
		if (is_string($templateparams) && strlen(trim($templateparams))>2){
			$templateparams = new Registry($templateparams);
		}
		else if (is_string($templateparams) && trim($templateparams)==""){
			$templateparams = ComponentHelper::getParams(RSVP_COM_COMPONENT);
		}
		$this->templateparams =$templateparams ;

		$this->lang = Factory::getApplication()->input->getString("lang", "");
		$this->hasAttendeesToLock =  0;

		$this->addTranslationToolbar();

		$this->setLayout("translate");

		parent::displaytemplate($tpl);
	}

	/**
	 * Add the page title and toolbar.
	 *
	 * @return  void
	 *
	 * @since   1.6
	 */
	protected function addTranslationToolbar()
	{
		Factory::getApplication()->input->set('hidemainmenu', true);

		JToolbarHelper::save('templates.savetranslation');
		JToolbarHelper::cancel('templates.close');
		$bar =  JToolBar::getInstance('toolbar');

		// Add a standard button
		$bar->appendButton('confirm', Text::_("RSVP_DELETE_TRANSLATION_WARNING"),  'trash',  'JEV_DELETE', "templates.deletetranslation", false);

	}


	protected function translationLinks ($row)
	{
		if ($this->languages)
		{
			JLoader::register('JevModal',JPATH_LIBRARIES."/jevents/jevmodal/jevmodal.php");
			JevModal::modal(".rsvp-translations .jevmodal");
			JEVHelper::stylesheet('eventsadmin.css', 'components/com_jevents/assets/css/');

			// Any existing translations ?
			$db = Factory::getDbo();
			$db->setQuery("SELECT language FROM #__jev_rsvp_template_translation where template_id= ".$row->id);
			$translations = $db->loadColumn();
			// test styling for existing translation
			//$translations[] = "cy-GB";
			?>
			<ul class="rsvp-translations item-associations gsl-list">
				<?php foreach ($this->languages as $id => $item) :

					$text = strtoupper($item->sef);
					$url = Route::_('index.php?option=com_rsvppro&task=templates.translate&template_id='.$row->id.'&pop=1&tmpl=component&lang=' . $item->lang_code);
					$img = HTMLHelper::_('image', 'mod_languages/' . $item->image . '.gif',
						$item->title,
						array('title' => $item->title),
						true
					);
					$url  = "javascript:jevModalPopup('translationPopup', '".$url ."', '". Text::sprintf("JEV_TRANSLATE_TEMPLATE_TO" ,  addslashes($item->title),  array('jsSafe'=>true) ) . "'); ";
					$tooltipParts = array( 	$img,  $item->title);
					$item->link = HTMLHelper::_('tooltip', implode(' ', $tooltipParts), null, null, $text, $url, null, 'hasTooltip label label-association jevmodal-link gsl-button gsl-button-small label-' . $item->sef .( in_array($item->lang_code, $translations)?" hastranslation":" hasnotranslation" ));
					?>
					<li>
						<?php
						echo $item->link;
						?>
					</li>
				<?php endforeach; ?>
			</ul>
			<?php
		}
	}


}