<?php

/**
 * JEvents Component for Joomla 1.5.x
 *
 * @version     $Id$
 * @package     JEvents
 * @copyright   Copyright (C) 2008-2021 GWE Systems Ltd
 * @license     GNU/GPLv2, see http://www.gnu.org/licenses/gpl-2.0.html
 * @link        https://www.jevents.net
 */
defined('JPATH_BASE') or die('Direct Access to this location is not allowed.');

use Joomla\CMS\Session\Session;
use Joomla\Registry\Registry;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Factory;
use Joomla\CMS\Filesystem\File;

include_once(RSVP_ADMINPATH . "/controllers/accounts.php");

class FrontAccountsController extends AdminAccountsController
{

	function __construct($config = array())
	{
		parent::__construct($config);

		$this->registerDefaultTask("notify");

		include_once(JPATH_ADMINISTRATOR . "/components/com_rsvppro/libraries/attendeehelper.php");
		$this->params = ComponentHelper::getParams("com_rsvppro");
		$this->helper = new RsvpAttendeeHelper($this->params);

	}

	function notify()
	{

		PluginHelper::importPlugin("rsvppro");

		$activePlugin = "";

		Factory::getApplication()->triggerEvent('onActiveGatewayClass', array(&$activePlugin, "notify"));

		if ($activePlugin != "" && class_exists($activePlugin))
		{
			$args = array();
			call_user_func_array(array($activePlugin, "notifyRsvpGateway"), array(&$args));
		}

	}

	function confirm()
	{

		PluginHelper::importPlugin("rsvppro");

		$activePlugin = "";

		Factory::getApplication()->triggerEvent('onActiveGatewayClass', array(&$activePlugin, "confirm"));

		if ($activePlugin != "" && class_exists($activePlugin))
		{
			$args = array();
			call_user_func_array(array($activePlugin, "confirmRsvpGateway"), array(&$args));
		}

	}

	function refund()
	{

		// Accessible as:
		// index.php/component/rsvppro/?task=accounts.refund&gateway=paypalexpress&atd_id=1

		PluginHelper::importPlugin("rsvppro");

		$activePlugin = "";

		Factory::getApplication()->triggerEvent('onActiveGatewayClass', array(&$activePlugin, "refund"));

		if ($activePlugin != "" && class_exists($activePlugin))
		{
			$args = array();
			call_user_func_array(array($activePlugin, "refundRsvpGateway"), array(&$args));
		}

	}

	function paymentpage()
	{
              if (File::exists(JPATH_SITE . "/components/com_jevents/assets/css/jevcustom.css")) {
		JEVHelper::stylesheet('jevcustom.css', 'components/' . JEV_COM_COMPONENT . '/assets/css/');
	  }
		PluginHelper::importPlugin("rsvppro");

		// Check for request forgeries
		Session::checkToken('request') or jexit('Invalid Token');

		$activePlugin = "";

		Factory::getApplication()->triggerEvent('onActiveGatewayClass', array(&$activePlugin, "paymentpage"));

		if ($activePlugin != "" && class_exists($activePlugin))
		{

			// fetch the session and the attendee
			$db = Factory::getDBO();
			$sql = "SELECT * FROM #__jev_attendees WHERE id=" . Factory::getApplication()->input->getInt('invoiceid', -1);
			$db->setQuery($sql);
			$attendee = $db->loadObject();
			if (!$attendee)
                        {
			    return;
                        }
			if ($attendee->at_id != Factory::getApplication()->input->getInt('rsvpid', -1))
                        {
			    return;
                        }

			// security
			if ($attendee->user_id == 0)
			{
				$em = Factory::getApplication()->input->getString("em", "");
				if ($em != "")
				{
					$emd = base64_decode($em);

					if (strpos($emd, ":") > 0)
					{
						list ( $emailaddress, $code ) = explode(":", $emd);

						$params = ComponentHelper::getParams("com_rsvppro");

						if ($em != base64_encode($emailaddress . ":" . md5($params->get("emailkey", "email key") . $emailaddress)))
						{
							return "";
						}
						else if ($emailaddress != $attendee->email_address)
						{
							return "";
						}
					}
					else
					{
						return "";
					}
				}
				else
				{
					return "";
				}
			}

			$sql = "SELECT * FROM #__jev_attendance WHERE id=" . $attendee->at_id;
			$db->setQuery($sql);
			$rsvpdata = $db->loadObject();
			if (!$rsvpdata) {
                            return;
                        }

			// attach balance info to the attendee
			if (!isset($attendee->outstandingBalances))
			{
				// This MUST be called before renderToBasicArray to populate the balance fields - so we do it here to be safe
				if (isset($attendee->params))
				{
					$xmlfile = JevTemplateHelper::getTemplate($rsvpdata);

					if (is_int($xmlfile) || file_exists($xmlfile))
					{
						$params = new JevRsvpParameter($attendee->params, $xmlfile, $rsvpdata, null);

						// do we need to update the payment method in the $params AND in the attendee->params data (to be safe)!
						$rsvpparamsarray = $params->renderToBasicArray();
						// find the paymentoptionlist
						foreach ($rsvpparamsarray as $pield => $pm)
						{
							if ($pm["type"] == "jevrpaymentoptionlist"){

								$newval = Factory::getApplication()->input->get("xmlfile",array(), 'ARRAY');
								if (isset($newval[$pield])){
									$attendeefields = new Registry($attendee->params);
									$oldval = $attendeefields->get($pield);
									$attendeefields->set($pield, $newval[$pield]);
									if (version_compare(PHP_VERSION, '5.4', '>='))
									{
										$attendee->params = $attendeefields->toString('JSON', array('bitmask' =>JSON_UNESCAPED_UNICODE));
									}
									else
									{
										$attendee->params = $attendeefields->toString();
									}
									//$attendee->params = str_replace('"'.$pield.'":"'.$pm["value"].'"','"'.$pield.'":"'.$newval[$pield].'"',$attendee->params);
									// must also update the database
									$db = Factory::getDbo();
									$db->setQuery("UPDATE #__jev_attendees set params=".$db->Quote($attendee->params). " WHERE id=" . $attendee->id);
									$db->execute();

									// reset the active gateway
									Factory::getApplication()->input->set("gateway",  $newval[$pield]);
									Factory::getApplication()->triggerEvent('onActiveGatewayClass', array(&$activePlugin, "paymentpage"));
									if (!$activePlugin  || !class_exists($activePlugin))
									{
										exit ("Invalid payment gateway");
									}

									$params->bind($newval);
								}
							}
						}

						$feesAndBalances = $params->outstandingBalance($attendee);
					}
				}
			}

			if ($rsvpdata->allrepeats){
				// get the first repeat
				$event = $this->dataModel->queryModel->getEventById($rsvpdata->ev_id, false, "icaldb");
			}
			else {
				$event = $this->dataModel->queryModel->listEventsById($attendee->rp_id, false, "icaldb");
			}

			// Create the base transaction - empty ones are useful since I may build a system for tracking abandoned transactions later
			$transaction = new rsvpTransaction();
			$transaction->attendee_id = $attendee->id;
			$transaction->store();

			$html = "";
			// Test needed for J4/J3.9 compatibility
			$dispatcher = method_exists(Factory::getApplication(),"getDispatcher") ? Factory::getApplication()->getDispatcher() : JEventDispatcher::getInstance();
			// load plugin parameters
			$pluginname = strtolower(str_replace("plgRsvppro", "", $activePlugin));

			$plugin =  PluginHelper::getPlugin("rsvppro", $pluginname);

			// create the plugin
			$gateway = new $activePlugin($dispatcher, (array) ($plugin));

			//call_user_func_array(array($gateway,"generatePaymentPage"),array(&$html, $attendee, $rsvpdata, $event, $transaction));
			$gateway->generatePaymentPage($html, $attendee, $rsvpdata, $event, $transaction);
			echo $html;
		}

	}

}
