<?php
/**
 * JEvents Component for Joomla 1.5.x
 *
 * @version     $Id: jevents.php 1402 2009-04-03 13:00:38Z geraint $
 * @package     JEvents
 * @copyright   Copyright (C)  2008-2021 GWE Systems Ltd
 * @license     GNU/GPLv2, see http://www.gnu.org/licenses/gpl-2.0.html
 * @link        https://www.jevents.net
 */


defined( 'JPATH_BASE' ) or die( 'Direct Access to this location is not allowed.' );

use Joomla\CMS\Language\Text;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Factory;
use Joomla\CMS\Filesystem\File;

if (version_compare(phpversion(), '5.0.0', '<')===true) {
	echo  '<div style="font:12px/1.35em arial, helvetica, sans-serif;"><div style="margin:0 0 25px 0; border-bottom:1px solid #ccc;"><h3 style="margin:0; font-size:1.7em; font-weight:normal; text-transform:none; text-align:left; color:#2f2f2f;">'.Text::_("RSVP_INVALID_PHP1").'</h3></div>'.Text::_("RSVP_INVALID_PHP2").'</div>';
	return;
}

jimport('joomla.filesystem.path');
$option = Factory::getApplication()->input->getCmd("option");
 include_once(JPATH_SITE."/components/com_jevents/jevents.defines.php");
include_once(JPATH_COMPONENT_ADMINISTRATOR.'/'.str_replace("com_","",$option).".defines.php");
JLoader::register('JevRegistry', JPATH_SITE . "/components/com_jevents/libraries/registry.php");

$registry	= JevRegistry::getinstance("jevents");

// In Joomla 1.6 ComponentHelper::getParams(JEV_COM_COMPONENT) is a clone so the menu params do not propagate so we force this here!
$newparams	= Factory::getApplication('site')->getParams();
$component = ComponentHelper::getComponent(RSVP_COM_COMPONENT);
$component->params = $newparams;

$jevparams = ComponentHelper::getParams(JEV_COM_COMPONENT);
// See http://www.php.net/manual/en/timezones.php
$tz=$jevparams->get("icaltimezonelive","");
if ($tz!="" && is_callable("date_default_timezone_set")){
	$timezone= date_default_timezone_get();
	date_default_timezone_set($tz);
	$registry->set("jevents.timezone",$timezone);
}

$lang = Factory::getLanguage();
$lang->load(RSVP_COM_COMPONENT, JPATH_ADMINISTRATOR);
// also load the plugin language file!
$lang->load( 'plg_jevents_jevrsvppro',JPATH_ADMINISTRATOR );

if (isset($_SERVER['HTTP_USER_AGENT']) && (strpos($_SERVER['HTTP_USER_AGENT'], "MSIE") !== false || strpos($_SERVER['HTTP_USER_AGENT'], "Internet Explorer") !== false))
{
	define ("GSLMSIE10" , 1);
}
else
{
	// Also needs JEvents 3.5.0 installed
	JLoader::register('JEventsVersion',JPATH_ADMINISTRATOR."/components/com_jevents/libraries/version.php");
	$jevversion = JEventsVersion::getInstance();

	define ("GSLMSIE10" , version_compare($jevversion->get("RELEASE"), "3.4.9999", "lt") ? 1 : 0);
}

$params = ComponentHelper::getParams(RSVP_COM_COMPONENT);

// disable Zend php4 compatability mode
@ini_set("zend.ze1_compatibility_mode","Off");

$cmd = Factory::getApplication()->input->getCmd('task', 'sessions.overview');

// Load JEvents custom css file if its created
jimport('joomla.filesystem.file');
if (File::exists(JPATH_SITE . "/components/com_jevents/assets/css/jevcustom.css"))
{
	// It is definitely now created, lets load it!
	JEVHelper::stylesheet('jevcustom.css', 'components/' . JEV_COM_COMPONENT . '/assets/css/');
}

if (strpos($cmd, '.') != false) {
	// We have a defined controller/task pair -- lets split them out
	list($controllerName, $task) = explode('.', $cmd);

	// Define the controller name and path
	$controllerName	= strtolower($controllerName);
	$controllerPath	= JPATH_COMPONENT.'/'.'controllers'.'/'.$controllerName.'.php';
	$controllerName = "Front".$controllerName;

	// If the controller file path exists, include it ... else lets die with a 500 error
	if (file_exists($controllerPath)) {
		require_once($controllerPath);
	} else {
		throw new Exception( 'Invalid Controller', 500);
	}
} else {
	// Base controller, just set the task
	$controllerName = null;
	$task = $cmd;
}

PluginHelper::importPlugin("jevents");

// Tell RSVP Pro javascript which framework to use
$frontendstyling = $params->get("frontendstyling", "B2");
Factory::getDocument()->addScriptDeclaration("var rsvpprofrontendstyling = '$frontendstyling';\n");

// Set the name for the controller and instantiate it
$controllerClass = ucfirst($controllerName).'Controller';
if (class_exists($controllerClass)) {
	$controller = new $controllerClass();

	// Perform the Request task
	$controller->execute($task);

	// Must reset the timezone back!!
	if ($tz && is_callable("date_default_timezone_set")){
		date_default_timezone_set($timezone);
	}

	// Redirect if set by the controller
	$controller->redirect();

} else {
	echo "missing controllerClass $controllerClass";
	exit();
}
