<?php
/**
 * copyright (C) 2009-2021 GWE Systems Ltd - All rights reserved
 */
// no direct access
defined('_JEXEC') or die('Restricted access');

use Joomla\Registry\Registry;
use Joomla\CMS\Table\Table;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Session\Session;
use Joomla\CMS\Factory;
use Joomla\CMS\HTML\HTMLHelper;

include_once(JPATH_SITE . "/components/com_jevents/jevents.defines.php");
JLoader::register('jevFilterProcessing', JPATH_SITE . "/components/com_jevents/libraries/filters.php");
JLoader::register('JevRsvpParameter', JPATH_ADMINISTRATOR . "/components/com_rsvppro/libraries/jevrsvpparameter.php");
JLoader::register('JevRsvpAttendance', dirname(__FILE__) . "/rsvppro/jevrattendance.php");
JLoader::register('JevRsvpDisplayAttendance', dirname(__FILE__) . "/rsvppro/jevrdisplayattendance.php");
JLoader::register('RsvpHelper', JPATH_ADMINISTRATOR . "/components/com_rsvppro/libraries/rsvphelper.php");
JLoader::register('JevTemplateHelper', JPATH_ADMINISTRATOR . "/components/com_rsvppro/libraries/templatehelper.php");
JLoader::register('JevDate', JPATH_SITE . "/components/com_jevents/libraries/jevdate.php");
JLoader::register('JevRsvpAttendees', JPATH_ADMINISTRATOR . "/components/com_rsvppro/libraries/jevrattendees.php");
JLoader::register('JEventsVersion', JPATH_ADMINISTRATOR . "/components/com_jevents/libraries/version.php");
$jevversion = JEventsVersion::getInstance();

include_once(JPATH_ADMINISTRATOR . "/components/com_rsvppro/rsvppro.defines.php");

/**
 * TODO
 *
 * 1. disable "Login message" for invitee only events (but allow for links from email)
 * 2. option to hide events from non-invitees
 * 3. Setup javascript to recognise if the event is repeating or not.
 */
// see http://www.setcronjob.com/

jimport('joomla.plugin.plugin');
JLoader::register('JevRegistry', JPATH_SITE . "/components/com_jevents/libraries/registry.php");

class plgJEventsJevrsvppro extends CMSPlugin {

	function __construct(& $subject, $config) {
		parent::__construct($subject, $config);

		// merge in the new config options from the component
		$params = ComponentHelper::getParams("com_rsvppro");
		$this->params->merge($params);

		Factory::getLanguage()->load('plg_jevents_jevrsvppro', JPATH_ADMINISTRATOR);
		Factory::getLanguage()->load('com_rsvppro', JPATH_SITE);
		Factory::getLanguage()->load('com_rsvppro', JPATH_ADMINISTRATOR);

		// Tell RSVP Pro javascript which framework to use
		$frontendstyling = $params->get("frontendstyling", "B2");
		Factory::getDocument()->addScriptDeclaration("var rsvpprofrontendstyling = '$frontendstyling';\n");

		jimport("joomla.filesystem.file");
		try {
			$xml = simplexml_load_string(file_get_contents(dirname(__FILE__) . "/jevrsvppro.xml"));
			$this->params->set("pluginversion", (string) $xml->version);
		} catch (Exception $ex) {
			$this->params->set("pluginversion", 1);
		}
		// up to date?
		/*
		  if ($this->params->get("setupstate",0)!=1){
		  $this->params->set("setupstate",1);
		  $row  = Table::getInstance('plugin');
		  }
		 */
	}

	/**
	 * When editing a JEvents menu item can add additional menu constraints dynamically
	 *
	 */
	function onEditMenuItem(&$menudata, $value, $control_name, $name, $id, $param) {

		// already done this param
		if (isset($menudata[$id]))
			return;

		static $matchingextra = null;
		// find the parameter that matches jevrsvp: (if any)
		if (!isset($matchingextra)) {
			$params = $param->getGroup('params');
			foreach ($params as $key => $element) {
				$val = $element->value;
				if (strpos($key, "jform_params_extras") === 0) {
					if (strpos($val, "jevrsvp:") === 0) {
						$matchingextra = $key;
						break;
					}
				}
			}
			if (!isset($matchingextra)) {
				$matchingextra = false;
			}
		}

		// either we found matching extra and this is the correct id or we didn't find matching extra and the value is blank
		if ((strpos($value, "jevrsvp:") === 0 && is_string($matchingextra) && strpos($matchingextra, $name) >= 0) || (($value == "" || $value == "0") && $matchingextra === false)) {
			$matchingextra = true;
			$invalue = str_replace(" ", "", $value);
			if ($invalue == "")
				$invalue = 'jevrsvp:0';

			$options = array();
			$options[] = HTMLHelper::_('select.option', 'jevrsvp:0', Text::_('JEV_NO_RESTRICTIONS'), 'id', 'title');
			$options[] = HTMLHelper::_('select.option', 'jevrsvp:1', Text::_('JEV_EVENTS_ATTENDING'), 'id', 'title');
			$options[] = HTMLHelper::_('select.option', 'jevrsvp:2', Text::_('JEV_EVENTS_INVITED_TO'), 'id', 'title');
			$options[] = HTMLHelper::_('select.option', 'jevrsvp:3', Text::_('JEV_EVENTS_ATTENDING_EVEN_IF_NOT_PAID'), 'id', 'title');
			$options[] = HTMLHelper::_('select.option', 'jevrsvp:4', Text::_('JEV_EVENTS_ANYONE_ATTENDING'), 'id', 'title');
			$options[] = HTMLHelper::_('select.option', 'jevrsvp:5', Text::_('JEV_EVENTS_ATTENDING_EVEN_IF_NOT_CONFIRMED_OR_PAID'), 'id', 'title');

			if ($control_name == "params") {
				// for CB
				$input = HTMLHelper::_('select.genericlist', $options, '' . $control_name . '[' . $name . ']', '', 'id', 'title', $invalue, $control_name . $name);
			} else {
				$input = HTMLHelper::_('select.genericlist', $options, $name, '', 'id', 'title', $invalue, $control_name . $name);
			}

			$data = new stdClass();
			$data->name = "jevrsvpser";
			$data->html = $input;
			$data->label = "JEV_RSVPPRO";
			$data->description = "JEV_RSVPPRO_DESC";
			$data->options = array();
			$menudata[$id] = $data;
		}

		// restrict visibility based on session access
		static $matchingextra2 = null;
		// find the parameter that matches jevrsvpsa: (if any)
		if (!isset($matchingextra2)) {
			$params = $param->getGroup('params');
			foreach ($params as $key => $element) {
				$val = $element->value;
				if (strpos($key, "jform_params_extras") === 0) {
					if (strpos($val, "jevrsvpsa:") === 0) {
						$matchingextra2 = $key;
						break;
					}
				}
			}
			if (!isset($matchingextra2)) {
				$matchingextra2 = false;
			}
		}

		// either we found matching extra and this is the correct id or we didn't find matching extra and the value is blank
		if ((strpos($value, "jevrsvpsa:") === 0 && is_string($matchingextra2) && strpos($matchingextra2, $name) >= 0) || (($value == "" || $value == "0") && $matchingextra2 === false)) {
			$matchingextra2 = true;
			$invalue = str_replace(" ", "", $value);
			if ($invalue == "")
				$invalue = 'jevrsvpsa:0';

			$options = array();
			$options[] = HTMLHelper::_('select.option', 'jevrsvpsa:0', Text::_('JEV_NO_RESTRICTIONS'), 'id', 'title');
			$options[] = HTMLHelper::_('select.option', 'jevrsvpsa:1', Text::_('JEV_WHERE_AUTHORISED_TO_REGISTER'), 'id', 'title');
			$options[] = HTMLHelper::_('select.option', 'jevrsvpsa:2', Text::_('JEV_WHERE_EVENT_IS_OPEN_FOR_REGISTRATIONS'), 'id', 'title');

			if ($control_name == "params") {
				// for CB
				$input = HTMLHelper::_('select.genericlist', $options, '' . $control_name . '[' . $name . ']', '', 'id', 'title', $invalue, $control_name . $name);
			} else {
				$input = HTMLHelper::_('select.genericlist', $options, $name, '', 'id', 'title', $invalue, $control_name . $name);
			}

			$data = new stdClass();
			$data->name = "jevrsvpsaser";
			$data->html = $input;
			$data->label = "JEV_RSVPPRO_SESSION_ACCESS";
			$data->description = "JEV_RSVPPRO_RESTRICT_BASED_ON_SESSION_ACCESS";
			$data->options = array();
			$menudata[$id] = $data;
		}
	}

	function onEventEdit(&$extraTabs, &$row, &$params) {
		if (RsvpHelper::canCreateSessions()) {
			$jevrattendance = new JevRsvpAttendance($this->params);
			return $jevrattendance->editAttendance($extraTabs, $row, $params);
		}
	}

	function onEditCustom(&$row, &$customfields) {
		if (!$this->params->get("attendance", 1)) {
			$eventid = intval($row->ev_id());
			ob_start();
			?>
			<input type="hidden" name="custom_rsvp_evid" value="<?php echo $eventid; ?>" />
			<input type="hidden" name="custom_rsvp_allowregistration" value="0"  />
			<?php
			$input = ob_get_clean();
			$customfield = array("label" => "", "input" => $input);
			$customfields["rsvppro"] = $customfield;
		}
	}

	/**
	 * Clean out custom fields for event details not matching global event detail
	 *
	 * @param unknown_type $idlist
	 */
	function onCleanCustomDetails($idlist) {
		// TODO
		return true;
	}

	/**
	 * Store custom fields
	 *
	 * @param iCalEventDetail $evdetail
	 */
	// TODO update reminder timestamps when saving event exception
	function onStoreCustomDetails($evdetail) {

	}

	/**
	 * Store custom fields
	 *
	 * @param iCalEventDetail $evdetail
	 */
	// TODO update reminder timestamps when event times have changed
	function onStoreCustomEvent($event) {
		// force registration on.
		//$event->_detail->_customFields["rsvp_allowregistration"]=1;
		// set capacity to 10
		//$event->_detail->_customFields["rsvp_capacity"]=10;
		// Could use this approach to set the price too!

		$jevrattendance = new JevRsvpAttendance($this->params);
		return $jevrattendance->storeAttendance($event);
	}

	/**
	 * Store custom fields
	 *
	 * @param iCalEventDetail $evdetail
	 */
	// TODO update reminder timestamps when event times have changed
	function onAfterSaveEvent($event, $dryrun = false) {
		if ($dryrun) {
			return true;
		}

		$jevrattendance = new JevRsvpAttendance($this->params);
		$jevrattendance->autoInvite($event);
		$jevrattendance->autoRemind($event);

		// if date/time or interval has changed then we need to update the existing reminders if they have already been sent
		if (Factory::getApplication()->input->getInt("updaterepeats",1)) {
			$jevrattendance->reinstateReminders($event);
		}
	}

	/**
	 * Clean out custom details for deleted event details
	 *
	 * @param comma separated list of event detail ids $idlist
	 */
	function onDeleteEventDetails($idlist) {
		// TODO remove any orphan attendee, invitee or reminder records
		return true;
	}

	function onDeleteCustomEvent($idlist) {
		$jevrattendance = new JevRsvpAttendance($this->params);
		return $jevrattendance->deleteAttendance($idlist);
	}

	function onListIcalEvents(& $extrafields, & $extratables, & $extrawhere, & $extrajoin, & $needsgroupdby = false, & $rptwhere = array()) {

		$mainframe = Factory::getApplication();
		if ($mainframe->isClient('administrator')) {
			return;
		}
		// find what is running - used by the filters
		$registry = JevRegistry::getinstance("jevents");
		$activeprocess = $registry->get("jevents.activeprocess", "");
		$moduleid = $registry->get("jevents.moduleid", 0);
		$moduleparams = $registry->get("jevents.moduleparams", false);

		// Have we specified restrictions on attending or invited on this menu/module
		$user = Factory::getUser();

		$compparams = ComponentHelper::getParams("com_jevents");

		$reg = Factory::getConfig();
		$modparams = $reg->get("jev.modparams", false);
		if ($modparams) {
			$compparams = $modparams;
		}

		$filters = array("hidefornoninvitees");

		// Do we need to filter on session access
		for ($extra = 0; $extra < 20; $extra++) {
			$extraval = $compparams->get("extras" . $extra, false);
			if (strpos($extraval, "jevrsvpsa:1") === 0) {
				$filters[] = "hidebasedonaccess";
				break;
			}
			if (strpos($extraval, "jevrsvpsa:2") === 0) {
				$filters[] = "hidebasedonaccess";
				$filters[] = "hidebasedonopenclose";
				$filters[] = "hidebasedoncapacity";
				break;
			}
		}

		if (version_compare(JVERSION, "1.6.0", 'ge')) {
			$filters = jevFilterProcessing::getInstance($filters, JPATH_SITE . "/plugins/jevents/jevrsvppro/filters", false, $moduleid);
		} else {
			$filters = jevFilterProcessing::getInstance($filters, JPATH_SITE . "/plugins/jevents/filters", false, $moduleid);
		}

		$filters->setWhereJoin($extrawhere, $extrajoin);
		if (!$needsgroupdby)
			$needsgroupdby = $filters->needsGroupBy();

		for ($extra = 0; $extra < 20; $extra++) {
			$extraval = $compparams->get("extras" . $extra, false);
			if (strpos($extraval, "jevrsvp:") === 0) {
				break;
			}
		}

		if ($this->params->get("showattendeecountinlists", 0)) {
			$ej = "#__jev_attendeecount as atdcnt ON atdcnt.at_id=atd.id AND (atd.allrepeats=1 OR atdcnt.rp_id=rpt.rp_id) ";
			if(!in_array($ej, $extrajoin)) {
				$extrajoin[] = $ej;
			}
			$extrafields .= ", \n atdcnt.atdcount as attendeeCount, atd.capacity, atd.waitingcapacity, atdcnt.atdcount as registrationguestcount ";

		}

		$extrafields .= ", \n atd.regopen, atd.regclose, atd.allrepeats,  atd.allowregistration";

		if (!$extraval && !$this->params->get("hidefull", 0))
			return true;
		if (!$extraval)
			$extraval = "";

		$invalue = str_replace("jevrsvp:", "", $extraval);
		$invalue = str_replace(" ", "", $invalue);
		if (substr($invalue, strlen($invalue) - 1) == ",") {
			$invalue = substr($invalue, 0, strlen($invalue) - 1);
		}
		$invalue = intval($invalue);

		// only for logged in users (overrule for for hidefull!)
		if ($user->id == 0 && (!$this->params->get("hidefull", 0) && !$this->params->get("showattendeecountinlists", 0))) {
			// 1 = attending
			// 2 = invited to
			if ($invalue >= 1) {
				$extrawhere[] = "1=0";
				return;
			} else {
				return;
			}
		}

		$atdeesjoined = false;
		// 1 = attending
		if ($invalue == 1) {
			$needsgroupdby = true;
			if (!$atdeesjoined) {
				$extrajoin[] = "#__jev_attendees as atdees ON atdees.at_id=atd.id AND atdees.user_id=" . $user->id . " AND atdees.attendstate=1";
				$atdeesjoined = true;
			}
			// show attendance or show attendance to invitees
			$extrawhere[] = "((atd.allrepeats=1 AND atdees.rp_id=0) OR (atd.allrepeats=0 AND atdees.rp_id=rpt.rp_id)) AND (atd.allowregistration=1 OR atd.allowregistration=2)";
			$rptwhere[] = " ((atd.allrepeats=1 AND atdees.rp_id=0) OR (atd.allrepeats=0 AND atdees.rp_id=rpt2.rp_id)) AND (atd.allowregistration=1 OR atd.allowregistration=2)";
		}
		// 2 = invited to
		else if ($invalue == 2) {
			// already joined to invites
			$needsgroupdby = true;
			$extrawhere[] = "((atd.allinvites=1 AND inv.rp_id=0) OR (atd.allinvites=0 AND inv.rp_id=rpt.rp_id)) AND atd.invites=1 AND inv.user_id=" . $user->id;
			$rptwhere[] = "((atd.allinvites=1 AND inv.rp_id=0) OR (atd.allinvites=0 AND inv.rp_id=rpt2.rp_id)) AND atd.invites=1 AND inv.user_id=" . $user->id;
		}
		// 3 = attending even if not paid
		else if ($invalue == 3) {
			$needsgroupdby = true;
			if (!$atdeesjoined) {
				$extrajoin[] = "#__jev_attendees as atdees ON atdees.at_id=atd.id AND atdees.user_id=" . $user->id . " AND (atdees.attendstate=1 OR atdees.attendstate=4)";
				$atdeesjoined = true;
			}
			// show attendance or show attendance to invitees
			$extrawhere[] = "((atd.allrepeats=1 AND atdees.rp_id=0) OR (atd.allrepeats=0 AND atdees.rp_id=rpt.rp_id)) AND (atd.allowregistration=1 OR atd.allowregistration=2)";
			$rptwhere[] = " ((atd.allrepeats=1 AND atdees.rp_id=0) OR (atd.allrepeats=0 AND atdees.rp_id=rpt2.rp_id)) AND (atd.allowregistration=1 OR atd.allowregistration=2)";
		}
		// 4 = Anyone attending
		else if ($invalue == 4) {
			if (!$this->params->get("hidefull", 0) && !$this->params->get("showattendeecountinlists", 0)) {
				$extrajoin[] = "#__jev_attendeecount as atdcnt ON atdcnt.at_id=atd.id  AND ((atd.allrepeats=1  AND atdcnt.rp_id=0) OR (atd.allrepeats=0  AND atdcnt.rp_id=rpt.rp_id)) ";
			}
			$needsgroupdby = true;
			// show attendance or show attendance to invitees
			$extrawhere[] = "((atd.allrepeats=1 AND atdcnt.rp_id=0) OR (atd.allrepeats=0 AND atdcnt.rp_id=rpt.rp_id)) AND (atd.allowregistration=1 OR atd.allowregistration=2) AND atdcnt.atdcount>0";
			$rptwhere[] = " ((atd.allrepeats=1 AND atdcnt.rp_id=0) OR (atd.allrepeats=0 AND atdcnt.rp_id=rpt2.rp_id)) AND (atd.allowregistration=1 OR atd.allowregistration=2) AND atdcnt.atdcount>0";
		}
		// 5 = attending even if not paid or is pending
		else if ($invalue == 5) {
			$needsgroupdby = true;
			if (!$atdeesjoined) {
				$extrajoin[] = "#__jev_attendees as atdees ON atdees.at_id=atd.id AND atdees.user_id=" . $user->id . " AND (atdees.attendstate=1 OR atdees.attendstate=3 OR atdees.attendstate=4)";
				$atdeesjoined = true;
			}
			// show attendance or show attendance to invitees
			$extrawhere[] = "((atd.allrepeats=1 AND atdees.rp_id=0) OR (atd.allrepeats=0 AND atdees.rp_id=rpt.rp_id)) AND (atd.allowregistration=1 OR atd.allowregistration=2)";
			$rptwhere[] = " ((atd.allrepeats=1 AND atdees.rp_id=0) OR (atd.allrepeats=0 AND atdees.rp_id=rpt2.rp_id)) AND (atd.allowregistration=1 OR atd.allowregistration=2)";
		}


		// Should we hide full events from non-attendees
		if ($this->params->get("hidefull", 0)) {
			// Must make sure we have the attendees linked too
			if ($invalue != 1 && $user->id > 0) {
				$needsgroupdby = true;
				if (!$atdeesjoined) {
					$extrajoin[] = "#__jev_attendees as atdees ON atdees.at_id=atd.id AND atdees.user_id=" . $user->id . " AND atdees.attendstate=1";
					$atdeesjoined = true;
				}
			}

			if (!$this->params->get("showattendeecountinlists", 0)) {
				$extrajoin[] = "#__jev_attendeecount as atdcnt ON atdcnt.at_id=atd.id  AND ((atd.allrepeats=1  AND atdcnt.rp_id=0) OR (atd.allrepeats=0  AND atdcnt.rp_id=rpt.rp_id)) ";
			}

			if (JEVHelper::isAdminUser($user)) {
				// always show to super admins
			} else if ($user->id > 0) {
				// MUST allow capacity null for imported or legacy events
				$extrawhere[] = "(atd.capacity=0 OR atd.capacity IS NULL
				OR ev.created_by=$user->id
				OR atdcnt.atdcount IS NULL
			OR (atdcnt.atdcount < (atd.capacity+atd.waitingcapacity) AND ((atd.allrepeats=1 AND atdcnt.rp_id=0) OR (atd.allrepeats=0 AND atdcnt.rp_id=rpt.rp_id)))
			OR atdees.user_id IS NOT NULL) ";
			} else {
				// MUST allow capacity null for imported or legacy events
				$extrawhere[] = "(atd.capacity=0 OR atd.capacity IS NULL
				OR atdcnt.atdcount IS NULL
			OR (atdcnt.atdcount < (atd.capacity+atd.waitingcapacity)  AND ((atd.allrepeats=1 AND atdcnt.rp_id=0) OR (atd.allrepeats=0 AND atdcnt.rp_id=rpt.rp_id)) )
			) ";
			}
		}

		return true;
	}

	function onListEventsById(& $extrafields, & $extratables, & $extrawhere, & $extrajoin) {

		$mainframe = Factory::getApplication();
		if ($mainframe->isClient('administrator')) {
			return;
		}
		// find what is running - used by the filters
		$registry = JevRegistry::getinstance("jevents");
		$activeprocess = $registry->get("jevents.activeprocess", "");
		$moduleid = $registry->get("jevents.moduleid", 0);
		$moduleparams = $registry->get("jevents.moduleparams", false);

		if (version_compare(JVERSION, "1.6.0", 'ge')) {
			$filters = jevFilterProcessing::getInstance(array("hidefornoninvitees"), JPATH_SITE . "/plugins/jevents/jevrsvppro/filters", false, $moduleid);
		} else {
			$filters = jevFilterProcessing::getInstance(array("hidefornoninvitees"), JPATH_SITE . "/plugins/jevents/filters", false, $moduleid);
		}

		$filters->setWhereJoin($extrawhere, $extrajoin);

		return true;
	}

	/*
	  function onGetEventData(&$row){
	  }
	 */

	function onDisplayCustomFields(&$row) {
		if (isset($row->hidedetail) && $row->hidedetail) {
			return "";
		}

		if (Factory::getApplication()->input->getCmd("jevtask", "") == "icals.export" || Factory::getApplication()->input->getCmd("task", "") == "attendees.ticket") {
			return;
		}
		$jevrattendance = new JevRsvpDisplayAttendance($this->params);
		return $jevrattendance->displayCustomFields($row);
	}

	function onDisplayCustomFieldsMultiRow(&$rows) {
		// Skip in list of sessions view - seems to have a problem with the attendee counting and is also SLOW!
		if (Factory::getApplication()->input->getCmd("task") == "sessions.list" || Factory::getApplication()->input->getCmd("task") == "sessions.overview" || (Factory::getApplication()->input->getCmd("view") == "sessions" && Factory::getApplication()->input->getCmd("layout") == "overview") || Factory::getApplication()->input->getString("jevtask") == "admin.listevents") {
			return;
		}

		// get all the attendance id info in one go
		$eventids = array(0);
		foreach ($rows as & $row) {
			$eventids[] = intval($row->ev_id());
		}

		static $rsvpdataArray;
		if (!isset($rsvpdataArray))
		{
			$db = Factory::getDBO();
			$sql = "SELECT * FROM #__jev_attendance WHERE ev_id in (" . implode(",", $eventids) . ")";
			$db->setQuery($sql);
			$rsvpdataArray = $db->loadObjectList('ev_id');
		}

		// Avoid recursive calls
		$staticid = json_encode($eventids);
		static $processed = array();
		if (isset($processed[$staticid]))
		{
			// if the same set of events are called a second time e.g. duplicate modules we need to set rsvpdata!!
			foreach ($rows as & $row) {
				if (isset($row->hidedetail) && $row->hidedetail) {
					continue;
				}
				if (isset($row->rsvpdata))
				{
					continue;
				}

				$eventid = intval($row->ev_id());
				if ($eventid > 0 && array_key_exists($eventid, $rsvpdataArray)) {
					$row->rsvpdata = $rsvpdata = $rsvpdataArray[$eventid];
				}
			}
			unset($row);

			return;
		}
		$processed[$staticid] = 1;

		$db = Factory::getDBO();
		$sql = "SELECT * FROM #__jev_attendance WHERE ev_id in (" . implode(",", $eventids) . ")";
		$db->setQuery($sql);
		$rsvpdataArray = $db->loadObjectList('ev_id');

		foreach ($rows as & $row) {
			if (isset($row->hidedetail) && $row->hidedetail) {
				continue;
			}

			$eventid = intval($row->ev_id());
			if ($eventid > 0 && array_key_exists($eventid, $rsvpdataArray)) {
				$row->rsvpdata = $rsvpdata = $rsvpdataArray[$eventid];
			}
		}
		unset($row);

		if (!($this->params->get("showattendeesinlists", 0) || $this->params->get("showattendeecountinlists", 0)) || count($rows) == 0) {
			return true;
		}
		$task = Factory::getApplication()->input->getString("jevtask");
		if ($task == 'icalevent.edit' || $task == 'icalevent.edit') {
			return true;
		}

		foreach ($rows as & $row) {
			if (isset($row->hidedetail) && $row->hidedetail) {
				continue;
			}

			$jevrattendance = new JevRsvpDisplayAttendance($this->params);

			$jevrattendance->displayCustomFieldsMultiRow($row, $rsvpdataArray);
		}
		unset($row);
	}

	public function onTranslateEvent(&$row, $language) {

		$eventid = intval($row->ev_id());

		$db = Factory::getDBO();
		$sql = "SELECT * FROM #__jev_attendance WHERE ev_id = " . $eventid;
		$db->setQuery($sql);
		$rsvpdata = $db->loadObject();

		if (!$rsvpdata) {
			return;
		}

		if (!isset($row->customfieldTranslations)) {
			$row->customfieldTranslations = array();
		}

		$sql = "SELECT * FROM #__jev_attendance_translations WHERE atd_id = " . $rsvpdata->id . " and language=" . $db->quote($language);
		$db->setQuery($sql);
		$rsvpdataTranslation = $db->loadObject();

		$customfields = array(
			array("name" => "attendintro", "type" => "html", "label" => Text::_('JEV_ATTEND_INTRO')) ,
			/*array("name" => "subject", "type" => "text", "label" => Text::_('JEV_EMAIL_SUBJECT')),
			array("name" => "message", "type" => "html", "label" => Text::_('JEV_EMAIL_MESSAGE')),
			array("name" => "remindersubject", "type" => "text", "label" => "JEV_REMINDER_EMAIL_SUBJECT"),
			array("name" => "remindermessage", "type" => "html", "label" => Text::_('JEV_REMINDER_EMAIL_MESSAGE')),*/
			array("name" => "sessionaccessmessage", "type" => "text", "label" => Text::_('JEV_SESSION_NOACCESS_MESSAGE')));

		$user = Factory::getUser();
		foreach ($customfields as $field) {
			$id = "rsvppro_" . $field["name"];
			$translation = new stdClass();
			$translation->label = "<strong>" . $field["label"] . "</strong>";
			$name = $field["name"];
			$translation->original = $field["type"] == "html" ? $rsvpdata->$name : nl2br($rsvpdata->$name);
			if (isset($rsvpdataTranslation->$name)) {
				$translation->translation = $rsvpdataTranslation->$name;
			} else {
				$translation->translation = "";
			}
			$fieldname = "translate_customfields[$id]";
			if ($field["type"] = "text") {
				$translation->translation = "<input type='text' value='$translation->translation' name='$fieldname' />";
			} else {
				$translation->translation = "<textarea rows='5' cols='40'   name='$fieldname' >" . $translation->translation . "</textarea>";
			}
			$row->customfieldTranslations[$id] = $translation;
		}

		return true;
	}

	public function onSaveTranslation($translationData) {
		if (!array_key_exists("translate_customfields", $translationData)) {
			return true;
		}

		$eventid = intval($translationData["ev_id"]);

		$db = Factory::getDBO();
		$sql = "SELECT * FROM #__jev_attendance WHERE ev_id = " . $eventid;
		$db->setQuery($sql);
		$rsvpdata = $db->loadObject();

		if (!$rsvpdata) {
			return;
		}

		$sql = "SELECT * FROM #__jev_attendance_translations WHERE atd_id = " . $rsvpdata->id . " and language=" . $db->quote($translationData["trans_language"]);
		$db->setQuery($sql);
		$rsvpdataTranslation = $db->loadObject();

		$customfields = array(
			array("name" => "attendintro", "type" => "html", "label" => Text::_('JEV_ATTEND_INTRO')),
			array("name" => "subject", "type" => "text", "label" => Text::_('JEV_EMAIL_SUBJECT')),
			array("name" => "message", "type" => "html", "label" => Text::_('JEV_EMAIL_MESSAGE')),
			array("name" => "remindersubject", "type" => "text", "label" => "JEV_REMINDER_EMAIL_SUBJECT"),
			array("name" => "remindermessage", "type" => "html", "label" => Text::_('JEV_REMINDER_EMAIL_MESSAGE')),
			array("name" => "sessionaccessmessage", "type" => "text", "label" => Text::_('JEV_SESSION_NOACCESS_MESSAGE')));

		$saveTranslation = array();
		$needToSave = false;
		$user = Factory::getUser();
		foreach ($customfields as $field) {
			$id = "rsvppro_" . $field["name"];
			if (isset($translationData["translate_customfields"][$id])) {
				$needToSave = true;
				$saveTranslation[$field["name"]] = $db->quote($translationData["translate_customfields"][$id]);
			}
			else {
				$saveTranslation[$field["name"]] = "''";
			}
		}
		if (count($saveTranslation)) {
			if ($rsvpdataTranslation){

				$sql = 'REPLACE INTO #__jev_attendance_translations (`id`, `atd_id`, `language`, `attendintro`,`subject`, `message`, `remindersubject`, `remindermessage`, `sessionaccessmessage`)
                            VALUES ( '.$rsvpdataTranslation->id.', '.$rsvpdataTranslation->atd_id.','. $db->quote($translationData["trans_language"]).','. implode(',',$saveTranslation). ')';
			}
			else {
				$sql = 'INSERT INTO #__jev_attendance_translations (`atd_id`,  `language`, `attendintro`,`subject`, `message`, `remindersubject`, `remindermessage`, `sessionaccessmessage`)
                            VALUES ( '.$rsvpdata->id.','. $db->quote($translationData["trans_language"]).','. implode(',',$saveTranslation). ')';
			}
		}
		$db->setQuery($sql);
		$db->execute();
		return true;
	}

	static function fieldNameArray($layout = 'detail') {

		// only offer in detail view
		$plugin = PluginHelper::getPlugin("jevents", "jevrsvppro");

		if (!$plugin)
		{
			return '';
		}
		$params = ComponentHelper::getParams("com_rsvppro");

		$return = array();

		$return['group'] = Text::_("JEV_ATTENDANCE_FIELDS", true);

		$labels = array();
		$values = array();

		if ($layout == "edit") {
			//$labels[] = Text::_("JEV_FIELD_TAB_LINK_ATTENDANCE", true);
			//$values[] = "TABLINK#JEV_ATTENDANCE";
			//$labels[] = Text::_("JEV_FIELD_TAB_LINK_INVITATIONS", true);
			//$values[] = "TABLINK#JEV_INVITATION_OPTIONS";
			//$labels[] = Text::_("JEV_FIELD_TAB_LINK_REMINDERS", true);
			//$values[] = "TABLINK#JEV_REMINDER_OPTIONS";

			$labels[] = Text::_("TAB_BODY_START_ATTENDANCE", true);
			$values[] = "TABSTART#JEV_ATTENDANCE";
			$labels[] = Text::_("TAB_BODY_START_INVITATIONS", true);
			$values[] = "TABSTART#JEV_INVITATION_OPTIONS";
			$labels[] = Text::_("TAB_BODY_START_REMINDERS", true);
			$values[] = "TABSTART#JEV_REMINDER_OPTIONS";

			$labels[] = Text::_("ATTENDANCE_TAB", true);
			$values[] = "JEV_ATTENDANCE";
			$labels[] = Text::_("INVITATIONS_TAB", true);
			$values[] = "JEV_INVITATION_OPTIONS";
			$labels[] = Text::_("REMINDERS_TAB", true);
			$values[] = "JEV_REMINDER_OPTIONS";

			$return['values'] = $values;
			$return['labels'] = $labels;
			return $return;
		}

		if ($layout == "detail") {
			$labels[] = Text::_("JEV_REGOPENING", true);
			$values[] = "REGOPENING";

			$labels[] = Text::_("JEV_REGCLOSED", true);
			$values[] = "REGCLOSED";

			$labels[] = Text::_("JEV_REGCLOSING", true);
			$values[] = "REGCLOSING";

			$labels[] = Text::_("JEV_ATTENDANCE_SUMMARY", true);
			$values[] = "JEV_ATTENDANCE_SUMMARY";

			$labels[] = Text::_("JEV_ATTENDANCE_LOGINFORM", true);
			$values[] = "RSVP_LOGIN";

			$labels[] = Text::_("JEV_TOTAL_CAPACITY", true);
			$values[] = "ATTENDCAP";

			$labels[] = Text::_("JEV_SOCIAL_ATTENDEES", true);
			$values[] = "SOCATTEND";

			$labels[] = Text::_("JEV_INVITEES", true);
			$values[] = "JEV_INVITES";

			$labels[] = Text::_("JEV_ATTENDANCE_FORM", true);
			$values[] = "JEV_ATTENDFORM";
			/*
			  $labels[] = Text::_("JEV_RSVP_MESSAGES",true);
			  $values[] = "JEV_RSVPMSG";
			 */
			$labels[] = Text::_("JEV_REMINDER_FORM", true);
			$values[] = "JEV_REMINDFORM";

			$labels[] = Text::_("JEV_INVITE_CODE", true);
			$values[] = "JEV_INVITECODE";

			$labels[] = Text::_("JEV_ADD_ATTENDEE_BTN", true);
			$values[] = 'ADD_ATTENDEE_BTN';

			$labels[] = Text::_("RSVP_BALANCE_PAID", true);
			$values[] = 'BALANCE_PAID';
		}

		if ($layout != "detail") {
			$labels[] = Text::_("JEV_TOTAL_CAPACITY", true);
			$values[] = "ATTENDCAP";
			$labels[] = Text::_("JEV_REGOPEN", true);
			$values[] = "REGOPEN";
			$labels[] = Text::_("JEV_REGCLOSED", true);
			$values[] = "REGCLOSED";
			$labels[] = Text::_("JEV_REGOPENING", true);
			$values[] = "REGOPENING";
			$labels[] = Text::_("JEV_REGCLOSING", true);
			$values[] = "REGCLOSING";
			$labels[] = Text::_("JEV_BOOKNOW", true);
			$values[] = "BOOKNOW";
			$labels[] = Text::_("JEV_BOOKNOWOPEN", true);
			$values[] = "BOOKNOWOPEN";
		}

		if ($layout == "detail" || $params->get("showattendeesinlists", 0)) {
			$labels[] = Text::_("JEV_ATTENDEES", true);
			$values[] = "ATTENDEES";

			$labels[] = Text::_("JEV_ATTENDEE_NAMES", true);
			$values[] = "ATTENDNAMES";
		}

		if ($layout == "detail" || $params->get("showattendeecountinlists", 0)) {
			$labels[] = Text::_("JEV_ATTENDEE_COUNT", true);
			$values[] = "ATTENDCOUNT";

			$labels[] = Text::_("JEV_ATTENDEE_GUESTS_COUNT", 0);
			$values[] = "ATTENDGUESTS";

			$labels[] = Text::_("JEV_REMAINING_SPACES", true);
			$values[] = "ATTENDSPACE";

			$labels[] = Text::_("JEV_HAS_SPACES", true);
			$values[] = "HASSPACES";

			$labels[] = Text::_("JEV_HAS_WAITING_SPACES", true);
			$values[] = "HASWAITSPACES";

			$labels[] = Text::_("JEV_RESPONDEE_COUNT", true);
			$values[] = "RESPCOUNT";
		}

		if ( $params->get("overrideprice", 0))
		{
			$labels[] = Text::_("JEV_FLATFEE", true);
			$values[] = "FLATFEE";
		}

		$labels[] = Text::_("JEV_TICKET_LINK", true);
		$values[] = "TICKETLINK";

		$labels[] = Text::_("JEV_ATTENDEE_IS_ATTENDING", true);
		$values[] = "ISATTENDING";

		// show attendee status
		if ($layout == "detail" || $params->get("showattendeesinlists", 0)) {
			$labels[] = Text::_("JEV_ATENDEE_STATUS", true);
			$values[] = "ATDEESTATUS";

			$labels[] = Text::_("JEV_ATENDEE_DIDATTEND", true);
			$values[] = "DIDATTEND";
		}

		$labels[] = Text::_("JEV_WAITING_NUMBER", true);
		$values[] = "ATTENDWAITCOUNT";

		$labels[] = Text::_("JEV_WAITING_SPACES",true);
		$values[] = "ATTENDWAIT";

		$labels[] = Text::_("JEV_INVITEES_ONLY",true);
		$values[] = "INVITEESONLY";


		// only offer the reminder in detail view
		if ($layout != "detail") {
			$return['values'] = $values;
			$return['labels'] = $labels;

			return $return;
		}

		$return['values'] = $values;
		$return['labels'] = $labels;

		return $return;
	}

	static function substitutefield($row, $code) {
		if (isset($row->hidedetail) && $row->hidedetail) {
			return "";
		}
		if ($code == "JEV_ATTENDANCE_SUMMARY") {
			if (isset($row->attendancedata))
				return $row->attendancedata;
		}

		if ($code == "BALANCE_PAID") {
			if (isset($row->attendee->feepaid))
				return  RsvpHelper::phpMoneyFormat($row->attendee->feepaid);
			else return "";
		}

		if ($code == "RSVP_LOGIN") {
			if (isset($row->rsvp_loginform))
				return $row->rsvp_loginform;
		}

		if ($code == "ATTENDANCE_OPTIONS" || $code == "INVITATION_OPTIONS" || $code == "EMAIL_REMINDER_OPTIONS") {
			return "";
		}

		static $allowaccessarray = array();
		if (!isset($allowaccessarray[$row->rp_id()])) {
			$user = Factory::getUser();
			if (isset($row->rsvpdata)) {
				$rsvpdata = $row->rsvpdata;
				$allowaccessarray[$row->rp_id()] = plgJEventsJevrsvppro::checkAccess($user, $rsvpdata, $row);
			} else {
				$allowaccessarray[$row->rp_id()] = false;
			}
		}
		$allowaccess = $allowaccessarray[$row->rp_id()];
		if (!$allowaccess) {
			$row->rsvp_attendanceform = isset($rsvpdata->sessionaccessmessage) ? $rsvpdata->sessionaccessmessage : "";
		}

		if ($code == "JEV_ATTENDFORM") {
			if (isset($row->rsvp_attendanceform))
				return $row->rsvp_attendanceform;
		}

		if (!$allowaccess) {
			return "";
		}

		if ($code == "ATTENDEES") {
			if (isset($row->rsvp_fullattendees))
				return $row->rsvp_fullattendees;
		}
		if ($code == "ATTENDNAMES") {
			if (isset($row->attendeenames))
				return implode(", ", $row->attendeenames);
		}

		if ($code == "SOCATTEND") {
			if (isset($row->rsvp_socialattendees))
				return $row->rsvp_socialattendees;
		}
		if ($code == "JEV_INVITES") {
			if (isset($row->rsvp_createinvitations))
				return $row->rsvp_createinvitations;
		}

		if ($code == "JEV_REMINDFORM") {
			if (isset($row->rsvp_reminderform))
				return $row->rsvp_reminderform;
		}

		if ($code == "JEV_RSVPMSG") {
			if (isset($row->rsvp_messagarea))
				return $row->rsvp_messagarea;
		}

		if ($code == "ATTENDCOUNT") {
			$waiting = isset($row->attendeeswaiting) ? $row->attendeeswaiting : 0;

			$attendeeCount = Text::_("RSVP_ATTENDEE_COUNT");
			if ($attendeeCount == "RSVP_ATTENDEE_COUNT") {
				$attendeeCount = "%s";
			}
			if (isset($row->rsvpdata) && $row->rsvpdata->allowregistration > 0) {
				if (isset($row->attendeeCount))
					return Text::sprintf($attendeeCount, intval($row->attendeeCount) - $waiting);
				else if (isset($row->_attendeeCount))
					return Text::sprintf($attendeeCount, intval($row->_attendeeCount) - $waiting);
				else
					return Text::sprintf($attendeeCount, 0);
			}
			if (Text::_("RSVP_NO_ATTENDANCE") == "RSVP_NO_ATTENDANCE") {
				return "";
			}
			return Text::_("RSVP_NO_ATTENDANCE");
		}

		if ($code == "RESPCOUNT") {
			$respCountFmt = Text::_("RSVP_RESPONDEE_COUNT");
			if ($respCountFmt == "RSVP_RESPONDEE_COUNT") {
				$respCountFmt = "%s";
			}
			if (isset($row->rsvpdata) && $row->rsvpdata->allowregistration > 0) {
				$sql = "SELECT count(a.id) FROM #__jev_attendees as a WHERE a.at_id=" . $row->rsvpdata->id;
				if (!$row->rsvpdata->allrepeats) {
					$sql .= " and a.rp_id=" . $row->rp_id();
				}
				$db = Factory::getDBO();
				$db->setQuery($sql);
				// convert to integer in case no entry (i.e. zero confirmed attendees)
				$respCount = intval($db->loadResult());

				return Text::sprintf($respCountFmt, intval($respCount));
			}
			if (Text::_("RSVP_NO_ATTENDANCE") == "RSVP_NO_ATTENDANCE") {
				return "";
			}
			return Text::_("RSVP_NO_ATTENDANCE");
		}

		if ($code == "ATTENDGUESTS") {
			$attendeeguestcount = Text::_("RSVP_ATTENDEE_GUESTS_COUNT");
			if ($attendeeguestcount == "RSVP_ATTENDEE_GUESTS_COUNT") {
				$attendeeguestcount = "%s";
			}
			if (isset($row->registrationguestcount))
				return Text::sprintf($attendeeguestcount, $row->registrationguestcount);
			else if (isset($row->_registrationguestcount))
				return Text::sprintf($attendeeguestcount, $row->_registrationguestcount);
			else if (isset($row->rsvpdata) && $row->rsvpdata->allowregistration > 0)
				return Text::sprintf($attendeeguestcount, 0);
			else {
				if (Text::_("RSVP_NO_ATTENDANCE") == "RSVP_NO_ATTENDANCE") {
					return "";
				}
				return Text::_("RSVP_NO_ATTENDANCE");
			}
		}

		if ($code == "ATTENDCAP") {
			if (isset($row->rsvpdata) && isset($row->rsvpdata->capacity) && $row->rsvpdata->capacity > 0)
				return $row->rsvpdata->capacity;
			else
				return Text::_("RSVP_NO_REGISTRATION_LIMIT");
		}

		if ($code == "FLATFEE") {
			if (isset($row->rsvpdata) && isset($row->rsvpdata->overrideprice) && $row->rsvpdata->overrideprice != "") {
				$registry = JevRegistry::getinstance("jevents");
				$registry->set("rsvpdata", $row->rsvpdata);
				$registry->set("event", $row);

				return RsvpHelper::phpMoneyFormat($row->rsvpdata->overrideprice);
			}
		}

		if ($code == "ATTENDSPACE" || $code == "HASSPACES" || $code == "HASWAITSPACES") {
			if (isset($row->rsvpdata) && isset($row->rsvpdata->capacity) && $row->rsvpdata->capacity > 0 && isset($row->attendeeCount))
				$space = intval($row->rsvpdata->capacity) - intval($row->attendeeCount);
			else if (isset($row->rsvpdata) && isset($row->rsvpdata->capacity) && $row->rsvpdata->capacity > 0 && isset($row->_attendeeCount))
			{
				$space = intval($row->rsvpdata->capacity) - intval($row->_attendeeCount);
				$row->attendeeCount = $row->_attendeeCount;
			}
			else if (isset($row->rsvpdata) && isset($row->rsvpdata->capacity) && $row->rsvpdata->capacity > 0)
				$space = intval($row->rsvpdata->capacity);
			else
				return Text::_('RSVP_ATTENDSPACE_EMPTY');

			if ($code == "HASWAITSPACES")
			{
				$waitingspace = 0;
				if ($space <= 0 &&  isset($row->attendeeCount))
				{
					if (isset($row->rsvpdata) && isset($row->rsvpdata->waitingcapacity) && $row->rsvpdata->waitingcapacity > 0)
					{
						$waitingspace = $space + $row->rsvpdata->waitingcapacity;
					}
					if ($waitingspace > 0)
					{
						return 1;
					}
				}
				return "";
			}
			if ($space > 0) {
				if ($code == "HASSPACES") {
					return 1;
				}
				if (Text::_("RSVP_REMAINING_SPACES") == "RSVP_REMAINING_SPACES") {
					return $space;
				} else {
					$format = Text::_("RSVP_REMAINING_SPACES");
					if (strpos($format, "[max") > 0) {
						$pattern = "#\[max([0-9]+)\]#";
						preg_match($pattern, $format, $matches);
						if (count($matches) == 2 && $space <= intval($matches[1])) {
							$format = str_replace($matches[0], "", $format);
							return sprintf($format, $space);
						}
						return "";
					}
					return Text::sprintf("RSVP_REMAINING_SPACES", $space);
				}
			} else if ($space == 0) {
				if ($code == "HASSPACES") {
					return "";
				}

				if (Text::_("JEV_NO_REMAINING_SPACES") == "JEV_NO_REMAINING_SPACES") {
					return $space;
				} else {
					return Text::_("JEV_NO_REMAINING_SPACES");
				}
			} else {
				if ($code == "HASSPACES") {
					return "";
				}
				return $space;
			}
		}

		if ($code == "ATTENDWAIT") {
			if (isset($row->rsvpdata) && isset($row->rsvpdata->waitingcapacity) && $row->rsvpdata->waitingcapacity > 0)
				return $row->rsvpdata->waitingcapacity;
			else
				return 0;
		}

		if ($code == "ATTENDWAITCOUNT") {
			if (isset($row->_attendeeswaiting))
				return $row->_attendeeswaiting;
			else if (isset($row->attendeeswaiting))
				return $row->attendeeswaiting;
			else
				return 0;
		}

		if ($code == "ATDEESTATUS") {
			if (isset($row->attendee) && isset($row->attendee->attendstate)) {
				$attendstates = array("JEV_ARE_NOT_ATTENDING", "JEV_ARE_ATTENDING", "JEV_MAY_BE_ATTENDING", "JEV_ATTENDING_PENDING_APPROVAL", "JEV_YOUR_ATTENDANCE_IS_AWAITING_PAYMENT");
				$state = Text::_($attendstates [$row->attendee->attendstate]);
				if ($row->attendee->waiting) {
					$state .= " : " . Text::_("JEV_YOU_ARE_ON_WAITINGLIST");
				}
				if (!$row->attendee->confirmed) {
					$state .= " : " . Text::_("JEV_EMAIL_AWAITING_CONFIRMATION");
				}
				return $state;
			} else {
				return "";
			}
		}

		if ($code == "ISATTENDING") {
			if (isset($row->attendee) && isset($row->attendee->attendstate) && $row->attendee->attendstate==1) {
				return Text::_("JEV_IS_ATTENDING");
			}
		}

		if ($code == "DIDATTEND") {
			if (isset($row->attendee) && isset($row->attendee->didattend)) {
				$state = Text::_("RSVP_ATTENDED");
				if ($row->attendee->didattend) {
					$state .= " : " . Text::_("JYES");
				} else {
					$state .= " : " . Text::_("JNO");
				}
				return $state;
			} else {
				return "";
			}
		}

		if ($code == "REGOPEN" || $code == "REGCLOSED" || $code == "REGOPENING" || $code == "REGCLOSING") {
			// detail page doesnt set these automatically
			if (isset($row->rsvpdata) && !isset($row->_regopen)) {
				$row->_regopen = $row->rsvpdata->regopen;
				$row->_regclose = $row->rsvpdata->regclose;
				$row->_allowregistration = $row->rsvpdata->allowregistration;
				$row->_allrepeats = $row->rsvpdata->allrepeats;
				$row->_sessionaccess = $row->rsvpdata->sessionaccess;
			}

			if (isset($row->rsvpdata) && isset($row->rsvpdata->capacity) && $row->rsvpdata->capacity > 0 && isset($row->attendeeCount))
				$space = intval($row->rsvpdata->capacity) - intval($row->attendeeCount);
			else if (isset($row->rsvpdata) && isset($row->rsvpdata->capacity) && $row->rsvpdata->capacity > 0 && isset($row->_attendeeCount))
				$space = intval($row->rsvpdata->capacity) - intval($row->_attendeeCount);
			else if (isset($row->rsvpdata) && isset($row->rsvpdata->capacity) && $row->rsvpdata->capacity > 0)
				$space = intval($row->rsvpdata->capacity);
			else
				$space = 99;
			if (isset($row->rsvpdata) && isset($row->rsvpdata->waitingcapacity) && $row->rsvpdata->waitingcapacity > 0)
			{
				$space += $row->rsvpdata->waitingcapacity;
			}
			if ($space <= 0) {
				if (Text::_("JEV_NO_REMAINING_SPACES") == "JEV_NO_REMAINING_SPACES") {
					return "";
				} else {
					if ($code == "REGOPEN" || $code == "REGCLOSED") {
						return "";
					}
					return Text::_("JEV_NO_REMAINING_SPACES");
				}
			}

			if (isset($row->_regopen) && $row->_regopen != "0000-00-00 00:00:00" && $row->_allowregistration) {
				// attendance by invitees only?
				if ($row->_allowregistration == 2 && isset($row->rsvpdata)) {
					$user = Factory::getUser();
					if ($user->id > 0) {
						// is this user invited?
						$db = Factory::getDBO();
						$db->setQuery("select user_id from #__jev_invitees where at_id=" . intval($row->rsvpdata->id) . " AND user_id=" . intval($user->id));
						$res = $db->loadResult();
						// not invited to return blank
						if (!$res && $row->created_by() != $user->id && !JEVHelper::isAdminUser() && !JEVHelper::canDeleteEvent($row, $user)) {
							return "";
						}
					} else {
						// can't do this for non-logged in users
						return "";
					}
				}
				// Must use strtotime format for force JevDate to not just parse the date itself!!!
				$jnow = new JevDate("+1 second");
				// use toMySQL to pick up timezone effects
				$now = $jnow->toMySQL();
				$regclose = $row->_regclose;
				$regopen = $row->_regopen;
				if ($row->_allrepeats) {
					if ($now > $regclose || $now < $regopen) {
						if ($now > $regclose && $code == "REGCLOSED") {
							return Text::_("JEV_REGCLOSED_MESSAGE");
						}

						if ($now < $regopen && $code == "REGOPENING") {
							$regopening = new JevDate(JevDate::strtotime($regopen));
							return Text::sprintf("JEV_REGOPENING_MESSAGE", $regopening->toFormat(Text::_("JEV_REGOPENING_FORMAT")));
						}
						return "";
					}
					if ($now < $regclose && $code == "REGCLOSING") {
						$regclosing = new JevDate(JevDate::strtotime($regclose));
						return Text::sprintf("JEV_REGCLOSING_MESSAGE", $regclosing->toFormat(Text::_("JEV_REGCLOSING_FORMAT")));
					} else if ($code == "REGOPEN") {
						return Text::_("JEV_REGOPEN_MESSAGE");
					} else {
						return "";
					}
				}
				// otherwise the start of the repeat
				else {
					// need to use corrected dtstart here!
					$eventstart = isset($row->_fixed_dtstart) ? $row->_fixed_dtstart : $row->dtstart();
					$repeatstart = $row->getUnixStartTime();
					$regclose = new JevDate(JevDate::strtotime($row->_regclose));
					$regclose = $regclose->toUnix();
					$regopen = new JevDate(JevDate::strtotime($row->_regopen));
					$regopen = $regopen->toUnix();

					$adjustedregclose = new JevDate($regclose + ($repeatstart - $eventstart));
					$adjustedregopen = new JevDate($regopen + ($repeatstart - $eventstart));
					// use toMySQL to pick up timezone effects
					$adjustedregcloseMySQL = $adjustedregclose->toMySQL();
					$adjustedregopenMySQL = $adjustedregopen->toMySQL();

					if ($code == "REGCLOSED") {
						if ($now > $adjustedregcloseMySQL) {
							return Text::_("JEV_REGCLOSED_MESSAGE");
						}
						return "";
					}
					if ($code == "REGCLOSING") {
						if ($now < $adjustedregcloseMySQL) {
							// local timezone!
							return Text::sprintf("JEV_REGCLOSING_MESSAGE", $adjustedregclose->toFormat(Text::_("JEV_REGCLOSING_FORMAT")));
						}
						return "";
					}

					if ($now < $adjustedregcloseMySQL && $now > $adjustedregopenMySQL) {
						if ($code == "REGOPEN") {
							return Text::_("JEV_REGOPEN_MESSAGE");
						} else {
							return "";
						}
					}
					if ($code == "REGOPENING") {
						if ($now < $adjustedregopenMySQL) {
							return Text::sprintf("JEV_REGOPENING_MESSAGE", $adjustedregopen->toFormat(Text::_("JEV_REGOPENING_FORMAT")));
						}
					}
				}
			} else
				return "";
		}
		if ($code == "BOOKNOW") {
			if (isset($row->rsvpdata) && $row->rsvpdata->allowregistration > 0) {
				$link = $row->viewDetailLink($row->yup(), $row->mup(), $row->dup(), true);
				return "<a href='$link' title='" . htmlspecialchars(Text::_("JEV_BOOKNOW_TEXT")) . "' >" . Text::_("JEV_BOOKNOW_TEXT") . "</a>";
			}
		}
		//Show when open only
		if ($code == "BOOKNOWOPEN") {
			if (isset($row->rsvpdata) && $row->rsvpdata->allowregistration > 0) {
				$jnow = new JevDate("+1 second");
				// use toMySQL to pick up timezone effects
				$now = $jnow->toMySQL();
				$eventstart = isset($row->_fixed_dtstart) ? $row->_fixed_dtstart : $row->dtstart();
				$repeatstart = $row->getUnixStartTime();

				$regenabled = $row->_allowregistration;
				$regclose = new JevDate(JevDate::strtotime($row->_regclose));
				$regclose = $regclose->toUnix();
				$regopen = new JevDate(JevDate::strtotime($row->_regopen));
				$regopen = $regopen->toUnix();
				$adjustedregclose = new JevDate($regclose + ($repeatstart - $eventstart));
				$adjustedregopen = new JevDate($regopen + ($repeatstart - $eventstart));
				// use toMySQL to pick up timezone effects
				$adjustedregclose = $adjustedregclose->toMySQL();
				$adjustedregopen = $adjustedregopen->toMySQL();

				if ($now < $adjustedregclose && $regenabled == 1) {
					$link = $row->viewDetailLink($row->yup(), $row->mup(), $row->dup(), true);
					return "<a href='$link' title='" . htmlspecialchars(Text::_("JEV_BOOKNOWOPEN_TEXT")) . "' >" . Text::_("JEV_BOOKNOWOPEN_TEXT") . "</a>";
				}
			}
		}
		if ($code == "TICKETLINK") {
			if (isset($row->ticketlink) && $row->ticketlink != "") {
				return $row->ticketlink;
			}
			else if(isset($row->rsvpdata)) {
				static $pluginparams = false;
				if (!$pluginparams){
					$plugin = PluginHelper::getPlugin("jevents", "jevrsvppro");
					$pluginparams = new Registry ($plugin->params);
				}
				$jevrattendees = new JevRsvpAttendees($pluginparams, false , $rsvpdata);
				$jevrattendees->view->setLayout("attendee");
				// For sites that want tickets in list views
				if (!isset($row->rsvp_attendanceform) && Factory::getApplication()->input->getCmd('task') != "icalrepeat.detail")
				{
					if (!isset($row->attendee)){
						$emailaddress = $jevrattendees->getEmailAddress("em2");
						if (!$emailaddress)
						{
							$emailaddress = $jevrattendees->getEmailAddress("em");
						}
						$attendee = $jevrattendees->isAttending($row->rsvpdata, $row, $emailaddress, true);
						$attending = false;
						if ($attendee)
						{
							$attending = true;
						}
						$row->attendee = $attendee;
					}
					$jevrattendees->attendee = $row->attendee;

					if (Factory::getApplication()->input->getCmd("task")!="accounts.paymentpage"){
						$jevrattendees->view->attendee = $row->attendee;
						$jevrattendees->view->rsvpdata =  $row->rsvpdata;
						$jevrattendees->view->row =  $row;
						// ticket template uses a static for this !
						unset($jevrattendees->view->templateInfo);
						$row->ticketlink = $jevrattendees->view->loadTemplate("ticket");
						return $row->ticketlink;
					}
				}
			}
		}

		if ($code == "JEV_INVITECODE") {
			// If universal code is provided then skip this for event detail views
			if (Factory::getApplication()->input->getCmd('task') == "icalrepeat.detail" && Factory::getApplication()->input->getInt('evid') > 0) {
				$rsvpparams = ComponentHelper::getParams("com_rsvppro");
				$code = md5($rsvpparams->get("emailkey", "email key") . "##" . Factory::getApplication()->input->getInt('evid') . "universal access");
				$user = Factory::getUser();
				if ($row->created_by() != $user->id && !JEVHelper::isAdminUser() && !JEVHelper::canDeleteEvent($row, $user)) {
					return "";
				}
				$link = $row->viewDetailLink($row->yup(), $row->mup(), $row->dup(), false) . "&ac=" . $code;
				$link = Route::_($link);
				return Text::_("RSVP_DIRECT_ACCESS_LINK") . " <a href='$link' title='" . htmlspecialchars(Text::_("RSVP_DIRECT_ACCESS_LINK")) . "' >" . $link . "</a>";
			}
		}

		if ($code == 'ADD_ATTENDEE_BTN' && RsvpHelper::canCreateSessions()) {

			// Load the modal behavior script.
JLoader::register('JevModal', JPATH_LIBRARIES . "/jevents/jevmodal/jevmodal.php");
JevModal::modal("a.jevmodal");
			$jinput = Factory::getApplication()->input;
			$itemid = $jinput->getInt('Itemid', 0);
			$rp_id  = $row->rsvpdata->allrepeats === '1' ? '0' : $row->rp_id;
			$link = 'index.php?option=com_rsvppro&task=attendees.edit&atd_id[]=' . $row->rsvpdata->id . '|' . $rp_id  . '&rp_id=' . $row->rp_id . '&Itemid=' . $itemid . '&tmpl=component&' . Session::getFormToken() . '=1';
			$link = Route::_($link);
			return "<a href='$link' title='" . htmlspecialchars(Text::_("RSVP_ADD_ATTENDEE_BTN_LINK")) . "' class=\"btn jevmodal\" rel=\"{handler: 'iframe', size: {x:600, y:500}}\" >" . Text::_('RSVP_ADD_ATTENDEE_BTN_LINK') . "</a>";
		}

		if ($code == 'INVITEESONLY' && isset($row->rsvpdata) && $row->rsvpdata->hidenoninvitees)
		{
			return Text::_("RSVP_EVENT_VISIBLE_TO_INVITEES_ONLY");
		}

		return '';
	}

	static function checkAccess($user, $rsvpdata, $event) {
		if ($rsvpdata->sessionaccess >= 0) {
			$access = $rsvpdata->sessionaccess;
		} else {
			$access = $event->access();
		}
		if (version_compare(JVERSION, "1.6.0", 'ge')) {
			$access = explode(",", $access);
			return count(array_intersect($access, JEVHelper::getAid($user, 'array'))) > 0;
		} else {
			return $access <= $user->aid;
		}
	}

}
