<?php

// no direct access
defined('_JEXEC') or die('Restricted access');

jimport('joomla.plugin.plugin');
jimport('joomla.log.log');

// Load Omnipay autoloader
require JPATH_SITE . '/libraries/jev_omnipay/vendor/autoload.php';

// Use Omnipay
use Joomla\Registry\Registry;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Log\Log;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\Session\Session;
use Joomla\CMS\Factory;
use Omnipay\Omnipay;

class plgRsvpproMollie extends CMSPlugin
{
	public function __construct(& $subject, $config)
	{

		// Enable Logging
		Log::addLogger(
			array(
		    'text_file' => 'com_rsvppro.mollie.log.php'
			), Log::ALL, 'com_rsvppro.mollie'
		);

		parent::__construct($subject, $config);
	}

	public function generatePaymentPage(&$html, $attendee, $rsvpdata, $event, &$transaction)
	{

		$plugin = PluginHelper::getPlugin("rsvppro", "mollie");
		$params = new Registry($plugin->params);

		$jinput = Factory::getApplication()->input;

		$invoice = $jinput->getString("invoiceid", '') . '_' . $transaction->transaction_id;

		// Amount, format number to 2 for Mollie to be happy.
		$amount = number_format(RsvpHelper::ceil_dec($jinput->getFloat("amount", 0), 2, "."), 2);

		$detaillink = Route::_(Uri::root() . $event->viewDetailLink($event->yup(), $event->mup(), $event->dup(), false), false);

		$currency = $params->get("Currency", "GBP");

		if (isset($rsvpdata->template) && is_numeric($rsvpdata->template))
		{
			$db = Factory::getDbo();
			$db->setQuery("Select params from #__jev_rsvp_templates where id=" . intval($rsvpdata->template));
			$templateParams = $db->loadObject();
			if ($templateParams)
			{
				$templateParams = json_decode($templateParams->params);
				$templateParams = RsvpHelper::translateTemplateParams($rsvpdata->template, $templateParams);

				// Clear if using Global Params
				if ((int) $templateParams->mleuseglobalsettings === 1)
				{
					$templateParams = $params;
				}
			}
			else
			{
				$templateParams = $params;
			}
		}
		else
		{
			$templateParams = $params;
		}

		$comparams = ComponentHelper::getParams("com_rsvppro");
		$user = JEVHelper::getUser($attendee->user_id);
		if ((int) $user->id === 0 && RsvpHelper::getTemplateSpecificParameter($rsvpdata,  "attendemails", 0))
		{
			$name = $attendee->email_address;
			$username = $attendee->email_address;
		}
		else
		{
			$name = $user->name;
			$username = $user->username;
		}

		// Load the attendee helper
		include_once(JPATH_ADMINISTRATOR . "/components/com_rsvppro/libraries/attendeehelper.php");

		$helper = new RsvpAttendeeHelper($comparams);

		$html = $helper->parseMessage($html, $rsvpdata, $event, $name, $username, $attendee, false, false, $transaction);

		$currency = isset($templateParams->Currency) ? $templateParams->Currency : $currency;
		$start_date = JEventsHTML::getDateFormat($event->yup(), $event->mup(), $event->dup(), 0);


		$mode_val = isset($templateParams->MollieMode) ? (int) $templateParams->MollieMode : (int) $templateParams->get('MollieMode');
		$testmode = $mode_val === 0 ? true : false;

		$keyfield = $testmode ? "mle_testapikey" : "mle_apikey";
		$apikey = isset($templateParams->$keyfield) ? $templateParams->$keyfield : $params->get($keyfield, "");

		// Mollie Params
		$mle = new stdClass();
		$mle->invoice = $invoice;
		$mle->apikey = $apikey;
		$mle->amount = $amount;
		$mle->currency = $currency;
		$mle->detaillink = $detaillink;
		$mle->transactionreference = "";

		// setup transaction data
		$transaction->amount = $amount;
		$transaction->currency = $currency;
		$transaction->attendee_id = $attendee->id;
		$transaction->gateway = 'mollie';
		$transaction->params = new stdClass();
		$transaction->params->apikey = $mle->apikey;
		$transaction->params->mode = $testmode;
		$transaction->params = json_encode($transaction->params);

		// Save the transaction data
		$transaction->store();

		$mle->rsvp_trans_id = $transaction->transaction_id;

		$data = new stdClass;
		$data->start_date = $start_date;
		$data->amount = $amount;
		$data->currency = $currency;
		$data->invoice = $invoice;
		/*
		if (!isset($event->firstrepeat))
		{
			$event->firstrepeat = $event->getOriginalFirstRepeat();
		}
		$data->title = $event->firstrepeat->title;
		 */
		$data->title = $event->title;
		$data->detaillink = $detaillink;

		// OmniPay Mollie
		$gateway = Omnipay::create('Mollie');

		$gateway->setApiKey($mle->apikey);

		$session = Session::getInstance('none', array());
		$session->set('rsvppro_checkout', json_encode($mle));

		// Send the purchase
		$response = $gateway->purchase(
				[
				    'amount' => $data->amount,
				    'currency' => $data->currency,
				    'invoice' => $data->invoice,
				    'description' => htmlspecialchars($data->title . " - " . $data->start_date),
				    'item_name' => htmlspecialchars($data->title . " - " . $data->start_date),
				    'cancelUrl' => $data->detaillink,
				    'notifyUrl' => JURI::root() . "index.php?option=com_rsvppro&task=accounts.notify&gateway=mollie&invoice=" . $mle->invoice,
				    'returnUrl' => JURI::root() . "index.php?option=com_rsvppro&task=accounts.notify&gateway=mollie"
				]
			)->send();

		// NOTE: we are using purchase and NOT completePurchase, so this sends the client off site and we call completePurchase later.

		if ($response->isRedirect())
		{
			// Store the Mollie transaction ID in your local database

			$transaction->gw_transid = $response->getTransactionReference(); // Transaction Reference, for refunds etc.
			$mle->transactionreference = $transaction->gw_transid;
			$session->set('rsvppro_checkout', json_encode($mle));


			// Redirect to the Mollie payment screen
			$response->redirect();
		}
		else
		{
			// Payment failed: display message to the customer
			echo $response->getMessage();
		}
	}

	static public function notifyRsvpGateway(&$args)
	{

		$app = Factory::getApplication();
		$session = Session::getInstance('none', array());
		$jinput = $app->input;

		$params = ComponentHelper::getParams(RSVP_COM_COMPONENT);

		// Get Mollie Credentials from current users session
		$mle = json_decode($session->get('rsvppro_checkout'));

		$gateway = Omnipay::create('Mollie');
		$gateway->setApiKey($mle->apikey);

		// We run this to complete the purchase and validate it was successful
		$response = $gateway->completePurchase(array(
		'amount' => $mle->amount,
		'currency' => $mle->currency,
		'token' => $jinput->getString('token', ''),
		'PayerID' => $jinput->getString('PayerID', ''),
		'transactionReference' => $mle->transactionreference,
		'notifyUrl' => JURI::root() . "index.php?option=com_rsvppro&task=accounts.notify&gateway=mollie&ipn=1&invoice=" . $mle->invoice


		))->send();


		$transaction = new rsvpTransaction();
		$transaction->load($mle->rsvp_trans_id);

		// Send confirmation email setting, set false unless payment is successful.
		$send_cm = false;


		if ($response->isPending())
		{
			// pending sale flow
			$transaction->logdata = 'Transaction is Pending, please check your Mollie Account: ' . $response->getMessage(); // Any message the gateway his given us.
		}
		else if ($response->isSuccessful())
		{
			// normal successful flow
			$transaction->logdata = 'Payment was Successful: ' . $response->getMessage();
			$transaction->paymentstate = 1;

			$send_cm = true;
		}
		else if ($response->isRedirect())
		{
			// redirect back to Mollie since there was a payment error, i.e. funding source
			$app->redirect($response->getRedirectUrl());

			return;
		}
		else
		{
			// normal failure flow
			$transaction->logdata = 'There was an error, Mollie responded with: ' . $response->getMessage(); // Any message the gateway his given us.
		}

		$transaction->gw_transid = $response->getTransactionReference(); // Transaction Reference, for refunds etc.
		// Store updated transaction
		$transaction->store();

		// Send Payment Confirmation
		if ($send_cm)
		{
			// fetch the session and the attendee
			$db = Factory::getDbo();
			$sql = "SELECT * FROM #__jev_attendees WHERE id=" . $transaction->attendee_id;
			$db->setQuery($sql);
			$attendee = $db->loadObject();

			$sql = "SELECT * FROM #__jev_attendance WHERE id=" . $attendee->at_id;
			$db->setQuery($sql);
			$rsvpdata = $db->loadObject();

			plgRsvpproMollie::notifyMolliePayment($transaction, $attendee, $rsvpdata);
		}
		// Session cleanup
		$session->clear('rsvppro_checkout');

		$app->redirect($mle->detaillink);
	}

	protected function Mollie_Message($text, $email)
	{
		plgRsvpproMollie::FixEmail($email);
		if ($email == '')
		{
			return;
		}
		$mail = Factory::getMailer();

		try {
		return $mail->sendMail($email, 'mollie log', $email, 'Mollie Message', $text, 0);
		}
		catch (Exception $e)
		{
			return false;
		}

	}

	protected function Mollie_Error($text, $email)
	{
		plgRsvpproMollie::FixEmail($email);
		if ($email == '')
		{
			return;
		}
		$mail = Factory::getMailer();

		try {
		return $mail->sendMail($email, 'mollie log', $email, 'Mollie Error', $text, 0);
		}
		catch (Exception $e)
		{
			return false;
		}

	}

	protected static function FixEmail(& $email)
	{
		if ($email == '')
		{
			// get Jevents params and send messages to admin if necessary
			$params = ComponentHelper::getParams("com_jevents");
			$jevadmin = $params->get("jevadmin", -1);
			if ($jevadmin == -1)
			{
				return;
			}
			$jevadmin = JEVHelper::getUser($jevadmin);
			$email = $jevadmin->email;
		}
	}

	private static function notifyMolliePayment($transaction, $attendee, $rsvpdata, $aftertransation = false)
	{
		// immediate notification
		if (((!$aftertransation && RsvpHelper::getTemplateSpecificParameter($rsvpdata,  "notifymlepay", 1) == 2) || ($aftertransation && RsvpHelper::getTemplateSpecificParameter($rsvpdata,  "notifymlepay", 1) == 1)) && $transaction->gateway == "mollie")
		{

			$comparams = ComponentHelper::getParams("com_rsvppro");
			include_once(JPATH_ADMINISTRATOR . "/components/com_rsvppro/libraries/attendeehelper.php");
			$helper = new RsvpAttendeeHelper($comparams);

			$rpid = $attendee->rp_id;

			$dataModel = new JEventsDataModel();
			$queryModel = new JEventsDBModel($dataModel);

			// Find the first repeat
			$vevent = $dataModel->queryModel->getEventById($rsvpdata->ev_id, false, "icaldb", false);
			if ($rpid == 0)
			{
				$repeat = $vevent->getFirstRepeat();
			}
			else
			{
				list($year, $month, $day) = JEVHelper::getYMD();
				$repeatdata = $dataModel->getEventData(intval($rpid), "icaldb", $year, $month, $day);
				if ($repeatdata && isset($repeatdata["row"]))
					$repeat = $repeatdata["row"];
			}

			$user = JEVHelper::getUser($attendee->user_id);
			if ($user->id == 0 && RsvpHelper::getTemplateSpecificParameter($rsvpdata,  "attendemails", 0))
			{
				$name = $attendee->email_address;
				$username = $attendee->email_address;
			}
			else
			{
				$name = $user->name;
				$username = $user->username;
			}

			$class = "plgRsvppro" . ucfirst($transaction->gateway);
			$pluginpath = version_compare(JVERSION, "1.6.0", 'ge') ? JPATH_SITE . "/plugins/rsvppro/$transaction->gateway/" : JPATH_SITE . "/plugins/rsvppro/";
			JLoader::register($class, $pluginpath . $transaction->gateway . ".php");

			$helper->notifyUser($rsvpdata, $repeat, $user, $name, $username, $attendee, 'mlepay', false, $transaction);
		}
	}

	public function phpMoneyFormat($amount, $template)
	{
		if (isset($template->Currency))
		{
			$digits = $template->CurrencyDigits;
			$symbol = $template->CurrencySymbol;
			$onLeft = (strcmp($template->CurrencyPlacement, 'left') == 0);
			$separator = $template->CurrencySeparator;
			$decimal = $template->CurrencyDecimal;
		}
		else
		{
			$eSessParams = ComponentHelper::getParams('com_rsvppro');

			$digits = $eSessParams->get("CurrencyDigits");
			$symbol = $eSessParams->get("CurrencySymbol");
			$onLeft = (strcmp($eSessParams->get("CurrencyPlacement"), 'left') == 0);
			$separator = $eSessParams->get("CurrencySeparator");
			$decimal = $eSessParams->get("CurrencyDecimal");
		}
		$formattedText = '';

		// negative amount
		if ($amount < 0)
			$formattedText .= '-';

		// currency symbol on the left
		if ($onLeft)
			$formattedText .= $symbol;

		// format with correct number of digits and separator
		if ($digits > 0)
		{
			$amount = RsvpHelper::ceil_dec(abs($amount), $digits, ".");
			$formattedText .= number_format(abs($amount), $digits, $decimal, $separator);
		}
		else
		{
			$formattedText = abs($amount);
		}

		// currency symbols on the right
		if (!$onLeft)
			$formattedText .= $symbol;


		return $formattedText;
	}

	public static function NotifyPayment($templateParams)
	{
		return $templateParams->get("notifymlepay", 1);
	}

	public static function PaymentMessageType()
	{
		return "mlepay";
	}

	static public function transactionDetailLink($transaction)
	{
		return "";

		/*
		$plugin = PluginHelper::getPlugin("rsvppro", "mollie");
		$params = new Registry($plugin->params);

		$transactionLogdata = json_decode($transaction->logdata);

		$mode_val = (int) $params->get('MollieMode');

		if ($mode_val === 0)
		{
			$host = 'https://www.sandbox.mollie.com';
		}
		else
		{
			$host = 'https://www.mollie.com';
		}

		return '<a href="https://' . $host . '/vst/id=' . $transactionLogdata->request->txn_id . '" target="_blank">' . $transactionLogdata->txn_id . '</a>';
		 */
	}

	public function onActiveGatewayClass(&$activeGatewayClass, $action = "notify")
	{
		$gateway = Factory::getApplication()->input->getString("gateway");

		if ($gateway == "mollie" || $gateway == "1" || strpos($gateway, "mollie_") === 0)
		{
			$activeGatewayClass = __CLASS__;
		}
	}

	public function activeGateways(&$activeGatewayClasses)
	{
		$activeGatewayClasses[] = __CLASS__;
	}

}
