<?php

/**
 * JEvents Component for Joomla 1.5.x
 *
 * @version     $Id$
 * @package     JEvents
 * @copyright   Copyright (C) 2015-2021 GWE Systems Ltd
 * @license     GNU/GPLv2, see http://www.gnu.org/licenses/gpl-2.0.html
 */
// no direct access
defined('_JEXEC') or die('Restricted access');

use Joomla\CMS\Language\Language;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Session\Session;
use Joomla\CMS\Factory;

function gwejson_skiptoken()
{
	// This file checks the token
	return true;

}

function ProcessJsonRequest(&$requestObject, $returnData)
{
	$returnData->discount = 0;
	$returnData->surcharge = 0;

	$lparams = ComponentHelper::getParams('com_languages');
	$sitelang = $lparams->get('site', 'en-GB');
	$sitelang = Language::getInstance($sitelang, 0);
	Factory::$language = $sitelang;

	// Load Plugin Language Files!
	$lang = Factory::getLanguage();
	$lang->load("plg_jevents_jevrsvppro", JPATH_ADMINISTRATOR);

	$params = ComponentHelper::getParams("com_rsvppro");

	// Enforce referrer
	if ($params->get("testreferrer", 0))
	{
		if (!array_key_exists("HTTP_REFERER", $_SERVER))
		{
			PlgSystemGwejson::throwerror("There was an error");
		}

		$live_site = $_SERVER['HTTP_HOST'];
		$ref_parts = parse_url($_SERVER["HTTP_REFERER"]);

		if (!isset($ref_parts["host"]) || $ref_parts["host"] != $live_site)
		{
			PlgSystemGwejson::throwerror("There was an error - missing host in referrer");
		}
	}


	if (!isset($requestObject->rp_id) || $requestObject->rp_id == 0)
	{
		PlgSystemGwejson::throwerror("There was an error - repeat id");
	}

	if (!isset($requestObject->fieldid) || intval($requestObject->fieldid) == 0)
	{
		PlgSystemGwejson::throwerror("There was an error - no such field");
	}

	$token = Session::getFormToken();

	if (!isset($requestObject->token) || $requestObject->token != $token)
	{
		PlgSystemGwejson::throwerror("There was an error - bad token.  Please refresh the page and try again.");
	}

	$db = Factory::getDBO();
	$db->setQuery("SELECT * FROM #__jevents_repetition as rpt LEFT JOIN #__jevents_vevent as evt on evt.ev_id=rpt.eventid where rp_id=" . (int) $requestObject->rp_id);
	$event = $db->loadObject();

	if (!$event)
	{
		PlgSystemGwejson::throwerror("There was an error - no such repeat");
	}

	if ($requestObject->error)
	{
		return "Error";
	}
	// title is actually the coupon code!
	if (isset($requestObject->title) && trim($requestObject->title) !== "")
	{
		$returnData->result = "title is " . $requestObject->title;
	}
	else
	{
		PlgSystemGwejson::throwerror("There was an error - no valid argument");
	}

	$db->setQuery("SELECT * FROM #__jev_rsvp_fields  where field_id=" . (int) $requestObject->fieldid);
	$field = $db->loadObject();

	if (!$field || $field->type != "jevrcoupon")
	{
		PlgSystemGwejson::throwerror("There was an error - not a coupon");
	}

	$db->setQuery("SELECT * FROM #__jev_attendance where id=" . (int) $requestObject->atd_id);
	$rsvpdata = $db->loadObject();

	if (strlen($requestObject->title) < 1)
	{
		$returnData->discount = 0;
		$returnData->surcharge = 0;
		return $returnData;
	}

	$db = Factory::getDbo();

	$fieldoptions = json_decode($field->options);
	$i = 0;

	if (isset($requestObject->atdee_id) && $requestObject->atdee_id > 0){
		$sql = "SELECT * FROM #__jev_attendees where id=" . (int) $requestObject->atdee_id;
		if (!$rsvpdata->allrepeats){
			$sql .= " AND rp_id=" . (int) $requestObject->rp_id;
		}
		$db->setQuery($sql);
		$attendee   = $db->loadObject();
		$created    = strtotime($attendee->created);
	}
	else {
		$created = mktime();
	}

	$validCode = false;

	foreach ($fieldoptions->label as $code)
	{
		if (trim($code) === trim($requestObject->title))
		{
			$validCode = true;
			if (isset($fieldoptions->validfrom)){

				$validfrom  = $fieldoptions->validfrom[$i];
				$validto	= $fieldoptions->validto[$i];

				$validFrom = strlen($validfrom) > 10 ? strtotime($validfrom) : strtotime(($validfrom ? $validfrom : "1970-01-01") . " 00:00:00");
				$validTo = strlen($validto) > 10 ? strtotime($validto) : strtotime(($validto ? $validto:"2199-12-31") . " 23:59:59");

				if ($created>=$validFrom && $created<=$validTo){
					// Coupontype = 0 for fixed and 1 for percentage
					if (isset($fieldoptions->type[$i]) && $fieldoptions->type[$i]==1){
						$discount = 0;
						$surcharge = -$fieldoptions->price[$i];
					}
					else {
						$discount = -$fieldoptions->price[$i];
						$surcharge = 0;
					}
				}
				else if ($created < $validFrom && trim($code) !== ''){
					PlgSystemGwejson::throwerror(Text::_("RSVP_COUPON_NOT_YET_ACTIVE", true));
				}
				else if ($created > $validTo && trim($code) !== ''){
					PlgSystemGwejson::throwerror(Text::_("RSVP_COUPON_HAS_EXPIRED", true));
				}
			}
			else {
				// coupontype = 0 for fixed and 1 for percentage
				if (isset($fieldoptions->type[$i]) && (int) $fieldoptions->type[$i] === 1){
					$discount = 0;
					$surcharge = -$fieldoptions->price[$i];
				}
				else {
					$discount = -$fieldoptions->price[$i];
					$surcharge = 0;
				}

			}
		}

		$i++;
	}

	if(!$validCode) {
		PlgSystemGwejson::throwerror(Text::_("RSVP_COUPON_NOT_VALID", true));
	}

	if (!isset($discount)){
		$returnData->discount = 0;
		$returnData->surcharge = 0;
		return $returnData;
	}

	$fieldparams = json_decode($field->params);

	if (isset($fieldparams->maxuses) && $fieldparams->maxuses > 0)
	{
		$sql = "SELECT * FROM #__jev_rsvp_couponusage  where atd_id=" . (int) $requestObject->atd_id;
		if (!$rsvpdata->allrepeats){
			$sql .= " AND rp_id=" . (int) $requestObject->rp_id;
		}
		$db->setQuery($sql);
		$couponusage = $db->loadObject();

		if ($couponusage){

			$couponparams = json_decode($couponusage->params);
			$fieldname = "field".$field->field_id;
			$couponcode = trim($requestObject->title);

			// Is this a valid coupon code and could we be at the max use limit!
			if (isset($couponparams->$fieldname) && isset($couponparams->$fieldname->$couponcode) &&
				(int) $couponparams->$fieldname->$couponcode +1 > $fieldparams->maxuses )
			{
				$canusecoupon = false;

				// Make sure we are not already using this same coupon
				if ($requestObject->atdee_id > 0){
					$sql = "SELECT * FROM #__jev_attendees where id=" . (int) $requestObject->atdee_id;

					if (!$rsvpdata->allrepeats){
						$sql .= " AND rp_id=" . (int) $requestObject->rp_id;
					}

					$db->setQuery($sql);
					$attendeedata = $db->loadObject();

					if ($attendeedata){
						$attendeeparams = json_decode($attendeedata->params);
						if (isset($attendeeparams->$fieldname) && trim($attendeeparams->$fieldname)==$couponcode){
							$canusecoupon	= true;
						}
					}
				}

				if (!$canusecoupon) {
					PlgSystemGwejson::throwerror(Text::_("RSVP_COUPON_ALREADY_USED", true));
				}
			}
		}
	}

	$returnData->discount = $discount;
	$returnData->surcharge = $surcharge;

	return $returnData;
}
