<?php

/**
 * The main file of the RSVPPRO SMS Reminders plugin
 *
 * @package     RSVPPRO
 * @version     1.0.0  
 *
 * @author      Geraint Edwards
 * @link        http://www.gwesystems.com
 * @copyright   (C) 2009-2021 GWE Systems Ltd. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 */
// No direct access
defined('_JEXEC') or die();

use Joomla\CMS\Language\Text;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\CMS\Factory;
use Joomla\CMS\Filesystem\File;


// Import library dependencies
jimport('joomla.plugin.plugin');

class plgRsvpproSmsreminders extends CMSPlugin
{

	function __construct(& $subject, $config)
	{
		parent::__construct($subject, $config);

		// merge in the new config options from the component
		$params = ComponentHelper::getParams("com_rsvppro");
		$this->params->merge($params);

		Factory::getLanguage()->load('plg_rsvppro_smsreminders', JPATH_ADMINISTRATOR);
		Factory::getLanguage()->load('com_rsvppro', JPATH_SITE);

	}

	function onEditReminders(&$extrareminders)
	{
		$extrareminders[2] = Text::_("RSVP_SMS_SMSONLY");
		$extrareminders[3] = Text::_("RSVP_SMS_SMS_AND_EMAIL");
		return;

	}

	function onSendReminder(&$reminder, &$sendemail, $rsvpdata, $event, $creator, $email, $subject, $message)
	{
		if ($reminder->user_id == 0)
			return;

		jimport('joomla.filesystem.file');
		if (File::exists(JPATH_SITE."/components/com_community/community.php")){
			if (!class_exists('CFactory'))
			{
				require_once( JPATH_ROOT . '/components/com_community/libraries/core.php');
				require_once (COMMUNITY_COM_PATH . '/libraries/template.php');
				require_once( COMMUNITY_COM_PATH . '/libraries/apps.php' );
			}

			$cuser = CFactory::getUser(intval($reminder->user_id));

			$mobile = $cuser->getInfo($this->params->get("smsmobilefield"));
			if (!$mobile)
			{
				return;
			}

			if($this->params->get("limitsenabled")) {

				$maxsms = $cuser->getInfo($this->params->get("smsnumberfield"));
				if (!$maxsms)
				{
					return;
				}

				$usedsms = intval($cuser->getInfo($this->params->get("smsusedfield")));
				if ($maxsms <= $usedsms)
				{
					return;
				}
			}

		}
		else {
			$juser = JEVHelper::getUser($reminder->user_id);
			
			$mobile = $juser->getInfo($this->params->get("smsmobilefield"));
			if (!$mobile)
			{
				return;
			}

			if($this->params->get("limitsenabled")) {
				$maxsms = $juser->getInfo($this->params->get("smsnumberfield"));
				if (!$maxsms)
				{
					return;
				}

				$usedsms = intval($juser->getInfo($this->params->get("smsusedfield")));
				if ($maxsms <= $usedsms)
				{
					return;
				}
			}
		}		

		// get and parse message
		$message = $this->params->get("smstemplate");
		$message = $this->parseMessage($message, $rsvpdata, $event, $reminder, $creator, $mobile);
		$message = substr($message, 0, 159);

		$email = $this->params->get("smsemail");
		if (strpos($email, "{TEL}")!==false){
			$email = str_replace("{TEL}", $mobile, $email);
		}

		try
		{
			$success = $this->sendMail($creator->email, $creator->name, $email, $subject, $message, 1, null, null);
		}
		catch (Exception $e)
		{

		}

		if ($this->params->get("limitsenabled") && isset($cuser)){
			$cuser->setInfo($this->params->get("smsusedfield"), $usedsms + 1);
		}
		else {
			
		}

		if (intval($rsvpdata->allowreminders) == 2)
		{
			$sendemail = false;
		}
		return true;

	}

	function parseMessage($message, $rsvpdata, $row, $iuser, $creator, $mobile)
	{
		// Stop email cloaking in email messages
		$message ="{emailcloak=off}".$message;

		$message = str_replace("{TEL}", $mobile, $message);
		$message = str_replace("{USERNAME}", $iuser->username, $message);
		$message = str_replace("{NAME}", $iuser->name, $message);
		$message = str_replace("{EVENT}", $row->title(), $message);
		$message = str_replace("{CREATOR}", $creator->name, $message);

		$event_up = new JEventDate($row->publish_up());
		$row->start_date = JEventsHTML::getDateFormat($event_up->year, $event_up->month, $event_up->day, 0);
		$row->start_time = JEVHelper::getTime($row->getUnixStartTime());

		$event_down = new JEventDate($row->publish_down());
		$row->stop_date = JEventsHTML::getDateFormat($event_down->year, $event_down->month, $event_down->day, 0);
		$row->stop_time = JEVHelper::getTime($row->getUnixEndTime());
		$row->stop_time_midnightFix = $row->stop_time;
		$row->stop_date_midnightFix = $row->stop_date;
		if ($event_down->second == 59)
		{
			$row->stop_time_midnightFix = JEVHelper::getTime($row->getUnixEndTime() + 1);
			$row->stop_date_midnightFix = JEventsHTML::getDateFormat($event_down->year, $event_down->month, $event_down->day + 1, 0);
		}

		$message = strip_tags(str_replace("{REPEATSUMMARY}", $row->repeatSummary(), $message));

		$regex = "#{DATE}(.*?){/DATE}#s";
		preg_match($regex, $message, $matches);
		if (count($matches) == 2)
		{
			$date = new JevDate($row->getUnixStartDate());
			$message = preg_replace($regex, $date->toFormat($matches[1]), $message);
		}

		$regex = "#{LINK}(.*?){/LINK}#s";
		preg_match($regex, $message, $matches);
		if (count($matches) == 2)
		{
			$Itemid = Factory::getApplication()->input->getInt("Itemid");
			list($year, $month, $day) = JEVHelper::getYMD();
			$link = $row->viewDetailLink($year, $month, $day, true, $Itemid);

			if (strpos($link, "/") !== 0)
			{
				$link = "/" . $link;
			}
			if (strpos($link, "plugins/jevents/rsvppro") !== false)
			{
				$link = str_replace("plugins/jevents/rsvppro/", "", $link);
			}
			if (strpos($link, "plugins/jevents/jevrsvppro/rsvppro") !== false)
			{
				$link = str_replace("plugins/jevents/jevrsvppro/rsvppro/", "", $link);
			}
			if (strpos($link, "plugins/jevents") !== false)
			{
				$link = str_replace("plugins/jevents/", "", $link);
			}

			$uri =  JURI::getInstance(JURI::base());
			$root = $uri->toString(array('scheme', 'host', 'port'));

			$link = $root . $link;
			if (strpos($link, "plugins/jevents/rsvppro") >= 0)
			{
				$link = str_replace("plugins/jevents/rsvppro/", "", $link);
			}
			if (strpos($link, "plugins/jevents/jevrsvppro/rsvppro") >= 0)
			{
				$link = str_replace("plugins/jevents/jevrsvppro/rsvppro/", "", $link);
			}
			if (strpos($link, "plugins/jevents") >= 0)
			{
				$link = str_replace("plugins/jevents/", "", $link);
			}
			if ($row->access() > version_compare(JVERSION, "1.6.0", 'ge') ? 1 : 0)
			{
				if (strpos($link, "?") > 0)
				{
					$link .= "&login=1";
				}
				else
				{
					$link .= "?login=1";
				}
			}

			$message = preg_replace($regex, "<a href='$link'>" . $matches[1] . "</a>", $message);
		}

		// convert relative to absolute URLs
		$message = preg_replace('#(href|src|action|background)[ ]*=[ ]*\"(?!(https?://|\#|mailto:|/))(?:\.\./|\./)?#', '$1="' . JURI::root(), $message);
		$message = preg_replace('#(href|src|action|background)[ ]*=[ ]*\"(?!(https?://|\#|mailto:))/#', '$1="' . JURI::root(), $message);

		$message = preg_replace("#(href|src|action|background)[ ]*=[ ]*\'(?!(https?://|\#|mailto:|/))(?:\.\./|\./)?#", "$1='" . JURI::root(), $message);
		$message = preg_replace("#(href|src|action|background)[ ]*=[ ]*\'(?!(https?://|\#|mailto:))/#", "$1='" . JURI::root(), $message);

		// Stop email cloaking in email messages
		$message =str_replace(array("{emailcloak=off}", "{* emailcloak=off}"), "", $message);

		return $message;

	}

	function sendMail($from, $fromname, $recipient, $subject, $body, $mode = 0, $cc = null, $bcc = null, $attachment = null, $replyto = null, $replytoname = null)
	{
		$params = ComponentHelper::getParams("com_rsvppro");
		$from = $params->get("overridesenderemail", $from);
		$fromname = $params->get("overridesendername", $fromname);
		$mail = Factory::getMailer();

		// Set AltBody so we get plain text version in the message too!
		if (JevJoomlaVersion::isCompatible("3.4")) {
			$mail->AltBody = $mail->normalizeBreaks($mail->html2text($body, true));
		}
		else {
			$textMsg = trim(strip_tags(preg_replace('/<(head|title|style|script)[^>]*>.*?<\/\\1>/s', '', $body)));
			if (!empty($textMsg)) {
				$mail->AltBody = html_entity_decode($textMsg, ENT_QUOTES, $mail->CharSet);
			}
		}
		try {
			return $mail->sendMail($from, $fromname, $recipient, $subject, $body, $mode, $cc, $bcc, $attachment, $replyto, $replytoname);
		}
		catch (Exception $e)
		{
			return false;
		}
	}

}
